/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains Number Field Classes
 * @author Erkan Cengiz
 * @version $Id: Number.js 216 2008-09-13 10:35:48Z erkan $
 */


/**
 * @class
 * Represents a NumberText. It wraps any HTML DOM element that can contains a text node as
 * a child node. The purpose of this class is to create a text node under the wrapped DOM element
 * and format its value as a number.
 *
 * @extends TextNode
 * @param node the DOM Element node that is wrapped by this object
 */
function NumberText(node) { 

  /**
   * NumberText Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b><a name="value">value</a></b></td><td>The value of the NumberText</td></tr>
   * <tr><td><b>allowSign</b></td><td> Allow Sign Flag. <code>true</code> means that sign ("-") character can be entered into the input field.</td></tr>
   * <tr><td><b>precisionDigits</b></td><td> Precision Digits. The number of the digits allowed to enter after the decimal seperator.</td></tr>
   * <tr><td><b>round</b></td><td> Specify true to round the value </td></tr>
   * <tr><td><b>padZeros</b></td><td> Specify true to pad zeros at the right side when precision digits are less than the specified.</td></tr>
   * <tr><td><b>currencyFormatting</b></td><td> If the number value includes a currency symbol (e.g. '120.23 $'), this property 
   * can be used to format it. Examples: 
   * <code><pre>
   * Value      Formatting   Result
   * ---------- -----------  --------
   * 123.23 $   $9           $120.23
   * 123.23 $   $9           $ 120.23
   * 123.23 TL  9 $          120.23 YTL
   *    
   * </pre></code>
   * </td></tr>
   * </table>
   *
   */
  this.props = null;

  try {
    this.base = TextBox;
    this.base(node);
    this.needsValueCloning = true;
  } catch (e) {
    /** @private */
    this.base = TextNode;
    this.base(node);
  }

  /** @type boolean */
  this.isNumberText = true;

  var pageInfo = OMgr.getPageInfo(this.node);

  /** Grouping separator */
  this.GS = pageInfo.GS;

  /** Decimal Separator */
  this.DS = pageInfo.DS;

  /**
   * Initializes this NumberText
   */
  this.init = function() {
    this.setValue(this.props.value);
  };

  this.setValue = function(value) {
    if (this.props.currencyFormatting) {
      this.setCurrencyValue(value);
      return;
    }
    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parseNum(value);
    this._setValue(cv.format(this.boolProp("padZeros")));
  };

  this.setCurrencyValue = function(value) {
    var val = value.before(" ",value);
    var sym = value.after(" ","?");
    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parseNum(val);
    value = cv.format(this.boolProp("padZeros"));
    value = this.props.currencyFormatting.replace(new RegExp("9","g"),value).replace(new RegExp("[$]","g"),sym);
    this._setValue(value); 
  };


  /** @private */
  this.ajaxRefreshHandler = function(rec,nodeName,attr) {

    if (!nodeName) nodeName = this.name;

    if (!attr) attr = "value";
    if (attr == "value") this.setValue("");
    else if (attr == "href") this.setHref("");

    if (rec) {
      if (!rec.owner.hasField(nodeName)) {
        throw new Exception("RESPONSE_FIELD_MISSING",nodeName);
      }

      if (attr == "value") {
        this.setValue(rec.get(nodeName));
      } else if (attr == "href") this.setHref( rec.get(nodeName) );
    }

  };


}



/**
 * @class
 * Represents a NumberBox Process Field that wraps a <code>TEXT INPUT</code> element and a
 * <code>HIDDEN INPUT</code> element to provide a controlled input field for entering Number data.
 * <p>While the <code>TEXT INPUT</code> field contains the formatted value, the <code>HIDDEN INPUT</code>
 * contains the pure number value to send to the server. Each time the user types in a new character
 * this class controls immediatelly wheter it is a allowed character and reformats the value in the
 * input field while transfering the pure number equivalance of it to the <code>HIDDEN INPUT</code> ready to
 * send to the server.
 * @extends ProcessField
 * @param node the DOM Element node that is wrapped by this object
 */
function NumberBox(node) {

  /**
   * NumberBox Properties Object.
   *
   * <br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>allowSign</b></td><td> Allow Sign Flag. <code>true</code> means that sign ("-") character can be entered into the input field.</td></tr>
   * <tr><td><b>precisionDigits</b></td><td> Precision Digits. The number of the digits allowed to enter after the decimal seperator.</td></tr>
   * <tr><td><b>round</b></td><td> Specify true to round the displayed value </td></tr>
   * <tr><td><b>padZeros</b></td><td> Specify true to pad zeros at the right side when precision digits are less than the specified.</td></tr>
   * <tr><td><b>nonBlank</b></td><td>  See <a href="ProcessField.html#nonBlank">ProcessField.props.nonBlank</a> </td></tr>
   * <tr><td><b>validValues</b></td><td> See <a href="ProcessField.html#validValues">ProcessField.props.validValues</a></td></tr>
   * <tr><td><b>valHandler</b></td><td> See <a href="ProcessField.html#valHandler">ProcessField.valHandler()</a></td></tr>
   * <tr><td><b>valErrorHandler</b></td><td> See <a href="ProcessField.html#valErrorHandler">ProcessField.props.valErrorHandler</a></td></tr>
   * </table>
   *
   */
  this.props = null;

  try {
    this.base = ProcessField;
    this.base(node);
  } catch (e) {
    this.base = MoreMotionObject;
    this.base(node);
  }
  /** @type boolean */
  this.isNumberBox = true;


  /**
   * The HTMLElement object of the <code>TEXT INPUT</code> Element that is visible and editable.
   * It contains the formatted number.
   * @type HTMLElement
   */
  this.elm = null; //-<

  /**
   * The HTMLElement object of the <code>HIDDEN INPUT</code> Element that contains the pure
   * number value to send to the server.
   * @type HTMLElement
   */
  this.selm = null; //-<

  var pageInfo = OMgr.getPageInfo(this.node);

  /** Grouping separator */
  this.GS = pageInfo.GS;

  /** Decimal Separator */
  this.DS = pageInfo.DS;

  /**
   * Initializes this NumberBox
   */
  this.init = function() {
    this.reformat();
  };

  /** @private */
  this.wrapElements = function() {
    this.elm = this.node;
    var parentNode = this.node.tagName == "INPUT" ? this.node.parentNode : this.node;
    var elms = parentNode.getElementsByTagName("input");
    if (elms && elms.length) {
      for (var i = 0; i < elms.length; i++) {
        var elm = elms[i];
        if ((elm.name.startsWith("pf_") && elm.name.endsWith(this.name)) || elm.name == this.name ) {
          this.selm = elm;
        } else if (elm.name == "__t") {
          this.elm = elm;
        }
      }
    }
  };

  this.wrapElements();

  /**
   * Sets the given pure number value to the <code>HIDDEN INPUT</code> element and
   * reformats the <code>TEXT INPUT</code> field
   */
  this.reformat = function(value) {
    this.selm.value = value || this.getIValue();
    this.format();
  };


  /**
   * Clears values of all the wrapped HTML Elements
   */
  this.clear = function() {
    this.elm.value = "";
    this.selm.value = "";
    this.prevValue = "";
  };

  /** @private */
  this.formatInput = function(blur) {

    if (this.prevValue && this.prevValue == this.elm.value) return true;

    if (this.elm.value == "") {
      this.selm.value = "";
      this.prevValue = "";
      this.setModified();
      return true;
    }

    var ilen = this.elm.value.length;

    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parse(this.elm.value);

    // var pos = 1;
    // var posSupport = this.elm.createTextRange || this.elm.selectionStart;

    // if (posSupport) pos = NumMgr.getCaretPos(this.elm);
    var pos = NumMgr.getCaretPos(this.elm);

    this.selm.value = cv.getNumber();
    // if (posSupport)
    {
       var fv = cv.format(blur && this.boolProp("padZeros"));
       if (this.numberProp("precisionDigits") > 0 &&
           this.elm.value.indexOf(this.DS) > -1 && fv.indexOf(this.DS) == -1) {
           fv += this.DS;
       }
       this.elm.value = fv;
    }

    if (this.elm.value.length > ilen) {
      pos++;
    } else if (this.elm.value.length < ilen) {
      pos = pos - 1;
    }
    // if (posSupport && !blur) {
    if (!blur) {
      NumMgr.setCaretPos(this.elm,pos);
    }

    this.setModified();

    this.prevValue = this.elm.value;

    return true;

  };

  /** @ignore */
  this.format = function() {

    if (this.selm.value == "") {
      this.elm.value = "";
      this.prevValue = this.elm.value;
      return true;
    }

    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parseNum(this.selm.value);

    this.elm.value = cv.format(this.boolProp("padZeros"));
    this.prevValue = this.elm.value;

    return true;

  };


  /**
   * Sets the value of this field and reformats the wrapped field(s).
   * @param value The value to set to the field.
   */
  this.setValue = function(value) {
    this.reformat(value);
  };

  this.getValue = function() {
    return this.selm.value;
  };

  /** @private */
  this.validate = function() {
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable(this.selm,this.elm) == false) return false;
    if (this.props.validValues && this.checkHasValidValue(this.selm,this.elm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
    return true;
  };

}

/**
 * @class
 * Represents a NumberDialog Process Field which wraps two <code>TEXT INPUT</code> elements and a
 * <code>HIDDEN INPUT</code> element to provide a controlled input field for entering Number data.
 * This class splits the given initial value into the integer and the remainder parts and places into the
 * <code>TEXT INPUT</code> fields.
 * <p>While the <code>TEXT INPUT</code> fields contains the formatted value, the <code>HIDDEN INPUT</code>
 * contains the pure number value to send to the server. Each time the user types in a new character
 * this class controls immediatelly wheter it is a allowed character and reformats the value in the
 * input fields while transfering the pure number equivalance of it to the <code>HIDDEN INPUT</code> ready to
 * send to the server.
 * @extends ProcessField
 * @param node the DOM Element node that is wrapped by this object
 */
function NumberDialog(node) {

  /**
   * NumberDialog Properties Object.
   *
   * <br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>allowSign</b></td><td> Allow Sign Flag. <code>true</code> means that sign ("-") character can be entered into the input field.</td></tr>
   * <tr><td><b>precisionDigits</b></td><td> Precision Digits. The number of the digits allowed to enter after the decimal seperator.</td></tr>
   * <tr><td><b>round</b></td><td> Specify true to round the displayed value </td></tr>
   * <tr><td><b>padZeros</b></td><td> Specify true to pad zeros at the right side when precision digits are less than the specified.</td></tr>
   * <tr><td><b>nonBlank</b></td><td>  See <a href="ProcessField.html#nonBlank">ProcessField.props.nonBlank</a> </td></tr>
   * <tr><td><b>validValues</b></td><td> See <a href="ProcessField.html#validValues">ProcessField.props.validValues</a></td></tr>
   * <tr><td><b>valHandler</b></td><td> See <a href="ProcessField.html#valHandler">ProcessField.valHandler()</a></td></tr>
   * <tr><td><b>valErrorHandler</b></td><td> See <a href="ProcessField.html#valErrorHandler">ProcessField.props.valErrorHandler</a></td></tr>
   * </table>
   *
   */
  this.props = null;

  try {
    this.base = ProcessField;
    this.base(node);
  } catch (e) {
    this.base = MoreMotionObject;
    this.base(node);
  }
  /** @type boolean */
  this.isNumberDialog = true;


  /**
   * The HTMLElement object of the <code>TEXT INPUT</code> Element on the left
   * @type HTMLElement
   */
  this.elm = null; //-<

  /**
   * The HTMLElement object of the <code>TEXT INPUT</code> Element on the right
   * @type HTMLElement
   */
  this.relm = null; //-<

  /**
   * The object of the HTMLElement that displays the decimal point. The element must have "DecimalPoint"
   * type as follows. <br>
   * <CODE>&lt;span mo:type="DecimalPoint"&gt?&lt;/span&gt;</CODE>
   * <p>&nbsp;
   * @type HTMLElement
   */
  this.dpelm = null; //-<

  /**
   * The HTMLElement object of the <code>HIDDEN INPUT</code> Element that contains the pure
   * number value to send to the server.
   * @type HTMLElement
   */
  this.selm = null; //-<

  var pageInfo = OMgr.getPageInfo(this.node);

  /** Grouping separator */
  this.GS = pageInfo.GS;

  /** Decimal Separator */
  this.DS = pageInfo.DS;

  this.init = function() {
    this.reformat();
  };

  /** @private */
  this.wrapElements = function() {

    var elms = this.node.getElementsByTagName("input");
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.name == "__LeftPart") this.elm = elm;
      else if (elm.name == "__RightPart") this.relm = elm;
      else if (elm.name.startsWith("pf_") || elm.name == this.name) this.selm = elm;
    }
    this.dpelm = OMgr.findChildNode(this.node,null,"mo:type","DecimalPoint");

  };

  this.wrapElements();
  if (this.dpelm) {
    var t = new TextNode(this.dpelm);
    t.setValue(this.DS);
  }

  this.reformat = function(value) {

    this.selm.value = value || this.getIValue();

    if (this.selm.value == "") {
      this.elm.value = "";
      this.relm.value = "";
      return;
    }

    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parseNum(this.selm.value);

    this.elm.value = cv.formatLeft();
    this.relm.value = cv.formatRight(this.boolProp("padZeros"));

  };

  /** @private */
  this.formatLeftPart = function(blur) {

    if (this.prevLValue && this.prevLValue == this.elm.value) return true;

    var ilen = this.elm.value.length;

    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS, this.boolProp("round"));
    cv.parseLAR(this.elm.value, this.relm.value);

    // var pos = 1;
    // var posSupport = this.elm.createTextRange || this.elm.selectionStart;
    // if (posSupport) pos = NumMgr.getCaretPos(this.elm);
    var pos = NumMgr.getCaretPos(this.elm);

    this.selm.value = cv.getNumber();
    // if (posSupport) this.elm.value = cv.formatLeft();
    this.elm.value = cv.formatLeft();

    if (this.elm.value.length > ilen) {
      pos++;
    } else if (this.elm.value.length < ilen) {
      pos = pos - 1;
    }
    // if (posSupport && !blur) NumMgr.setCaretPos(this.elm,pos);
    if (!blur) NumMgr.setCaretPos(this.elm,pos);

    this.prevLValue = this.elm.value;

    this.setModified();

    return true;
  };

  /** @private */
  this.formatRightPart = function(blur) {

    if (this.prevRValue && this.prevRValue == this.relm.value) return true;

    var ilen = this.relm.value.length;

    var cv = new NumberValue(this.boolProp("allowSign"), this.props.precisionDigits, this.GS, this.DS);
    cv.parseLAR(this.elm.value, this.relm.value);

    // var pos = 1;
    // var posSupport = this.relm.createTextRange || this.relm.selectionStart;
    // if (posSupport) pos = NumMgr.getCaretPos(this.relm);
    var pos = NumMgr.getCaretPos(this.relm);

    this.selm.value = cv.getNumber();

    // if (posSupport) this.relm.value = cv.formatRight(false);
    this.relm.value = cv.formatRight(blur && this.boolProp("padZeros"));

    if (this.relm.value.length > ilen) {
      pos++;
    } else if (this.relm.value.length < ilen) {
      pos = pos - 1;
    }
    // if (posSupport && !blur) NumMgr.setCaretPos(this.relm,pos);
    if (!blur) NumMgr.setCaretPos(this.relm,pos);

    this.setModified();

    this.prevRValue = this.relm.value;
    return true;
  };

  /**
   * Sets the value of this field and reformats the wrapped field(s).
   * @param value The value to set to the field.
   */
   this.setValue = function(value) {
    this.reformat(value);
  };

  this.getValue = function() {
    return this.selm.value;
  };

  /** @private */
  this.clear = function() {
    this.elm.value = "";
    this.relm.value = "";
    this.selm.value = "";
    this.prevLValue = "";
    this.prevRValue = "";
  };

  /** Validates this NumberDialog field and returns <code>true</code> if validation is successful,
   * <code>false</code> otherwise.
   * @type boolean
   */
  this.validate = function() {
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable() == false) return false;
    if (this.props.validValues && this.checkHasValidValue(this.selm,this.elm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
    return true;
  };

}

/** @ignore */
function NumberValue(allowSign, precDigs, GS, DS, round) {

  this.allowSign = allowSign;
  this.precDigs = precDigs * 1;
  this.GS = GS;
  this.DS = DS;
  this.round = round;

  this.sign = "";
  this.lpart = "";
  this.rpart = "";

  this.parseNum = function(value) {

    value = value + "";
    if (this.round) {
      value = this.round(value * 1, this.precDigs) + "";
    }

    if (value.startsWith("-")) {
      if (this.allowSign) this.sign = value.charAt(0);
      value = value.after("-");
    }

    this.lpart = value.before(".",value);
    this.rpart = value.after(".","");
    if (this.rpart && this.rpart.length > this.precDigs) {
      this.rpart = this.rpart.substring(0,this.precDigs);
    }

  };

  this.parse = function(value) {

    this.sign = value.charAt(0);
    if (this.sign != "-" || !this.allowSign) this.sign = "";

    this.lpart = value.before(this.DS,value);
    this.rpart = value.after(this.DS,"");

    var re = /\D/g;
    this.lpart = this.lpart.replace(re,"");
    if (this.rpart) this.rpart = this.rpart.replace(re,"");
    if (this.rpart && this.rpart.length > this.precDigs) {
      this.rpart = this.rpart.substring(0,this.precDigs);
    }

  };

  this.parseLAR = function(lpart, rpart) {

    this.lpart = lpart;
    this.rpart = rpart;

    this.sign = lpart.charAt(0);
    if (this.sign != "-" || !this.allowSign) this.sign = "";

    var re = /\D/g;
    this.lpart = this.lpart.replace(re,"");
    this.rpart = this.rpart.replace(re,"");
    if (this.rpart && this.rpart.length > this.precDigs) {
      this.rpart = this.rpart.substring(0,this.precDigs);
    }

  };

  this.getNumber = function() {
    return this.sign + this.lpart + (this.rpart.length == 0 ? "" : ("." + this.rpart) );
  };

  this._getGrouped = function(value) {
    var cnt = 0;
    var out = "";
    for (var i = value.length - 1; i > -1; i--) {
      if (++cnt == 4) {
        cnt = 1;
        out = this.GS + out;
      }
      out = value.charAt(i) + out;
    }
    return out;
  };

  this.format = function(padzeros) {
    var s = this.sign + this._getGrouped(this.lpart);
    var r = this.formatRight(padzeros);
    if (r) s += this.DS + r;
    return s;
  };

  this.formatLeft = function() {
    return this.sign + this._getGrouped(this.lpart);
  };

  this.formatRight = function(padzeros) {
   if (this.precDigs <= 0) return "";
    var s = this.rpart;
    if (padzeros && s.length < this.precDigs) {
     s += "0000000000000000000000000000000";
      s = s.substring(0,this.precDigs);
    }
    return s;
  };

  /** @private */
  this.round = function(val, prec) {
    if (prec <= 0) return val;
    var x = Math.pow(10,prec);
    return Math.round(val * x) / x;
  };

}

/**
 * @class
 * Manager Class that provides static methods to manage {@link NumberBox} and {@link NumberDialog} objects. A static object of this class
 * is avaialable with name <CODE>NumMgr</CODE>.
 * <PRE>var NumMgr = new NumberManager();</PRE>
 *
 */
function NumberManager() {

  /**
   * Formats the users input on the given HTML <code>TEXT INPUT</code> element.
   * This method should be called by the onkeyup event.
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @param blur true when called by the blur event
   * @returns <code>true</code> if formating is successful
   * @type boolean
   */
  this.formatInput = function(node,blur) {
    var f = OMgr.getObject(node) || OMgr.getEnclosingObject(node);
    if (f) return f.formatInput(blur);
    return null;
  };

  /**
   * Formats the users input on the left part of the {@link NumberDialog}
   * This method should be called by the onkeyup event of the
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @param blur true when called by the blur event
   * @returns <code>true</code> if formating is successful
   * @type boolean
   */
  this.formatLeftPart = function(node,blur) {
    var f = OMgr.getEnclosingObject(node);
    if (f) return f.formatLeftPart(blur);
    return null;
  };

  /**
   * Formats the users input on the right part of the {@link NumberDialog}
   * This method should be called by the onkeyup event of the
   * @param node The DOM Node of the <code>TEXT INPUT</code> element
   * @param blur true when called by the blur event
   * @returns <code>true</code> if formating is successful
   * @type boolean
   */
  this.formatRightPart = function(node,blur) {
    var f = OMgr.getEnclosingObject(node);
    if (f) return f.formatRightPart(blur);
    return null;
  };

  /** @private */
  this.getCaretPos = function(elm) {

    /* for IE */
    if (document.selection) {
      try {
   var Sel = document.selection.createRange();
        Sel.moveStart ('character', -elm.value.length);
        return Sel.text.length;
      } catch (e) {}
    }

    else if (elm.selectionStart || elm.selectionStart == '0') {
      return elm.selectionStart;
    }

    return 1;

  };

  /** @private */
  this.setCaretPos = function(elm, pos) {

    if (WinMgr.isGecko) {
      elm.selectionStart = pos;
      elm.selectionEnd = pos;

    } else if (WinMgr.isOpera) {
      elm.focus();
      elm.setSelectionRange(pos,pos);

    } else if (elm.setSelectionRange) {
      elm.focus();
      elm.setSelectionRange(pos,pos);
    }

    else if (elm.createTextRange) {
      var docrange = document.selection.createRange();
      var inprange = elm.createTextRange();
      inprange.move('character',pos);
      inprange.select();
    }

  };

}

/**
 * Global Singleton NumberManager object
 * @type NumberManager
 */
var NumMgr = new NumberManager();
