/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */


/**
 * @fileoverview
 * Contains Data Source Navigator Classes
 * @author Erkan Cengiz
 * @version $Id: Navigator.js 189 2008-08-04 12:47:11Z mustafa $
 */

/**
 * @class
 * Represents a Data Source Navigator.
 * @extends MoreMotionObject
 *
 * @param node the DOM Element node that is marked as a Navigator by means of "mo:type" attribute with
 * a value of "Navigator".
 *
 *
 */
function Navigator(node) {

  /**
   * Navigator Properties Object.
   *
   *<br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>dataSource</b></td><td> The name of the data source to navigate over</td></tr>
   * <tr><td valign="top"><b>submitType</b></td><td> The Submit Type of the navigation request. The valid values are:
   * <ol>
   *   <li>Http Get</li>
   *   <li>Http Post</li>
   *   <li>Ajax</li>
   * </ol></td></tr>
   * <tr><td valign="top"><b>pagingInfo</b></td><td> This property contains paging info for the specified data source. It contains
   * the following sub-properties that are separated with semicolumn chars.
   * <ul>
   *   <li><code><b>noi</b></code> - Number of items</li>
   *   <li><code><b>cp</b></code> - Current page number</li>
   *   <li><code><b>ipp</b></code> - Items per page</li>
   *   <li><code><b>ppb</b></code> - Pages per block</li>
   *   <li><code><b>lp</b></code> - Last page flag</li>
   *   <li><code><b>mp</b></code> - More pages flag</li>
   *   <li><code><b>fp</b></code> - The number of the first page in the block</li>
   *   <li><code><b>tp</b></code> - The number of the last page in the block</li>
   * </ul>
   * Example value: <code>noi:340;cp:5;ipp:10;ppb:5;lp:false;mp:true;fp:51;tp:60</code>
   *
   * </td></tr>
   * <tr><td valign="top"><b>focusTo</b></td><td> The code of the input element which should gain focus
   * after the navigation. The codes are:
   * <ul>
   *   <li><code><b>cp</b></code> - Current page element</li>
   *   <li><code><b>ipp</b></code> - Items per page element</li>
   *   <li><code><b>ppb</b></code> - Pages per block element</li>
   * </ul>
   * <tr><td valign="top"><b><a name="callbackFunc">callbackFunc</a></b></td><td> The name of the Javascript function to be called after the Ajax navigation</td></tr>
   * <tr><td valign="top"><b>errorHandler</b></td><td> The name of the Javascript function to be called to handle the errors during the Ajax navigation</td></tr>
   * <tr><td valign="top"><b>commandName</b></td><td>The name of the Ajax {@link ProcessCommand} to use for the Ajax navigation</td></tr>
   *
   * <tr><td valign="top"><td><b>requiredBlocks</b></td><td> The names of the blocks (separated with commas) to be
   * validated before submitting the form with this command  </td></tr>
   *
   * <tr><td valign="top"><b>classPrefix</b></td><td>
   * The prefix of the CSS Class. Please note that a set of CSS Styles should be supplied in a CSS
   * file where there are styles that have certain suffixes and the same prefixes.
   *
   * <br/><br/><u>The CSS style classes that must be provided:</u><br/>
   *
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>currpage_normal</b> {}</code> - The style of the currentPage element
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>currpage_onlastpage</b> {}</code> - The style of the currentPage element when navigator is on the last page
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>pageindex_normal</b> {}</code> - The style of a page index element
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>pageindex_selected</b> {}</code> - The style of the page index element that corresponsd to the current page
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>itemsperpage</b> {}</code> - The style of the itemsPerPage element
   * <br/>&nbsp;&nbsp;<code><i>classPrefix</i>_<b>pagesperblock</b> {}</code> - The style of the pagesPerBlock element
   * </td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isNavigator = true;

  /**
   * Handle to the "currentPage" element whose "mo:type" is "NavigatorCurrentPage".
   * @type HTMLElement
   */
  this.cpelm = null;

  /**
   * Handle to the "itemsParPage" element whose "mo:type" is "NavigatorItemsPerPage".
   * @type HTMLElement
   */
  this.ippelm = null;

  /**
   * Handle to the "pagesPerBlock" element whose "mo:type" is "NavigatorPagesPerBlock".
   * @type HTMLElement
   */
  this.ppbelm = null;

  /**
   * Handle to the "pageIndex" element whose "mo:type" is "NavigatorPageIndex".
   * @type HTMLElement
   */
  this.pielm = null;

  /** @private */
  this.params = new Array();


  /** @private */
  this.wrapElements = function() {
    this.cpelm = OMgr.findChildNode(this.node,null,"mo:type","NavigatorCurrentPage");
    this.ippelm = OMgr.findChildNode(this.node,null,"mo:type","NavigatorItemsPerPage");
    this.ppbelm = OMgr.findChildNode(this.node,null,"mo:type","NavigatorPagesPerBlock");
    this.pielm = OMgr.findChildNode(this.node,null,"mo:type","NavigatorPageIndex");
  };

  this.wrapElements();

  this.init = function() {

    if (this.props.pagingInfo) {
      var settings = this.props.pagingInfo.split(";");
      for (var i = 0; i < settings.length; i++) {
        var s = settings[i].trim();
        var n = s.before(":","").trim();
        var v = s.after(":","").trim();
        if (n && v) {
          if (n == "cp") this.props.currentPage = v * 1;
          else if (n == "ipp") this.props.itemsPerPage = v * 1;
          else if (n == "ppb") this.props.pagesPerBlock = v * 1;
          else if (n == "noi") this.props.numberOfItems = v * 1;
          else if (n == "lp") this.props.lastPage = v == "true";
          else if (n == "mp") this.props.morePages = v == "true";
          else if (n == "fp") this.props.fromPage = v * 1;
          else if (n == "tp") this.props.toPage = v * 1;
        }
      }
     this.saveProps();
    }

    if (this.cpelm) {
      this.cpelm.value = this.props.currentPage;

      if (this.boolProp("lastPage")) {
        OMgr.setClass(this.cpelm,this.props.classPrefix,"currpage_onlastpage");
      } else {
        OMgr.setClass(this.cpelm,this.props.classPrefix,"currpage_normal");
      }

      if (this.numberProp("pagesPerBlock") > 0 || this.boolProp("lastPage")) {
        this.cpelm.title = OMgr.resource("NAVIGATOR_NUMBER_OF_ITEMS",this.props.numberOfItems);
      }

      if (!WinMgr.isIE) {
        this.cpelm.style.marginTop = "2px";
      }
    }

    if (this.props.arrangeMargins != false) {
      var imgs = this.node.getElementsByTagName("img");
      for (var i = 0; i < imgs.length; i++) {
        var img = imgs[i];
        img.style.marginTop = '2px';
      }
    }

    this.preparePageIndex();

    if (this.ippelm) {
      this.ippelm.value = this.props.itemsPerPage;
      OMgr.setClass(this.ippelm,this.props.classPrefix,"itemsperpage");
    }

    if (this.ppbelm) {
      this.ppbelm.value = this.props.pagesPerBlock;
      OMgr.setClass(this.ppbelm,this.props.classPrefix,"pagesperblock");
    }

    if (this.props.focusTo) {
      var elm = null;
      if (this.props.focusTo == "cp") elm = this.cpelm;
      else if (this.props.focusTo == "ipp") elm = this.ippelm;
      else if (this.props.focusTo == "ppb") elm = this.ppbelm;
      if (elm) {
        elm.focus();
        elm.select();
      }
      this.props.focus = null;
    }

  };

  /** @private */
  this.preparePageIndex = function() {
    var pi = this.pielm;
    if (pi) {
      pi.innerHTML = "";
      var cp = this.numberProp("currentPage");
      var fp = this.numberProp("fromPage");
      var tp = this.numberProp("toPage");
      var ipp = this.numberProp("itemsPerPage");
      for (var i = fp; i <= tp; i++) {
        var span = document.createElement("span");
        pi.appendChild(span);
        span.innerHTML = i;
        if (cp == i) span.className = this.props.classPrefix + "_pageindex_selected";
        else span.className = this.props.classPrefix + "_pageindex_normal";

        if (!WinMgr.isIE && this.boolProp("arrangeMargins")) {
          span.style.margin = "3px 1px 0px 1px";
        }
        eval("span.onclick = function() {NavMgr.navigateTo(this,'" + i + "')}");
      }
    }
  };

  /** @private */
  this.keyUp = function(e, elm) {

    var keyCode = (e.keyCode) ? e.keyCode : e.keyChar;
    this.keyCode = keyCode;

    var type = OMgr.getAttribute(elm,"mo:type");
    if (type == "NavigatorCurrentPage") {

      if (keyCode == 39 || keyCode == 40) {
        this.navigateTo("nextPage","cp");
        return false;
      }
      else if (keyCode == 37 || keyCode == 38) {
        this.navigateTo("previousPage","cp");
        return false;
      }
      else if (keyCode == 13) {
        this.navigateTo(elm.value,"cp");
        return false;
      }

    } else if (type == "NavigatorItemsPerPage") {
      if (keyCode == 13) {
        this.itemsPerPage = elm.value;
        this.navigateTo(null,"ipp");
        return false;
      }

    } else if (type == "NavigatorPagesPerBlock") {
      if (keyCode == 13) {
        this.pagesPerBlock = elm.value;
        this.navigateTo(null,"ppb");
        return false;
      }
    }

    return true;
  };

  /**
   * Navigates to the specified page.
   * @param page The page to navigate. The valid values are "nextPage", "previousPage",
   * "firstPage", "lastPage", "nextBlock", "previousBlock" or a page number.
   * @param focusTo The code of the input element which should gain focus
   * after the navigation. Codes: cp: currentPage Element, ipp: itemsPerPage Element, ppb: pagesPerBlock Element
   */
  this.navigateTo = function(page, focusTo) {

    this.focusTo = focusTo;

    if (page) {
      if (page == "nextPage") this.currentPage = this.numberProp("currentPage") + 1;
      else if (page == "previousPage") this.currentPage = this.numberProp("currentPage") - 1;
      else if (page == "nextBlock") this.currentPage = this.numberProp("currentPage") + this.numberProp("pagesPerBlock");
      else if (page == "previousBlock") this.currentPage = this.numberProp("currentPage") - this.numberProp("pagesPerBlock");
      else if (page == "firstPage") this.currentPage = "1";
      else if (page == "lastPage") this.currentPage = "-1";
      else this.currentPage = page;
    }

    var submitType = this.props.submitType.before(".",this.props.submitType);
    if (submitType == "3") {
      this.navigateWithAjaxRequest();
    } else {
      this.navigateWithHttpRequest(submitType);
    }

 };


  /** @private */
  this.navigateWithAjaxRequest = function() {

    this.acc = new RequestParameterAccumulator();
    this.prepareRequestParameters();

    var data = new Object();

    data.navigator = this;
    if (this.props.callbackFunc) data.callbackFunc = this.props.callbackFunc;

    var prm = new Object();
    prm.userData = data;
    prm.requestParams = this.acc.toString();
    prm.errorHandler = this.props.errorHandler;

    prm.busyImage = this.props.busyImage;

    prm.callbackFunc = NavMgr.ajaxNavigationCallBack;

    var pform = OMgr.findParentObject(this.node,"mo:type","ProcessForm");

    if (pform) {
      prm.formName = pform.name;
      prm.commandName = this.props.commandName;
      AjaxMgr.submitProcessForm(prm);
    } else {
      prm.commandName = this.props.commandName;
      AjaxMgr.executeAjaxCommand(prm);
    }

  };

  /** @private */
  this.ajaxNavigationCallBack = function(response) {

    this.props.pagingInfo = response.getPagingInfoStr(this.props.dataSource);
    this.props.focusTo = this.focusTo;
    this.init();

    var data = response.request.props.userData;
    var func = data.callbackFunc;
    if (data.callbackFunc) {
      if (func.after) func = eval(func);
      response.request.props.userData = data;
      func(response);
    }

  };


  /** @private */
 this.navigateWithHttpRequest = function(submitType) {
   this.form = new GhostForm();
   this.prepareRequestParameters();
   this.form.form.action = "display.doms";
   var method = submitType == "1" ? "GET" : "POST";
   this.form.submit(method);
 };

  /** @private */
  this.collectInputs = function() {
    var elems = OMgr.getChildInputElements(this.node);
    for (var i = 0; i < elems.length; i++) {
      var elm = elems[i];
      var type = OMgr.getAttribute(elm,"mo:type");
      if (type) {
        var name = null;
        if (type == "NavigatorItemsPerPage") name = "itemsPerPage";
        else if (type == "NavigatorPagesPerBlock") name = "pagesPerBlock";
        else if (type == "NavigatorCurrentPage") name = "currentPage";
        if (name) {
          this.params[name] = elm.value;
        }
        continue;
      }
      if (!elm.checked || elm.checked == true) {
        this.params[elm.name] = elm.value;
      }
    }
  };

  /** @private */
  this.addParam = function(name, value) {
    if (this.acc) {
      this.acc.add(name, value);

    } else {
      this.form.addParameter(name, value);
    }
  };

  /** @private */
  this.prepareRequestParameters = function(form) {
    this.collectInputs();
    if (this.form) {
      var pageInfo = OMgr.getPageInfo(null);
      this.addParam("pg",pageInfo.name);
      this.addParam("_lang",pageInfo.lang);
    }
    for (var name in this.params) {
      var value = this.params[name];

      if (value && typeof(value) != "function") {
        if (name == "currentPage") {
          if (!this.currentPage) this.currentPage = value;

        } else if (name == "itemsPerPage") {
          if (!this.itemsPerPage) this.itemsPerPage = value;

        } else if (name == "pagesPerBlock") {
          if (!this.pagesPerBlock) this.pagesPerBlock = value;

     /*   } else if (name == "ci") {
          this.props.componentInstance = value;  */

        } else {
          this.addParam(name,value);
        }
      }
    }
    if (this.focusTo) this.addParam("_focusto",this.focusTo);
    this.addParam("_refreshds",this.getRefreshSettings());
    this.addParam("_rand",(new Date()).getTime());
  };

  /** @private */
  this.getRefreshSettings = function(navigate) {
    var acc = new Accumulator(";");
    acc.add("ds:" + this.props.dataSource);
    acc.add("cp:" + this.currentPage || this.props.currentPage);
    if (this.itemsPerPage) acc.add("ipp:" + this.itemsPerPage);
    if (this.pagesPerBlock) acc.add("ppb:" + this.pagesPerBlock);
    /* if (this.props.componentInstance) acc.add("ci:" + this.props.componentInstance); */
    return acc.toString();
  };

  /**
   * Adds the given parameter to the navigation request parameters.
   * @param name The name of the parameter
   * @param value The value of the parameter
   */
  this.addRequestParameter = function(name, value) {
    this.params[name] = value;
  };

  /**
   * The callback function to be called after the ajax navigation.
   * Before submiting the navigation request with ajax this field is checked to see it contains
   * a function. If this field is null then <a href="#callbackFunc">this.props.callbackFunc</a> property is
   * checked to see if it contains a callback function name.
   */
  this.callbackFunc = null;

  /**
   * The custom data object to pass to the callback function defined with <a href="#callbackFunc">this.props.callbackFunc</a>
   * property.
   */
  this.callbackCustomData = null;

};

/**
 * @class
 * Manager Class that provides methods to manage {@link Navigator} objects.
 * A static object of this class is available with name <CODE>OMgr</CODE>.
 * <PRE>var NavMgr = new NavigatorManager();</PRE>
 */

function NavigatorManager() {

  /**
   * Returns the enclosing navigator object.
   * @param name The name of the Navigator
   * @param node A DOM element node to start the search.
   * If omitted <code>document.body</code> is used as the default value.
   */
  this.getNavigator = function(name, node) {
    if (!node) node = document.body;
    return OMgr.findChildObject(node,"type:Navigator","mo:name",name);
  };

  /**
   * Locates the specified Navigator object and navigates to the
   * specified page.
   * @param node A DOM element node that is enclosed by the navigator.
   * @param page The page to navigate. The valid values are "nextPage", "previousPage",
   * "firstPage", "lastPage", "nextBlock", "previousBlock" or a page number.
   */
  this.navigateTo = function(node, page) {
    OMgr.getEnclosingObject(node).navigateTo(page, "cp");
  };

  this.keyUp = function(node, e) {
    return OMgr.getEnclosingObject(node).keyUp(e, node);
  };

  /** @private */
  this.ajaxNavigationCallBack = function(response) {
    var navigator = response.request.props.userData.navigator;
    navigator.ajaxNavigationCallBack(response);
  };


};

var NavMgr = new NavigatorManager();
