/* @compact */

/* Copyright (C) MOR YAZILIM. http://www.moryazilim.com
 * All rights reserved. This software is published under the terms of the
 * GNU Lesser General Public License. http://www.gnu.org/licenses/lgpl.txt
 */

/**
 * @fileoverview
 * Contains Process Management Framework related classes and functions
 * @author Erkan Cengiz
 * @version $Id: Process.js 193 2008-08-14 16:09:44Z erkan $
 */


/** @private */
function RecordChanged(elm, fname) {
  PMgr.recordChanged(elm, fname);
}


/**
 * @class
 * Manager Class that provides methods to manage the Process Based Wrapper Objects of MoreMotion.
 * A static object of this class is avaialable with name <CODE>PMgr</CODE>.
 * <PRE>var PMgr = new ProcessObjectManager();</PRE>
 */
function ProcessObjectManager() {

  /**
   * Sets the initial value of an {@link OptionBox} Field
   * @param node The DOM Node of the OptionBox
   */
  this.selectOption = function(node) {
    var f = OMgr.getEnclosingObject(node);
    if (f) f.selectOption();
  };


  /**
   * Returns the ProcessForm object associated with the specified name
   * @param name The name of the {@link ProcessForm}
   * @type ProcessForm
   */
  this.getForm = function(name) {
    var forms = document.forms;
    for (var i = 0; i < forms.length; i++) {
      var form = forms[i];
      if (OMgr.getAttribute(form,"mo:type") == "ProcessForm" && OMgr.getAttribute(form,"mo:name") == name) {
        return OMgr.getObject(form);
      }
    }
    return null;
  };


  /**
   * Returns the specified ProcessField object.
   * @param formName The name of the {@link ProcessForm}
   * @param blockName The name of the {@link ProcessBlock} in the ProcessForm
   * @param fieldName The name of the {@link ProcessField} in the ProcessBlock
   * @param recordNumber The sequence number of the ProcessRecord in the ProcessBlock. If this
   * parameter is omited "1" is assumed.
   * @type ProcessField
   */
  this.getField = function(formName, blockName, fieldName, recordNumber) {
    try {
      if (recordNumber == null) recordNumber = 1;
      return this.getForm(formName).getBlock(blockName).getRecord(recordNumber).getField(fieldName);
    } catch (e) {}
    return null;
  }

  /**
   * Submits the specified ProcessForm using the specified ProcessCommand.
   * @param formName The name of the {@link ProcessForm}
   * @param commandName The name of the {@link ProcessCommand}
   * @param target The target frame. This parameter is optional.
   * @return <code>true</code> if form is submitted, <code>false</code> if submittion is cancelled because of a
   * validation error.
   * @type boolean
   */
  this.submitProcessForm = function(formName, commandName, target) {
    if (PENDING_EVENT_Q > 0) {
      alert(OMgr.resource("NOT_READY_FOR_SUBMIT"));
      return false;
    }  
    if (formName.after) {
      return this.getForm(formName).submit(commandName, target);
    } else {
      var prm = formName;
      return this.getForm(prm.formName).submit(prm.commandName, prm.target);
    }
  };

  /**
   * Returns the ProcessField object that encloses the given the DOM Element node
   * @param <HTMLElement> Node of a DOM Element
   * @type ProcessField
   */
  this.getEnclosingField = function(node) {

    if (OMgr.getAttribute(node,"mo:field")) {
      return OMgr.getObject(node);
    }
    node = OMgr.findParentNode(node,"mo:field");
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Returns the ProcessRecord object that encloses the given DOM Element node
   * @param <HTMLElement> Node of a DOM Element
   * @type ProcessRecord
   */
  this.getEnclosingRecord = function(node) {

    if (OMgr.getAttribute(node,"mo:type") == "ProcessRecord") {
      return OMgr.getObject(node);
    }
    node = OMgr.findParentNode(node,"mo:type","ProcessRecord");
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Returns the ProcessBlock object that encloses the given DOM Element node
   * @param <HTMLElement> Node of a DOM Element
   * @type ProcessBlock
   */
  this.getEnclosingBlock = function(node) {

    if (OMgr.getAttribute(node,"mo:type") == "ProcessBlock") {
      return OMgr.getObject(node);
    }
    node = OMgr.findParentNode(node,"mo:type","ProcessBlock");
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };

  /**
   * Returns the ProcessForm object that encloses the given DOM Element node
   * @param <HTMLElement> Node of a DOM Element
   * @type ProcessForm
   */
  this.getEnclosingForm = function(node) {

    if (OMgr.getAttribute(node,"mo:type") == "ProcessForm") {
      return OMgr.getObject(node);
    }
    node = OMgr.findParentNode(node,"mo:type","ProcessForm");
    if (node) {
      return OMgr.getObject(node);
    }
    return null;
  };


  /** @private */
  this.getContainedBlockNode = function(node) {
    if (OMgr.getAttribute(node,"mo:type") == "ProcessRecord") {
      return node;
    } else {
      return OMgr.findChildNode(node,null,"mo:type","ProcessRecord",4);
    }
  };

/**
 * Sets the Process Record that the specified element resides in into the modified status.
 * @param node an HTMLElement object node is located in the Process Record to set its status
 * @param fname The name of the Process Field which is modified
 */
  this.recordChanged = function(node, fname) {
    var prec = this.getEnclosingRecord(node);
    if (prec) prec.setModified(fname);
  };


  this.fieldChanged = function(node) {
    var f = this.getEnclosingField(node);
    if (f) f.setModified();
  };


  /** @private - For compatibility */
  this.setInitialOption = function(elm, value) {
   OMgr.setInitialOption(elm,value)
  };

  /** @private */
  this.toggleSelection = function(elm,color) {
    var rec = OMgr.findParentObject(elm,"mo:type","ProcessRecord");
    if (rec) rec.checkRecord(elm.checked,color);
  };

  /** @private */
  this.toggleAllSelections = function(elm,blockName) {
    var node = OMgr.findParentNode(elm,"mo:type","ProcessForm")
    if (node) {
      var block = OMgr.findChildObject(node,"type:ProcessBlock","mo:name",blockName);
      if (block) block.processAllSelections(elm.checked == true);
    }
  };

  /** @private */
  this.keyCode = null;
  /** @private */
  this.specialSelect = function(node,color) {

    if (this.keyCode == null || !(this.keyCode == 16 || this.keyCode == 17) ) return;

    var r = OMgr.getObject(node);
    if (r) {
      var selected = r.isSelected();
      if (this.keyCode == 16 && !selected) {
        r.checkRecord(true);
      } else if (this.keyCode == 17 && selected) {
        r.checkRecord(false);
      }
    }

  };

  /** @private */
  this.specialKeyDown = function(e) {
    this.keyCode = (e.keyCode) ? e.keyCode : e.keyChar;
  };

  /** @private */
  this.specialKeyUp = function(e) {
    this.keyCode = null;
  };

}


/** Global Singleton ProcessObjectManager object
 * @type ProcessObjectManager
 */
var PMgr = new ProcessObjectManager();


/** @private */
function BlockInfo(bim,name) {
  
  this.bim = bim;
  this.name = name;
  this.seq = null;
  this.rpp = null;
  this._pbinfo_created = null;  
  
  this.setPBInfo = function(b) {

    b.setId(this.seq);
    if (this._pbinfo_created) return;
    
    var s = this.seq + ";";
    var prec = b.getRecord(1);
    if (prec) {
      var fields = prec.getFields();
      for (var i = 0; i < fields.length; i++) {
        if (i > 0) s += ",";
        var f = fields[i];

        if (f.getProcessFieldNames) {
          s += f.getProcessFieldNames(",");
        } else {
          if (f.elm && f.elm.type == "checkbox") s += "c:";
          s += f.name;
        }
      }
    } else return;

    if (bim.acc.forAjax) {
      bim.acc.add("_pbinfo_" + this.name, s);
    } else {
      var pbinfo = this.bim.form.createHidden("_pbinfo_" + this.name);
      pbinfo.value = s;
    }
    
    this._pbinfo_created = true;

  };
  
  
}

/** @private */
function BlockInfoManager(acc,form) {
  
  this.acc = acc;
  this.form = form;
  this.blocks = new Array();
  this.blockSeq = 0;
  
  this.getBlock = function(name) {
    var bi = this.blocks[name];
    if (!bi) {
      bi = new BlockInfo(this,name);
      bi.seq = ++this.blockSeq;
      this.blocks[name] = bi;
    }  
    return bi;
  };
  
  this.addBlock = function(name) {return this.getBlock(name);};

  this.getSequence = function(name) {
    return this.getBlock(name).seq;
  };
  
  this.parseRequiredBlocksStr = function(str) {
    
    var blks = str.split(",");

    for (var i = 0; i < blks.length; i++) {
      var def = blks[i].trim();
      var rpp = def.after(":");
      var blockName = def.before(":",def);
      this.getBlock(blockName).rpp = rpp;
    }
    
  };
  
}

/**
 * @class
 * Represents of a Process Form.
 * A Process Form is outer most container for process based elements in a page.
 * It may contain {@link ProcessBlock}, {@link ProcessCommand} and process objects.
 * <PRE><CODE>
 *   &lt;form mo:type="ProcessForm" mo:name="OrderUpdate" action="ProcessManager.doms"
 *            mo:props="formConfigName:'OrderUpdate'" &gt;
 *     ...
 *   &lt;/form&gt;
 * </CODE></PRE> The <CODE>formConfigName</CODE> property defines the configuration name of the form.
 * @extends MoreMotionObject
 */
function ProcessForm(node) {

  /**
   * ProcessForm Properties Object.
   *
   * <br/><br/><table border="0" cellspacing="3" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>formConfigName</b></td><td> The configuration name of the form.
   * If it is not specified then the name of this object is used as the configuration name.</td>
   * </tr>
   * </table>
   */
  this.props = null;

  /** @private */
  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isProcessForm = true;


  /**
   * Returns the Process Block associated with the specified name.
   * @param name The name of the Process Block
   * @type ProcessBlock
   */
  this.getBlock = function(name) {
    var block = OMgr.findChildObject(this.node,"type:ProcessBlock","mo:name",name);
    if (block) {
      block.form = this;
      return block;
    }
    return null;
  };

  /**
   * Returns the Process Command associated with the specified name
   * @param name The name of the Process Command
   * @return The Process Command associated with the name
   * @type ProcessCommand
   */
  this.getCommand = function(name) {
    var elms = this.node.elements["__pcommand"];
    if (!elms) return null;
    if (!elms.length) elms = [elms];
    for (var i = 0; i < elms.length; i++) {
      var cmd = new ProcessCommand(elms[i]);
      if (cmd.name == name) {
        return cmd;
      }
    }
    return null;
  };

  this.setAction = function(action) {
    this._action = this.node.action;
    this.node.action = action;
  };

  this.resetAction = function() {
    if (this._action) this.node.action = this._action;
  }

  /**
   * Resets all the HTML Input fields existing in this Process Form and removes the modified statuses
   * of all the {@link ProcessRecord} objects.
   */
  this.reset = function() {
    this.node.reset();
    /*
    var elms = this.node.elements;
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.name == "__selection") {
        var rs = new RecordStatus(elms[i]);
        if (rs.isModified()) {
          rs.setUnModified();
          rs.save();
        }
      }
    } */
  };

  /** Disables all the command buttons existing in this Process Form */
  this.disableCommandButtons = function() {

    var elms = this.node.elements["__pcommand"];
    if (!elms) return;
    if (!elms.length) elms = [elms];
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.type == "submit" || elm.type == "button") {
        elm.disabled = true;
      }
    }

  };

  /** Enables all the command buttons existing in this Process Form */
  this.enableCommandButtons = function() {

    var elms = this.node.elements["__pcommand"];
    if (!elms) return;
    if (!elms.length) elms = [elms];
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      if (elm.type == "submit" || elm.type == "button") {
        elm.disabled = false;
      }
    }

  };

  /** Creates a hidden input element with the given name.
   * @param name The name of the hidden element.
   */
  this.createHidden = function(name) {

    var frm = this.node;
    var elm = frm.elements[name];
    
    if (elm) {
      elm.value = "";
    } else {
      elm = OMgr.createElement("INPUT",name);
      elm.type = "hidden";
      frm.appendChild(elm);
    }
    return elm;

  };

  /**
   * Submits this Process Form with the specified command.
   * @param commandName The name of the Process Command
   * @param targetParam The value for the form.target attribute to set before submit.
   */
  this.submit = function(commandName, targetParam) {

    if (this.boolProp("formDisabled")) return false;

    var c = this.getCommand(commandName);
    if (!c) {
      alert(OMgr.resource("PROCESS_COMMAND_MISSING",commandName,this.name));
      return false;
    }

    if (this._validate(c, true) == false) return false;

    if (c.props.confirmMessage && !confirm(c.props.confirmMessage)) {
      return false;
    }

    /*
     * Set Hidden Fields
     */

    this.createHidden("_cmd").value = commandName;
    var pageInfo = OMgr.getPageInfo(this.node);
    this.createHidden("_originpage").value = pageInfo.name;
    if (pageInfo.variant) this.createHidden("_variant").value = pageInfo.variant;
    this.createHidden("_scope").value = this.props.formConfigName || this.name;
    if (this.props.lang != "ml") this.createHidden("_lang").value = pageInfo.lang;
    this.createHidden("_rand").value = (new Date()).getTime();
    this.createHidden("_enc").value = pageInfo.charset;

    if (c.props.beforeSubmitFunc) {
      try {
        var func = eval(c.props.beforeSubmitFunc);
        if (func(this, commandName) == false) return false;
      } catch (e) {
        alert(OMgr.resource("INVALID_BEFORE_SUBMIT_FUNCTION_DEFINITION",c.props.beforeSubmitFunc + " : " + e.message));
        return false;
      }
    }

    var target = null;

    if (targetParam) {
      var lc = targetParam.trim().toLowerCase();
      if (lc == "*") target = null;
      else if (lc == "default") target = "";
      else if (lc == "new window") target = "_blank";
      else if (lc == "whole page") target = "_top";
      else if (lc == "parent frame") target = "_parent";
      else target = targetParam;
    }

    var origTarget = null;
    if (target != null) {
      origTarget = this.node.target;
      this.node.target = target;
    }

    if (!this.node.target) {
      this.props.formDisabled = "true";
      this.saveProps();
      this.disableCommandButtons();
    }

    this.eliminateTemplateRecords();

    this.node.submit();

    if (origTarget != null) {
      this.node.target = origTarget;
    }

    return true;

  };


  /** @private */
  this.ajaxSubmit = function(prm) {
    
    if (PENDING_EVENT_Q > 0) {
      alert(OMgr.resource("NOT_READY_FOR_SUBMIT"));
      return false;
    }  

    if (this.boolProp("formDisabled")) return false;

    var c = this.getCommand(prm.commandName);
    if (!c) {
     c = new ProcessCommand(null);
     c.name = prm.commandName;
    }

    if (prm.beforeSubmitFunc) c.props.beforeSubmitFunc = prm.beforeSubmitFunc;
    if (prm.validateInput) c.props.validateInput = prm.validateInput;
    if (prm.confirmMessage) c.props.confirmMessage = prm.confirmMessage;
    if (prm.busyImage) c.props.busyImage = prm.busyImage;
    if (prm.submitMethod) c.props.submitMethod = prm.submitMethod;
    if (prm.requiredBlocks) c.props.requiredBlocks = prm.requiredBlocks;

    var acc = new RequestParameterAccumulator(prm.requestParams);
    acc.forAjax = true;

    if (this._validate(c, true, acc) == false) return false;

    if (c.props.confirmMessage && !confirm(c.props.confirmMessage)) {
      return false;
    }

    /*
     * Set Hidden Fields
     */
    acc.add("_cmd",prm.commandName);
    acc.add("_originpage",this.props.originPage);
    acc.add("_variant",this.props.variant);
    acc.add("_scope",this.props.formConfigName || this.name);
    acc.add("_usedsourceareas",AjaxMgr.getAlreadyUsedSourceAreas());
    acc.add("_targetpage",OMgr.getPageInfo().name);

    var func = prm.beforeSubmitFunc || c.props.beforeSubmitFunc;
    if (func) {
      try {
        if (func.after) func = eval(func);
        if (func(this, prm.commandName) == false) return false;
      } catch (e) {
        alert(OMgr.resource("INVALID_BEFORE_SUBMIT_FUNCTION_DEFINITION",c.props.beforeSubmitFunc + " : " + e.message));
        return false;
      }
    }

    this.collectAjaxHiddenBoxes(acc);

    var props = new Object();
    props.callbackFunc = prm.callbackFunc;
    props.userData = prm.userData;

    var request = new AjaxRequest(acc,null,props);

    request.service = "AjaxProcessManager.doms";
    request.busyImage = prm.busyImage || c.props.busyImage;
    request.waitForResponse = prm.waitForResponse;

    request.pageInfo = OMgr.getPageInfo(this.node);

    request.method = prm.submitMethod || "POST";
    request.send();

    return true;

  };

  /** @private */
  this.collectAjaxHiddenBoxes = function(acc) {

    var elms = this.node.elements["__ajax_hidden"];
    if (!elms) return;
    if (!elms.length) elms = [elms];
    for (var i = 0; i < elms.length; i++) {
      var elm = elms[i];
      acc.add( OMgr.getAttribute(elm,"mo:name"),elm.value);
    }

  };


  /**
   * Clears all the {@link ProcessBlock} objects existing in this Process Form
   */
  this.clear = function() {
    var blocks = OMgr.getObjects(this.node,"mo:type","ProcessBlock");
    for (var i = 0; i < blocks.length; i++) {
      blocks[i].clear();
    }
  };

  /**
   * Validates this Process Form according to the given command name
   * @param commandName The name of the Process Command
   */
  this.validate = function(commandName) {

    var c = this.getCommand(commandName);
    if (c == null) {
      alert(OMgr.resource("PROCESS_COMMAND_MISSING", commandName, this.name));
      return false;
    }

    return this._validate(c, false);
  };

  /** @private */
  this._validate = function(c, forSubmit, acc) {

    if (c.props.requiredBlocks) {
      
      if (!acc) acc = new Object();
      if (forSubmit == true) acc.forSubmit = true;
      acc.rpps = new Array();
      acc.bim = new BlockInfoManager(acc,this);

      acc.bim.parseRequiredBlocksStr(c.props.requiredBlocks);
      
      for (var name in acc.bim.blocks) {
        
        var bi = acc.bim.blocks[name];
        var b = this.getBlock(bi.name);
        if (b) {
          if (forSubmit) {
            bi.setPBInfo(b);
            if (bi.rpp && b.checkSelectedModified(bi.rpp) == false) {
              return false;
            }
          }

          acc.validate = c.boolProp("validateInput");

          acc.parentRecNum = new Stack();

          if (b.validate(acc) == false) return false;
        }
      }
    }
    return true;

  };

  /** @private */
  this.eliminateTemplateRecords = function() {
    var blocks = OMgr.getObjects(this.node,"mo:type","ProcessBlock");
    for (var i = 0; i < blocks.length; i++) {
      var block = blocks[i];
      if (block.boolProp("recordManip")) {
        var node = block.getTemplateRowNode();
        if (node) node.parentNode.removeChild(node);
      }
    }
  };

}

/**
 * @class
 * A class to iterate over the ProcessRecords of a ProcessBlock.
 * The Object of this class is created either by
 * {@link ProcessBlock#getRecordIterator} or {@link ProcessBlock#getRecordIterator}.
 */
function ProcessRecordIterator(blockOrRecord) {

  /** The ProcessBlock object
   * @type ProcessBlock
   */
  this.block = blockOrRecord.isProcessBlock ? blockOrRecord : blockOrRecord.getBlock();

  /** @private */
  this.recordManip = this.block.boolProp("recordManip");

  /** @private */
  this.index = -1;
  /** @private */
  this.rowNode = null;
  /** @private */
  this.recordNode = null;

  /** @private */
  this.nodes = new Array();

  /** @private */
  this.init = function() {
    this.nodes = new Array();
    this._collectRecordNodes(this.block.node);
    this._first = true;
  };

  /** @private */
  this._collectRecordNodes = function(parentNode) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var attr = OMgr.getAttribute(node,"mo:type");
        if (attr && attr == "ProcessRecord") {
          if (this._first == false || !this.recordManip) {
            this.nodes[this.nodes.length] = node;
          }
          this._first = false;
        }
        else if (node.hasChildNodes) {
          this._collectRecordNodes(node);
        }
      }
    }

  };

  this.init();

  /**
   * Locates the iterator to the top of the records so that the next <CODE>{@link #next}</CODE> method call
   * returns the first record.
   */
  this.top = function() {
    this.index = -1;
    this.rowNode = null;
    this.recordNode = null;
  };

  /**
   * Locates the iterator to the bottom of the records so that the next <CODE>{@link #previous}</CODE> method
   * call returns the last record.
   */
  this.bottom = function() {
    this.index = this.nodes.length;
    this.rowNode = null;
    this.recordNode = null;
  };

  /**
   * Iterates to the next ProcessRecord. If the current pointer in already on
   * the last record and there is no more next record to iterate then <CODE>false</CODE>
   * otherwise <CODE>true</CODE> is returned.
   * @type boolean
   */
  this.next = function() {

    if (this.nodes.length > 0 && this.nodes.length > this.index+1) {
      this.index++;
      this.recordNode = this.nodes[this.index]
      this.rowNode = this.recordNode.tagName == "TR" ? this.recordNode : this.recordNode.parentNode.parentNode;
      return true;
    }

    this.rowNode = null;
    this.recordNode = null;
    return false;

  };

  /**
   * Iterates to the previous ProcessRecord. If the current pointer in already on
   * the first record and there is no more previous record to iterate then <CODE>false</CODE>
   * otherwise <CODE>true</CODE> is returned.
   * @type boolean
   */
  this.previous = function() {

    if (this.nodes.length > 0 && this.index > 0 && this.nodes.length > this.index-1) {
      this.index--;
      this.recordNode = this.nodes[this.index]
      this.rowNode = this.recordNode.tagName == "TR" ? this.recordNode : this.recordNode.parentNode.parentNode;
      return true;
    }

    this.rowNode = null;
    this.recordNode = null;
    return false;
  };

  /**
   * Returns the currently located ProcessRecord object
   * @type ProcessRecord
   */
  this.getRecord = function() {
    if (this.recordNode) {
      return OMgr.getObject(this.recordNode);
    }
    return null;
  };

  /**
   * Returns the DOM node of currently located ProcessRecord
   * @type DOMnode
   */
  this.getRecordNode = function() {
    return this.recordNode;
  };

  /**
   * Returns the row node (<CODE>TR</CODE> Element) of currently located ProcessRecord
   * @type HTML_TR_Element
   */
  this.getRowNode = function() {
    return this.rowNode;
  };

  /**
   * Returns the index in the ProcessBlock of currently located ProcessRecord
   * @type integer
   */
  this.getIndex = function() {
    return this.index + 1;
  };

  if (blockOrRecord.isProcessRecord) {
    for (var i = 0; i < this.nodes.length; i++) {
      var node = this.nodes[i];
      if (node == blockOrRecord.node)  {
        this.index = i;
        this.recordNode = node;
        this.rowNode = node.tagName == "TR" ? node : node.parentNode.parentNode;
        return this;
      }
    }
    alert("Record cannot be located");
  }
  
  return this;

}

/**
 * @class
 * Represents a Process Block.
 * A Process Block may contain one or more {@link ProcessRecord} objects.
 * This class wraps a container DOM element that can contain child DOM elements.
 * <PRE><CODE>
 *   &lt;table mo:type="ProcessBlock" mo:name="customers" mo:props="warnColor:'#F5CAC0'" &gt;
 *     ...
 *   &lt;/table&gt;
 * </CODE></PRE> The <CODE>mo:name</CODE> attribute of the element specifies the name of the process block
 * @extends MoreMotionObject
 */
function ProcessBlock(node) {

  /**
   * ProcessBlock Properties Object.
   *
   * <br/><br/><table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td valign="top"><b>recordManip</b></td><td> The flag that specifies that the process records of this
   * block can be manipulated (add, remove, clone, etc). Please note that record manipulation also
   * requires an additional record in the block to be used as the template. When this flag is true and
   * when this object is initialized it locates the first record in the block and makes it invisible
   * to use it as a template.</td>
   * <tr><td><b>selectionColor</b></td><td>The code of a color to use to mark the selected
   * process records of this block</td>
   * <tr><td><b>warnColor</b></td><td>The code of a color to use to mark the whole process block background
   * along with a warning such as "No record is selected".</td>
   * </tr>
   * </table>
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isProcessBlock = true;

  /** @private */
  this.id = "1";

  /** @private */
  this.setId = function(id) {
    this.id = id + "";
    this.prefix = "pf_" + id + "_";
  };
  
  /** @private */
  this.init = function() {
    if (this.boolProp("recordManip")) {
      this.prepareTemplateRecord();
    }
  };

  /** @private */
  this.hasOnlyOneTD = function(parentNode) {
    var first = true;
    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1 && node.tagName == "TD") {
        if (first == false) return false;
        first = false;
      }
    }
    return !first;
  };

  /** @private */
  this.isRegular = function() {

    if (this._isRegular != null) return this._isRegular;

    var recnode = OMgr.findChildNode(this.node, null, "mo:type", "ProcessRecord",4);

    this._isRegular =
      recnode && (
       recnode.tagName == "TR" || (
         recnode.parentNode.tagName == "TD" && this.hasOnlyOneTD(recnode.parentNode.parentNode)
       )
      );

    return this._isRegular;

  };


  /** @private */
  this.canManipulateRecords = function(alertMessage) {
    if (!this.isRegular()) {
      if (alertMessage) alert(OMgr.resource("RECORD_MANIPULATION_NOT_ALLOWED"));
      return false;
    }
    return true;
  };


  /** Returns the Process Form that this Process Block resides in.
   * @type ProcessForm
   */
  this.getForm = function() {
    return PMgr.getEnclosingForm(this.node);
  };

  /**
   * Returns the {@link ProcessRecord} object associated with the specified record index.
   * Note: To process all the records of the parent ProcessBlock
   * {@link ProcessRecordIterator} object should be used.
   * @param index The index number of the ProcessRecord
   * @type ProcessRecord
   */
  this.getRecord = function(index) {
    var itr = this.getRecordIterator();
    for (var i = 0; i != index; i++) {
      if (!itr.next()) break;
    }
    if (i == index) return itr.getRecord();
    return null;
  };


  /**
   * Returns a {@link ProcessRecordIterator} object to iterate over the
   * process records of this block.
   * @type ProcessRecordIterator
   */
  this.getRecordIterator = function() {
    return new ProcessRecordIterator(this);
  };

  /**
   * Returns the first Process Record of this block
   * @type ProcessRecord
   */
  this.getFirstRecord = function() {
    return this.getRecord(1);
  };

  /** @private */
  this.getLastRecord = function() {
    var itr = this.getRecordIterator();
    if (itr) {
      itr.bottom();
      itr.previous();
      return itr.getRecord();
    }
    return null;
  };

  /** @private */
  this.setStatusFieldName = function(elm) {
    elm.name = "px_" + this.id + "_recstat";
  };


  /** @private */
  this._hasInnerPB = null;
  this.hasInnerPB = function() {
    if (this.isInnerPB) return false;
    if (this._hasInnerPB == null) {
      this._hasInnerPB = OMgr.findChildNode(this.node, null, "mo:type", "ProcessBlock") ? true : false;
    }
    return this._hasInnerPB;
  };

  /**
   * Validates the Process Fields existing in this Process Block.
   * @type boolean
   * @return <code>true</code> if all the fields are validates <code>false</code> otherwise.
   */
  this.validate = function(acc) {

    var itr = this.getRecordIterator();
    var i = 0;
    while (itr.next()) {
      var r = itr.getRecord();
      r.block = this;
      if (acc) {
        if (i == 0) acc.parentRecNum.push(i);
        else acc.parentRecNum.set(i);
      }
      if (r.validate(acc) == false) return false;
      i++;
    }
    return true;
  };

  /**
   * Clears the values of all the Process Fields existing in this Process Block
   */
  this.clear = function() {
    var itr = this.getRecordIterator();
    while (itr.next()) {
      itr.getRecord().clear();
    }
  };

  /** 
   * Checkes or unchecks all the process records of this block.
   * @param check Boolean value: true or false
   */
  this.processAllSelections = function(check) {

    var color = this.props.selectionColor;
    var itr = this.getRecordIterator();
    while (itr.next()) {
      itr.getRecord().checkRecord(check, color);
    }

  };

  /**
   * Sets a background color to all the Process Records in this Process Block
   * to warn the user that no selection is made.
   */

  this.setWarnColor = function() {

    var firstElm = null;

    var color = this.props.warnColor;
    var itr = this.getRecordIterator();
    while (itr.next()) {
      var node = itr.getRecordNode();
      if (color) {
        node.style.backgroundColor = color;
      }
      if (firstElm == null) {
        firstElm = OMgr.getChildNode(node,"input","_selected");
      }
    }

    if (firstElm) firstElm.focus();
  };

  /**
   * Removes all the marks from the Process Records.
   */
  this.removeWarnColor = function() {

    var color = this.props.warnColor;
    if (!color) return;

    var itr = this.getRecordIterator();
    while (itr.next()) {
      var node = itr.getRecordNode();
      node.style.backgroundColor = null;
    }

  };


  /** @private */
  this.checkSelectedModified = function(rpp, innerpb) {

    var onlyOneSelectedCheck = rpp == "OnlyOneSelected";
    var atLeastOneSelectedCheck = rpp == "AtLeastOneSelected";
    var atLeastOneModifiedCheck = rpp == "AtLeastOneModified";

    if (onlyOneSelectedCheck || atLeastOneSelectedCheck || atLeastOneModifiedCheck) {

      var selectionCount = 0;
      var modifiedCount = 0;
      var recordCount = 0;

      var itr = this.getRecordIterator();
      while (itr.next()) {
        var rs = itr.getRecord().getRecordStatus();
        recordCount++;
        if (rs.isSelected) selectionCount++;
        if (rs.isModified()) modifiedCount++;
        itr.getRecordNode().style.backgroundColor = "";
      }

      if (onlyOneSelectedCheck && selectionCount == 0) {
        alert(OMgr.resource("NO_RECORD_SELECTED"));
        this.setWarnColor();
        return false;
      }

      if (onlyOneSelectedCheck && selectionCount > 1) {
        alert(OMgr.resource("ONLY_ONE_RECORD_CAN_BE_SELECTED"));
        this.setWarnColor();
        return false;
      }

      if ((!innerpb || recordCount > 0) && atLeastOneSelectedCheck && selectionCount == 0) {
        alert(OMgr.resource("NO_RECORD_SELECTED"));
        this.setWarnColor();
        return false;
      }

      if ((!innerpb || recordCount > 0) && atLeastOneModifiedCheck && modifiedCount == 0) {
        alert(OMgr.resource("NO_RECORD_MODIFIED"));
        this.setWarnColor();
        return false;
      }

    }

    return true;

  };


  /**
   * Copies the specified Process Record.
   * The record that is identified by <code>fromRecordNumber</code> parameter is copied
   * before the record identified by <code>toRecordNumber</code> parameter.
   * @param fromRecordNumber The number of the source record
   * @param beforeRecordNumber The number of record that the new record is copied before it.
   * If this parameter is omitted the new record is appended at the block.
   * @return The copied record.
   * @type ProcessRecord
   */
  this.copyRecord = function(fromRecordNumber, beforeRecordNumber) {

    if (!this.canManipulateRecords(true)) return null;

    var sourceNode = null;
    if (fromRecordNumber) {
      var sourceRec = this.getRecord(fromRecordNumber);
      sourceNode = sourceRec.getRowNode();
    } else {
      sourceNode = this.getTemplateRowNode();
      if (!sourceNode) return null;
      fromRecordNumber = "t";
    }

    var clonedNode = sourceNode.cloneNode(true);
    clonedNode.style.display = "";

    var newRecordNumber = this.getRecordSequence();
    this.changeElementIds(clonedNode,fromRecordNumber,newRecordNumber);

    var beforeNode = null;
    if (beforeRecordNumber) {
      var beforeRec = this.getRecord(beforeRecordNumber);
      beforeNode = beforeRec.getRowNode();
    }

    var targetNode = this.getDOMNode().insertBefore(clonedNode, beforeNode);
    this.templateRecord.clone(newRecordNumber);

    var nr = new ProcessRecord(this,newRecordNumber);
    if (fromRecordNumber == "t") nr.clear();
    return nr;
  };

  /**
   * Deletes all the ProcessRecords of this Process Block.
   */
  this.removeAllRecords = function() {
    var itr = this.getRecordIterator();
    while (itr.next()) {
      var node = itr.getRowNode();
      node.parentNode.removeChild(node);
    }
    if (WinMgr.isGecko) this.node.innerHTML = this.node.innerHTML;
  };


  /**
   * Creates a new Process Record and appends it at the bottom of the block.
   * If a Process Field exists in the record that needs the record number (e.g. {link CheckBox}
   * then you have to call {@link #setRecordNumbers} method of the block.
   * @return The created Process Record
   * @type ProcessRecord
   */
  this.addRecord = function() {
    if (!this.canManipulateRecords(true)) return null;
    var node = this.getTemplateRowNode();
    if (node) {
      var newNode = node.parentNode.appendChild(node.cloneNode(true));
      newNode.style.display = "";
      newNode.removeAttribute("mo:template");
      OMgr.initialize(newNode);
      if (OMgr.getAttribute(newNode,"mo:type") == "ProcessRecord") {
       return OMgr.getObject(newNode);
      }
      var recordNode = OMgr.findChildNode(newNode,null,"mo:type","ProcessRecord",7);
      if (recordNode) return OMgr.getObject(recordNode);
    }
    return null;
  }

  /**
   * Loops over the records of this block and sets the record numbers of the specified ProcessFields
   * existing in the ProcessRecords
   * @param fields The names of the Process Fields separated with commas.
   */
  this.setRecordNumbers = function(fields) {
    fields = fields.split(",");
    var itr = this.getRecordIterator();
    var recnum = 0;
    while (itr.next()) {
       recnum++;
       var r = itr.getRecord();
       for (var i = 0; i < fields.length; i++) {
         var f = r.getField(fields[i]);
         if (f && f.setRecordNumber) {
           f.setRecordNumber(recnum);
         }
       }
    }
  };

  /** @private */
  this.getTemplateRowNode = function(silent) {

    var parentNode = OMgr.getChildNode(this.node,"TBODY");
    if (parentNode && parentNode.hasChildNodes) {
      for (var i = 0; i < parentNode.childNodes.length; i++) {
        var node = parentNode.childNodes[i];
        if (node.nodeType == 1) {
          if (PMgr.getContainedBlockNode(node)) {
            return node;
          }
        }
      }
    }
    if (!silent) alert(OMgr.resource("TEMPLATE_RECORD_MISSING",this.name));
    return null;
  };

  /** @private */
  this.prepareTemplateRecord = function() {

    var node = this.getTemplateRowNode();
    if (node) {
      node.style.display = "none";
      node.setAttribute("mo:template","true");
    } else {
      alert(OMgr.resource("CANNOT_PREPARE_TEMPLATE_RECORD"));
      return;
    }

  };

}

/**
 * @class Represents the status of an Process Record
 * @param elm The hidden element object named <CODE>px_recstat</CODE>
 * existing in the Process Record
 */
function RecordStatus(elm) {

  /** Keeps the names of the modified fields separated with commans */
  this.modifiedFields = "";

  /** Flag to indicate that the Process Record is selected */
  this.isSelected = false;

  /** @private */
  this.parentRecNum = "";

  /** @private */
  this.elm = elm;

  /** @private */
  var value = elm.value;

  if (value) {
    var values = value.split(";");
    this.modifiedFields = values[0];
    this.isSelected = values[1] == "true";
    this.parentRecNum = values[2];
  }

  /**
   * Adds the specified field to the modified fields list
   * @param fieldName The name of the field to add to the modified fields list
   */
  this.addModifiedField = function(fieldName) {
    if (!this.modifiedFields) this.modifiedFields = fieldName;
    else {
      var s = " " + this.modifiedFields + " ";
      if (s.indexOf(fieldName) == -1) this.modifiedFields += " " + fieldName;
    }
  };

  /**
   * Removes the specified field from the modified fields list
   * @param fieldName The name of the field to remove from the modified fields list.
   */
  this.removeModifiedField = function(fieldName) {
    if (!this.modifiedFields) return;
    if (this.modifiedFields.indexOf(fieldName) == -1) return;
    if (this.modifiedFields == fieldName) { this.modifiedFields = ""; return; }
    var fields = this.modifiedFields.split(" ");
    var buf = "";
    for (var i = 0; i < fields.length; i++) {
      if (fields[i] != fieldName) {
        if (buf) buf += " ";
        buf += fields[i];
      }
    }
    this.modifiedFields = buf;
  };

  /**
   * Sets the status to unmodified
   */
  this.setUnModified = function() {
    this.modifiedFields = "";
  };

  /**
   * Returns the modified status
   * @type boolean
   */
  this.isModified = function() {
    return this.modifiedFields != "";
  };

  /** @private */
  this.toString = function() {
    return this.modifiedFields + ";" + (this.isSelected ? "true" : "") + ";" + this.parentRecNum;
  };

  /**
   * Saves the Record Status to the hidden status element
   */
  this.save = function() {
    var value = this.toString();
    this.elm.value = value == ";;" ? "" : value;
  };

}

/**
 * @class
 * Represents a Process Record. This class wraps a container DOM element that can contain child DOM nodes.
 * Although there is no restriction in the type of the DOM element, for the repeating process records <CODE>TR</CODE>
 * element is probably the most suitable one. <p>
 * A Process record should be in a {@link ProcessBlock} and may contain one or more {@link ProcessField} objects.
 *
 * <PRE><CODE>
 * &lt;tr mo:type="ProcessRecord" &gt;
 *   &lt;td&gt;
 *     &lt;input name="__selection" type="checkbox" value="checked" onclick="PMgr.toggleSelection(this);"&gt;
 *   &lt;/td&gt; <font color="gray">
 *   &lt;td&gt;
 *     &lt;input name="pf_NAME" type="text" onchange="PMgr.fieldChanged(this);"
 *               mo:type="EditBox" mo:field="true" mo:name="NAME" mo:iValue="" /&gt;
 *   &lt;/td&gt; </font>
 * &lt;/tr&gt;
 * </CODE></PRE>Optionally a process record may contain a
 * <CODE>INPUT type="checkbox"</CODE> element with name <CODE>__selection</CODE> as the record selection element.
 * <p>
 *
 * After using the {@link #remove}, {@link #clone}, {@link #moveBottom}, {@link #moveDown},
 * {@link #moveTop}, {@link #moveUp} methods of this object you have to call
 * {@link ProcessBlock#setRecordNumbers} method of the owner Process Block object if there are
 * Process Fields exists in the record that needs the record number e.g. {@link CheckBox}.
 *
 * @extends MoreMotionObject
 * @param node the DOM Element node that is wrapped by this objects
 *
 */
 function ProcessRecord(node) {

  /** @private */
  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isProcessRecord = true;

  /** @private */
  this.getSelectionElement = function() {
    return OMgr.getChildNode(this.node,"input","__selection");
  };

  /** @private */
  this.getRecordStatusElement = function() {
    if (!this._rse) {
      this._rse = OMgr.getChildNode(this.node,"input","px_","_recstat");
      if (!this._rse) {
        var elm = OMgr.createElement("INPUT", "px_recstat");
        elm.type = "hidden";
        var insertNode = OMgr.getChildNode(this.node,"td");
        if (insertNode) {
          insertNode.appendChild(elm);
          this._rse = elm;
        }
      }
    }
    return this._rse;
  };

  /**
   * Returns RecordStatus object for this record
   * @type RecordStatus
   */
  this.getRecordStatus = function() {
    return new RecordStatus(this.getRecordStatusElement());
  };

  /** Returns the modified status of this Process Record.
   * @type boolean
   */
  this.isModified = function() {
    return this.getRecordStatus().isModified();
  };

  /** Returns the selected status of this Process Record
   * @type boolean
   */
  this.isSelected = function() {
    return this.getRecordStatus().isSelected;
  };


  /** @private */
  this.block = null;

  /** Returns the Process Block that this Process Record resides in
   * @type ProcessBlock
   */
  this.getBlock = function() {
    if (!this.block) this.block = PMgr.getEnclosingBlock(this.node);
    return this.block;
  };

  /**
   * Returns the specified inner {@link ProcessBlock} object contained in this record.
   * @type ProcessBlock
   */
  this.getInnerBlock = function(blockName) {
    var node = OMgr.findChildNode(this.node,"type:ProcessBlock","mo:name",blockName);
    if (node) return OMgr.getObject(node);
    return null;
  };

  /**
   * Returns the Process Field located in this ProcessRecord
   * @param name The name of the Process Field
   * @type ProcessField
   * @return The instance of the Process Field.
   */
  this.getField = function(name) {
    var fnode = OMgr.findChildNode(this.node,null,"mo:name",name);
    if (fnode) {
      return OMgr.getObject(fnode);
    }
    return null;
  };

  /**
   * Returns an Array of {@link ProcessField} objects contained in this record.
   * @type ProcessField[]
   */
  this.getFields = function() {
    var fields = new Array();
    this._collectFields(fields, this.node);
    return fields;
  };

  /** @private */
  this._collectFields = function(fields, parentNode) {

    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var attr = OMgr.getAttribute(node,"mo:field");
        if (attr) {
          var f = OMgr.getObject(node);
          if (f) fields[fields.length] = f;
        }
        if (node.hasChildNodes && OMgr.getAttribute(node,"mo:type") != "ProcessBlock") {
          this._collectFields(fields, node);
        }
      }
    }

  };

  /**
   * Returns the previous ProcessRecord
   * @type ProcessRecord
   */
  this.getPrevious = function() {
    var node = this.getRowNode().previousSibling;
    while (node) {
      if (node.nodeType == 1) {
        if (!OMgr.getAttribute(node,"mo:template")) {
          var recordNode = PMgr.getContainedBlockNode(node);
          if (recordNode) {
            return OMgr.getObject(recordNode);
          }
        }
      }
      node = node.previousSibling;
    }
    return null;
  };

  /**
   * Returns the next ProcessRecord
   * @type ProcessRecord
   */
  this.getNext = function() {
    var node = this.getRowNode().nextSibling;
    while (node) {
      if (node.nodeType == 1) {
        if (!OMgr.getAttribute(node,"mo:template")) {
          var recordNode = PMgr.getContainedBlockNode(node);
          if (recordNode) {
            return OMgr.getObject(recordNode);
          }
        }
      }  
      node = node.nextSibling;
    }
    return null;
  };

  /**
   * Creates a ProcessRecordIterator, locates it to this record and returns it.
   * @type ProcessRecordIterator
   */
  this.getRecordIterator = function() {
    return new ProcessRecordIterator(this);
  };

  /**
   * Returns the index of this Process Record within the parent Process Block.
   * @type integer
   */
  this.getRecordIndex = function() {
    return this.getRecordIterator().getIndex();
  };

  /** @private */
  this._setSelectedStatus = function(checked) {
    var me = this.getModifiedElement();
    if (!me) return;
    me.value = checked + "|" + me.value.after("|");

  };

  /** @private */
  this._checkRecord = function(checked, color) {
    if (checked) {
      this.node.style.backgroundColor = color || this.getBlock().props.selectionColor;
    } else {
      this.node.style.backgroundColor = "";
    }

    var recstat = this.getRecordStatus();
    if (recstat) {
      recstat.isSelected = checked;
      recstat.save();
    }

  };

  /** @private */
  this.selectRecord = function(checked, color) {
    this.checkRecord(checked,color);
  };

  /**
   * Checks or unchecks this ProcessRecord
   * @param checked Boolean value: true or false
   * @param color The selection color
   */
  this.checkRecord = function(checked, color) {
    var e = this.getSelectionElement();
    if (e && e.disabled != true) {
      e.checked = checked;
      this._checkRecord(checked,color);
    }
  };

  /**
   * Removes this ProcessRecord
   */
  this.remove = function() {
    if (!this.getBlock().canManipulateRecords(true)) return;
    var node = this.getRowNode();
    node.parentNode.removeChild(node);
  };

  /**
   * Moves this ProcessRecord to the top of the ProcessBlock
   * @type ProcessRecord
   * @return This Record
   */
  this.moveTop = function() {
    if (!this.getBlock().canManipulateRecords(true)) return null;
    var parentNode = this.getRowNode().parentNode;
    for (var i = 0; i < parentNode.childNodes.length; i++) {
      var node = parentNode.childNodes[i];
      if (node.nodeType == 1) {
        var recordNode = OMgr.findChildNode(node,null,"mo:type","ProcessRecord");
        if (recordNode) {
          parentNode.insertBefore(parentNode.removeChild(this.getRowNode()),node);
        }
      }
    }
    return this;
  };

  /**
   * Moves this ProcessRecord before the previous record
   * @type ProcessRecord
   * @return This Record
   */
  this.moveUp = function() {
    if (!this.getBlock().canManipulateRecords(true)) return null;
    var node = this.getRowNode();
    var rec = this.getPrevious();
    if (rec) {
      var pnode = node.parentNode;
      pnode.insertBefore(pnode.removeChild(node),rec.getRowNode());
    }
    return this;
  };

  /**
   * Moves this ProcessRecord after the next record
   * @type ProcessRecord
   * @return This Record
   */
  this.moveDown = function() {
    if (!this.getBlock().canManipulateRecords(true)) return null;
    var node = this.getRowNode();
    var rec = this.getNext();
    if (rec) {
      rec = rec.getNext();
      var pnode = node.parentNode;
      if (rec) {
        pnode.insertBefore(pnode.removeChild(node),rec.getRowNode());
      } else {
        pnode.appendChild(pnode.removeChild(node));
      }
    }
    return this;
  };


  /**
   * Moves this record to the bottom of the Process Block
   * @type ProcessRecord
   * @return This Record
   */
  this.moveBottom = function() {
    if (!this.getBlock().canManipulateRecords(true)) return null;
    var parentNode = this.getRowNode().parentNode;
    parentNode.appendChild(parentNode.removeChild(this.getRowNode()));
    return this;
  };

  /**
   * Sets the visibility of the record.
   * @param value <CODE>true</CODE> or <CODE>false</CODE>
   */
  this.setVisible = function(value) {
    if (!this.getBlock().canManipulateRecords(true)) return;
    var rowNode = this.getRowNode();
    if (rowNode) rowNode.style.display = value == true ? "" : "none";
  };

  /**
   * Returns <CODE>true</CODE> if this record is visible
   * @type boolean
   */
  this.isVisible = function(){
    var rowNode = this.getRowNode();
    return rowNode && rowNode.style.display != "none";
  };

  /**
   * Clones this ProcessRecord and returns it
   * @type ProcessRecord
   */
  this.clone = function() {
    if (!this.getBlock().canManipulateRecords(true)) return null;
    var rowNode = this.getRowNode();
    var nextRecord = this.getNext();
    var newNode = null;
    if (nextRecord) {
      newNode = rowNode.parentNode.insertBefore(rowNode.cloneNode(true),nextRecord.getRowNode());
    } else {
      newNode = rowNode.parentNode.appendChild(rowNode.cloneNode(true));
    }
    if (newNode) {
      var recordNode = PMgr.getContainedBlockNode(newNode);
      if (recordNode) {
        var newRecord = OMgr.getObject(recordNode);
        this.cloneValues(this,newRecord);
        return newRecord;
      }
    }
    return null;

  };


  /** @private */
  this.cloneValues = function(r1, r2) {
    var fields = r1.getFields();
    for (var i = 0; i < fields.length; i++) {
      var f = fields[i];
      if (f.needsValueCloning) {
        r2.getField(f.name).setValue( f.getValue() );
      }
    }
  };

  /**
   * Sets the modified status of this Process Record to "modified".
   * @param fieldName The name of the field that is modified.
   * @param modified Optional boolean parameter. If it contains <CODE>false</CODE> then modified status
   * of the record ise set to "unmodified".
   */
  this.setModified = function(fieldName,modified)  {
    if (modified == false) {
      this.setUnModified(fieldName); 
      return;
    }
    var recstat = this.getRecordStatus();
    if (recstat) {
      recstat.addModifiedField(fieldName);
      recstat.save();
    }
  };

  /**
   * Sets the modified status of this Process Record to "unmodified".
   * <p>if <CODE>fieldName</CODE> parameter is supplied, then only that field is removed
   * from the modified fields list. If after removing no field name remains in the
   * list then the modified status of this record becomes "unmodified".
   * @param fieldName The name of the field that is unmodified.
   */
  this.setUnModified = function(fieldName) {
    var recstat = this.getRecordStatus();
    if (recstat) {
      if (fieldName) recstat.removeModifiedField(fieldName);
      else recstat.setUnModified();
      recstat.save();
    }
  };

  /** Clears the Process Fields contained in this Process Record */
  this.clear = function() {
    var fields = this.getFields();
    for (var i = 0; i < fields.length; i++) {
      var f = fields[i];
      if (f.clear) f.clear();
      if (f.isModified()) this.setModified(f.name);
      else this.setUnModified(f.name);
    }
    return this;
  };

  /**
   * Validates the fields of this record and returns <CODE>true</CODE> if validation is
   * successful and <CODE>false</CODE> otherwise.
   * @type boolean
   */
  this.validate = function(acc) {

    var fields = this.getFields();
    for (var i = 0; i < fields.length; i++) {
      var f = fields[i];

      if (!acc || acc.validate) {
        if (f.validate) {
          if (f.validate(acc) == false) return false;
        }
      }

      if (acc && (acc.forAjax || acc.forSubmit)) {
        f.setSubmitPrefix(this.block.prefix);
        if (acc.forAjax) f.appendRequestParams(acc);
      }

    }

    var rse = this.block.isInnerPB || (acc && (acc.forAjax || acc.forSubmit)) ? this.getRecordStatusElement() : null;

    if (rse && acc && this.block.isInnerPB) {
      var recstat = new RecordStatus(rse);
      if (recstat) {
        recstat.parentRecNum = acc.parentRecNum.peek(1);
        recstat.save();
      }
    }

    if (rse && acc && (acc.forAjax || acc.forSubmit)) {
       this.block.setStatusFieldName(rse);
       if (acc.forAjax) acc.add(rse.name,rse.value);
    }

    if (this.block.hasInnerPB()) {
      var innerBlocks = this.getInnerBlocks();
      if (innerBlocks) {
        for (var j = 0; j < innerBlocks.length; j++) {
          var b = innerBlocks[j];
          b.isInnerPB = true;
          if (acc.forAjax || acc.forSubmit) {
            var bi = acc.bim.getBlock(b.name);
            bi.setPBInfo(b);
            if (bi.rpp && b.checkSelectedModified(bi.rpp) == false) {
              return false;
            }
          }
          
          if (b.validate(acc) == false) return false;
        }
      }
    }


    return true;
  };

  /** @private */
  this.getInnerBlocks = function() {
    var blocks = new Array();
    var b = OMgr.findChildObject(this.node,null,"mo:type","ProcessBlock");
    if (b == null) return null;
    blocks[0] = b;
    return blocks;
  };

  /**
   * Returns the HTML <code>TR</code> Element node that encloses this Process Record.
   * @type HTML_TR_Element
   */
  this.getRowNode = function() {
    return this.node.tagName == "TR" ? this.node : this.node.parentNode.parentNode;
  };

}

/**
 * @class
 * Represents a Process Field.
 * @extends MoreMotionObject
 * @param node the DOM Element node that is wrapped by this field
 */
function ProcessField(node) {

  /**
   * ProcessField Properties Object
   *
   * <br/><br/>
   * <table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b><a name="nonBlank">nonBlank</a></b></td><td> Flag that specifies that this field must contain a value</td></tr>
   * <tr><td><b><a name="isNumber">isNumber</a></b></td><td> Flag that specifies that this field must contain a valid number</td></tr>
   * <tr><td valign="top"><b><a name="validValues">validValues</a></b></td><td> The valid values that can be entered into this 
   * Process Field. The values should be separated with ";" characters.
   * e.g. "AAA;BBB;CCC". If the value of this Process Field is number then number ranges can be defined.
   * e.g. "1:20;30:40".</td></tr>
   * <tr><td valign="top"><b><a name="valHandler">valHandler</a></b></td><td> The name of the Validation Handler Function.
   * If this property is defined then it is called by the validate() method of this object
   * and this object is passed to it as the parameter.
   * <p>
   * <u>Example:</u><br/><pre>
   *   function MyValidationHandler(field) &#123;
   *     if (field.elm.value.indexOf("x") == -1) &#123;
   *       alert("Error!")
   *       return false;
   *     &#125;
   *     return true;
   *   &#125;
   * </pre>
   * </p>
   *
   * </td></tr>
   * <tr><td valign="top"><b><a name="valErrorHandler">valErrorHandler</a></b></td><td> The name of the event handler function. If this property is present and
   * if the validation of this Process Field fails then this handler is invoked to handle the error and an {@link Exception} object passed to it.
   * The handler should return normally if it handles the error or throw the {@link Exception} object.
   * <p>
   * <u>Example:</u><pre>
   *   function MyErrorHandler(ex) &#123; <br/>&nbsp;&nbsp;&nbsp;&nbsp;if (ex.code == "CANNOT_BE_BLANK") return; <br/>&nbsp;&nbsp;&nbsp;&nbsp;throw ex; <br/>&nbsp;&nbsp;&#125;
   * </pre>
   * </p>
   *
   * </td></tr>
   * </table>
   *
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isProcessField = true;

  this.elm = this.node;

  /**
   * This flag denotes that the value of this field should be set additionally by calling
   * the setValue() method after the process record that this field resides in is cloned.
   * Usually this flag needs to be set for the fields that wrappes one or more <CODE>HTML SELECT</CODE> elements
   *
   * @type boolean
   */
  this.needsValueCloning = null;


  /**
   * Sets the value of this field.
   * @param value The value to set to the field.
   */
  this.setValue = function(value) {
    this.node.value = value;
  };

  /**
   * Returns the value of this field
   */
  this.getValue = function() {
    return this.node.value;
  };

  /** Clears the value of this Process Field. */
  this.clear = function() {
    if (this.node) {
      if (this.node.checked) {
        this.node.checked = false;
      } else {
        this.node.value = "";
      }
    }
  };

  /**
   * Adds the name of this field to the modified fields list
   * of the parent ProcessRecord if the value of this field is
   * different that the initial value.
   */
  this.setModified = function() {
    var r = this.getRecord();
    if (r) r.setModified(this.name,this.getValue() != this.getIValue());
  };

  /** Returns <code>true</code> if this Process Field is modified by the user
   * <code>false</code> otherwise.
   * @type boolean
   */
  this.isModified = function() {
    return this.getValue() == this.getIValue();
  };

  /** @private */
  this.getElement = function() {
    return this.node;
  };

  /** Returns the Process Record that this Process Field resides in
   * @type ProcessRecord
   */
  this.getRecord = function() {
    return PMgr.getEnclosingRecord(this.node);
  };

  /** @private */
  this.valHandler = function() {
    return this.props.valHandler || this.props.customValFunction;
  };

  /** Validates this Process Field with the built-in validation methods that are applicable to the type
   * of the Process Field.
   * @type boolean
   * @see #checkValueIsAvailable
   * @see #checkMaxLength
   * @see #checkValueIsNumber
   * @see #checkHasValidValue
   * @see #runValidationHandler
   */
  this.validate = function() {
    var elm = this.node;
    if (this.boolProp("nonBlank") && this.checkValueIsAvailable(elm) == false) return false;
    if (this.props.maxLength && this.checkMaxLength(elm) == false) return false;
    if (this.props.isNumber && this.checkValueIsNumber(elm) == false) return false;
    if (this.props.validValues && this.checkHasValidValue(elm) == false) return false;
    if (this.valHandler() && this.runValidationHandler() == false) return false;
    return true;
  };

  this.setSubmitPrefix = function(prefix) {
     if (this.selm) {
       if (this.selm.name.startsWith("pf_")) this.selm.name = prefix + this.name;
     } else if (this.elm) {
       if (this.elm.name && this.elm.name.startsWith("pf_")) this.elm.name = prefix + this.name;
     }
  };

  this.appendRequestParams = function(acc) {
    if (this.selm) acc.add(this.selm.name,this.selm.value);
    else if (this.elm) acc.add(this.elm.name,this.elm.value);
  };


  /**
   * If this Process Field is located in a {@link TabPanel} this method instructs the
   * {@link TabManager} to focus to that Tab. Otherwise no action is taken.
   */
  this.focusTab = function() {
    try {
      TabMgr.focusEnclosingTab(this.node);
    } catch(e) {}
  };

  /**
   * Focuses on the specified HTML Element and notifies the user with the specified message.
   * @param elm The HTML Element object to focus
   * @param msgCode The message code
   * @param msgArg1 The message argument 1
   * @param msgArg2 The message argument 2
   * @param msgArg3 The message argument 3
   * @return <code>true</code> if {@link #valErrorHandler} property is defined and the error handler covered the
   * error <code>false</code> otherwise.
   * @type boolean
   */
  this.notify = function(elm, msgCode, msgArg1, msgArg2, msgArg3) {

    var ex = new Exception(msgCode, msgArg1, msgArg2, msgArg3);
    ex.elementToFocus = elm;
    ex.object = this;

    if (this.props.valErrorHandler) {
      var handler = new ErrorHandler(this.props.valErrorHandler,"valErrorHandler");
      try {
        handler.run(ex);
        return true;
      } catch (e) {
        if (!e.alert) {alert(e.message);}
      }
    }

    ex.alert();

    if (this.props.maxLength && elm.createTextRange) {
      var maxLength = this.props.maxLength * 1;
      var range = elm.createTextRange();
      range.moveStart("character", maxLength);
      range.moveEnd("character", elm.value.length + 1);
      range.select();
    } else if (maxLength && elm.setSelectionRange) {
      elm.setSelectionRange(maxLength, elm.value.length);
    } else {
      if (elm.select) elm.select();
    }

    return false;
  };


  /**
   * Runs the Validation Handler Function defined for this Process Field.
   * @return <code>true</code> is the validation is successful <code>false</code> otherwise.
   * @type boolean
   */
  this.runValidationHandler = function() {
   var vhandler = this.valHandler();
    if (vhandler) {
      var handler = new EventHandler(vhandler,"valHandler");
      return handler.run(this);
    }
    return true;
  };

  /**
   * Checks if this Process Field has a value.
   * @param elmToValidate The HTML Element object whose value to be checked
   * @param elmToFocus The HTML Element object to focus when validation fails
   * @return <code>true</code> is the validation is successful <code>false</code> otherwise.
   * @type boolean
   */
  this.checkValueIsAvailable = function(elmToValidate,elmToFocus) {
    var elm = elmToValidate || this.elm;
    if (elm.value != "") return true;
    if (this.notify(elmToFocus||elm, "CANNOT_BE_BLANK") == false) return false;
    return true;
  };

  /**
   * Checks if the value of this Process Field is a valid number.
   * @param elmToValidate The HTML Element object whose value to be checked
   * @param elmToFocus The HTML Element object to focus when validation fails
   * @return <code>true</code> is the validation is successful <code>false</code> otherwise.
   * @type boolean
   */
  this.checkValueIsNumber = function(elmToValidate,elmToFocus) {
    var elm = elmToValidate || this.elm;
    if (elm.value == "") return true;
    if (!isNaN(elm.value)) return true;
    if (this.notify(elmToFocus||elm, "MUST_BE_NUMBER") == false) return false;
    return true;
  };

  /**
   * Checks if this Process Field has a valid value.
   * The valid values need to be specified with {@link #validValues} property prior to callling this method.
   * @param elmToValidate The HTML Element object whose value to be checked
   * @param elmToFocus The HTML Element object to focus when validation fails
   * @return <code>true</code> is the validation is successful <code>false</code> otherwise.
   * @type boolean
   */
  this.checkHasValidValue = function(elmToValidate,elmToFocus) {

    var elm = elmToValidate || this.elm;
    if (elm.value == "") return true;

    var vvals = this.props.validValues.split(";");

    var v = parseFloat(elm.value,10);
    for (var i = 0; i < vvals.length; i++) {
      var val = vvals[i];
        if (val.indexOf(":") > -1) {
        var ft = val.split(":");
        if (ft.length > 1) {
          var v1 = parseFloat(ft[0],10);
          var v2 = parseFloat(ft[1],10);
          if ( v >= v1 && v <= v2 ) {
            return true;
          }
        }
      } else {
        if (val == elm.value) {
          return true;
        }
      }
    }
    if (this.notify(elmToFocus||elm, "INVALID_VALUE", vvals) == false) return false;
    return true;

  };

  /**
   * Runs the handler whose name is stored in this.props object
   * @param handlerPropName The name of the property that is contained in <CODE>this.props</CODE> object
   * @param obj The object to pass to the handler. If omitted <CODE>this</CODE> is passed
   * @returns true, false or an Object depending on the handlers responsibility
   */
  this.runHandler = function(handlerPropName, obj) {
    var handlerName = this.props[handlerPropName];
    if (handlerName) {
      var handler = new EventHandler(handlerName, handlerPropName);
      return handler.run(obj || this);
    }
    return true;
  };


}


/**
 * @class
 * Represents a Process Command.
 * This class wraps an <CODE>INPUT type="botton"</CODE> or an <CODE>INPUT type="hidden"</CODE> element whose name is
 * "__pcommand". The wrapped element must reside in a <CODE>FORM</CODE> that ise wrapped by
 * {@link ProcessForm} class
 * <p><u>Example:</u>
 * <PRE><CODE>
 * &lt;input name="__pcommand" type="button" value="Delete"
 *   onclick="if (PMgr.submitProcessForm('ProcessForm1','ExecuteCommand1' ,'Default' ) == false) return false;"
 *   mo:name="ExecuteCommand1" mo:props="requiredBlocks:'ProcessBlock1', validateInput:true" /&gt;
 * </CODE></PRE>
 * The <CODE>value</CODE> and the <CODE>onclick</CODE> attributes are not applicable if the element
 * type is <CODE>INPUT type="hidden"</CODE>.
 * In that case the element is just a process command definition and <CODE>PMgr.submitProcessForm()</CODE> call
 * needs to be triggered by another DHTML event procedure of another element in the page.
 * <br/>
 * @param node The DOM node of the HTML HIDDEN element that stores the information about this command
 *
 */
function ProcessCommand(node) {


  /**
   * ProcessCommand Properties Object.
   *
   * <br/><br/><table border="1" style="border-collapse: collapse" cellspacing="3">
   * <tr><td><b>requiredBlocks</b></td><td> The names of the blocks (separated with commas) to be
   * validated before submitting the form with this command  </td>
   * <tr><td><b>validateInput</b></td><td>Flag that denotes that Field validations should be executed or not when this
   * command is given</td>
   * <tr><td><b>confirmMessage</b></td><td> The confirmation message text to be prompted to the user when
   * this command is given </td>
   * <tr><td><b>beforeSubmitFunc</b></td><td>The name of the javascript function to be called before the
   * ProcessForm that this command resides in is submitted with this command.
   * The function receives {@link ProcessForm} object and the command name.</td>
   * </table>
   */
  this.props = null;

  this.base = MoreMotionObject;
  this.base(node);
  /** @type boolean */
  this.isProcessCommand = true;

}

/**
 * @class
 * This class wrappes a HTML <code>&lt;INPUT type="text"&gt;</code> element that exists in a {@link ProcessRecord}.
 * <PRE><CODE>
 * &lt;input
 *   name="pf_NAME" type="text" onchange="PMgr.fieldChanged(this);"
 *   mo:type="EditBox" mo:field="true" mo:name="NAME" mo:iValue=""
 *   mo:props="nonBlank:true, isNumber:true, validValues:'1:20'"
 * /&gt;
 * </CODE></PRE> The name attribute of the element should have a <CODE>"pf_"</CODE> prefix indicate
 * that it is a {@link ProcessField}.
 * <br>
 * @extends ProcessField
 */
function EditBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isEditBox = true;

}

/**
 * @class
 * Wrapper class of a HTML <code>&lt;TEXTAREA&gt;</code> Element that exists in a Process Block.
 * <PRE><CODE>
 * &lt;textarea
 *   name="pf_DESC" onchange="PMgr.fieldChanged(this);"
 *   mo:type="EditArea" mo:field="true" mo:name="DESC" mo:iValue=""
 *   mo:props="nonBlank:true, maxLength:255"
 * /&gt;
 * </CODE></PRE> The name attribute of the element should have a <CODE>"pf_"</CODE> prefix indicate
 * that it is a {@link ProcessField}.
 * @extends ProcessField
 */
function EditArea(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isEditArea = true;

  /** The length of the text that can be entered into the wrapped <code>TEXTAREA</code> element.
   * @type int
   */

  /**
   * @param elmToValidate The HTML Element object which is checked for maximum length to validate
   * @param elmToFocus The HTML Element object which is checked for maximum length to focus
   */

  this.checkMaxLength = function(elmToValidate,elmToFocus) {
    var elm = elmToValidate || this.elm;
    if (elm.value == "") return true;
    var maxLength = this.props.maxLength * 1;
    if (elm.value.length <= maxLength) return true;
    if (this.notify(elmToFocus||elm, "MAX_LENGTH_EXCEEDED", maxLength) == false) return false;
    return true;
  };

}

/**
 * @class
 * Wrapper class of a HTML <code>&lt;INPUT type="hidden"&gt;</code> Element that exists in a Process Block.
 * <PRE><CODE>
 * &lt;input
 *   name="pf_ID" type="hidden"
 *   mo:type="HiddenBox" mo:field="true" mo:name="NAME" mo:iValue=""
 * /&gt;
 * </CODE></PRE> The name attribute of the element should have a <CODE>"pf_"</CODE> prefix to indicate
 * that it is a {@link ProcessField}.
 * <br>
 * @extends ProcessField
 */
function HiddenBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isHiddenBox = true;

  this.validate = function() {};

}

/**
 * @class
 * Wrapper class of a HTML <code>&lt;INPUT type="checkbox"&gt;</code> Element that exists in a Process Block.
 * <PRE><CODE>
 * &lt;input
 *   name="pf_RETIRED" type="checkbox" onchange="PMgr.fieldChanged(this);"
 *   mo:type="CheckBox" mo:field="true" mo:name="RETIRED" mo:iValue="false"
 * /&gt;
 * </CODE></PRE>The name attribute of the element should have a <CODE>"pf_"</CODE> prefix indicate
 * that it is a {@link ProcessField}.
 * @extends ProcessField
 */
function CheckBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isCheckBox = true;

  this.validate = function(index) {};

  this.setValue = function(value) {
    if (value.toBool) value = value.toBool();
    this.node.checked = value == true;
  };

  this.setRecordNumber = function(recnum) {
    this.elm.value = recnum;
  };

  this.getValue = function() {
    return "" + this.node.checked;
  };

  this.isModified = function() {
    return this.getIValue() == this.checked + "";
  };

  this.appendRequestParams = function(acc) {
    if (this.elm.checked) {
      acc.add(this.elm.name,this.elm.value);
    }
  };

}

/**
 * @class
 * Wrapper class of an one-line HTML <code>&lt;SELECT&gt;</code> Element that exists in a Process Block.
 * <PRE><CODE>
 * &lt;select
 *   name="pf_CITIES" onchange="PMgr.fieldChanged(this);"
 *   mo:type="ComboBox" mo:field="true" mo:name="CITIES" mo:iValue="2" mo:needsInit="true"
 *   mo:props="nonBlank:true" &gt;
 *   &lt;option value="1"&gt;Option 1&lt;option&gt;
 *   &lt;option value="2"&gt;Option 2&lt;option&gt;
 * &lt;/select&gt;
 * </CODE></PRE> The name attribute of the element should have a <CODE>"pf_"</CODE> prefix indicate
 * that it is a {@link ProcessField}.
 * @extends ProcessField
 */
function ComboBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isComboBox = true;

  this.needsValueCloning = true;

  this.init = function() {
    var iValue = this.getIValue();
    if (iValue) {
      OMgr.setInitialOption(this.node, iValue);
    }
  };

}

/**
 * @class
 * Wrapper class of a multi-line HTML <code>&lt;SELECT&gt;</code> Element that exists in a Process Block.
 * <PRE><CODE>
 * &lt;select
 *   name="pf_CITIES" size="2" onchange="PMgr.fieldChanged(this);"
 *   mo:type="ListBox" mo:field="true" mo:name="CITIES" mo:iValue="2" mo:needsInit="true"
 *   mo:props="nonBlank:true" &gt;
 *   &lt;option value="1"&gt;Option 1&lt;option&gt;
 *   &lt;option value="2"&gt;Option 2&lt;option&gt;
 * &lt;/select&gt;
 * </CODE></PRE> The name attribute of the element should have a <CODE>"pf_"</CODE> prefix indicate
 * that it is a {@link ProcessField}.
 * @extends ProcessField
 */
function ListBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isListBox = true;

  this.needsValueCloning = true;

  this.init = function() {
    var iValue = this.getIValue();
    if (iValue) {
      OMgr.setInitialOption(this.node, iValue);
    }
  }

}


/** @ignore */
function OptionGroup(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @type boolean */
  this.isOptionGroup = true;

  this.validate = function(index) {};

}

/**
 * @class
 * Wrapper class of a DOM Node that contains text data. This class provides methods to manage a
 * read-only text field so that it can be assigned values and hyperlinks dynamically. This class
 * also provides ajaxRefreshHandler interface which means the wrapped text can be refreshed by the
 * {@link MoreMotionAjaxManager#refreshFields} method along with other type of ProcessFields.
 * @extends ProcessField
 * @extends TextNode
 */
function TextBox(node) {

  /** @private */
  this.base = ProcessField;
  this.base(node);
  /** @private */
  this.base2 = TextNode;
  this.base2(node);

  /** @type boolean */
  this.isTextBox = true;

  /** @private */
  this.ajaxRefreshHandler = function(rec,nodeName,attr) {

    if (!nodeName) nodeName = this.name;

    if (!attr) attr = "value";
    if (attr == "value") this.setValue("");
    else if (attr == "href") this.setHref("");

    if (rec) {
      if (!rec.owner.hasField(nodeName)) {
        throw new Exception("RESPONSE_FIELD_MISSING",nodeName);
      }

      if (attr == "value") this._setValue( rec.get(nodeName) );
      else if (attr == "href") this.setHref( rec.get(nodeName) );
    }

  };

}

/** @ignore */
/* For Javascript Docs */
function _ignore() {
  new EditBox();
  new ComboBox();
  new CheckBox();
  new EditArea();
  new HiddenBox();
  new ListBox();
  new OptionBox();
  new TextBox();
  new ProcessField();
}
