/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.cache;

import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import org.moremotion.servlet.SystemConfig;


/**
 * Responsible for registered MoreMotion files change detection.
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: FileChangeDetector.java 51 2008-03-10 11:46:54Z erkan $
 */ 
public class FileChangeDetector implements Runnable { 
  
  private Map fileMap = new HashMap();
  private Map cbeMap = new HashMap();
  private Thread t;
  private boolean stop = false;
  
  private static final String TOUCH_FILE_NAME = "touch-me-for-reload.txt";
  
  private static FileChangeDetector Instance;
  
  /**
   * Creates new FileChangeDetector
   */
  private FileChangeDetector() {
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  /**
   * Returns a single instance of FileChangeDetector class.
   */
  public static synchronized FileChangeDetector getInstance() {
    if (Instance == null) {
      Instance = new FileChangeDetector();
    }
    return Instance;
  }
  
  /**
   * Adds a file.
   * 
   * @param f File to be checked for modification.
   * @param fcn FileChangeNotification object which will be called back when the file has been changed.
   */
  public void addFile(File f, FileChangeNotification fcn) {
    addFile(f, fcn, null);
  }
  
  /**
   * Adds a file.
   * 
   * @param f File to be checked for modification.
   * @param fcn FileChangeNotification object which will be called back when the file has been changed.
   * @param rf Return file. This parameter can be used to group the files so that they are processed if
   * any of the files in the group changes.
   */
  public void addFile(File f, FileChangeNotification fcn, File rf) {   
    
    if (f == null || fcn == null) return;
    
    if (rf == null) rf = f;
    
    synchronized (fileMap) {
      
      fileMap.put(f.getAbsolutePath(), new FileEntry(f));
      
      String cbeKey = fcn.getClass().getName() + "@" + rf.getAbsolutePath();
      CallBackEntry cbe = (CallBackEntry) cbeMap.get(cbeKey);
      if (cbe == null) {
        cbe = new CallBackEntry(fcn,rf);
        cbeMap.put(cbeKey, cbe);
      }
      cbe.addFile(f);
    }
    
  }
  
  public void removeFile(File f, FileChangeNotification fcn, File rf) {
    String cbeKey = fcn.getClass().getName() + "@" + rf.getAbsolutePath();
    CallBackEntry cbe = (CallBackEntry) cbeMap.get(cbeKey);
    synchronized (cbe) {
      cbe.removeFile(f);
    }
  }
  
  public void run() {
    while(!stop) {
      try {
        synchronized (fileMap) {
          HashMap x = new HashMap(fileMap);
          clearFlags();
          Iterator it = x.values().iterator();
          while(it.hasNext()) {
            FileEntry fe = (FileEntry) it.next();
            
            if (fe.isDeleted()) {
              processAssociatedCBEsForDelete(fe.f);
              fe.syncDel();
            
            } else if (fe.isChanged()) {
              processAssociatedCBEsForChange(fe.f);
              fe.syncLM();
            } 
          }
        }
        t.sleep(1000L * SystemConfig.configurationScanIntervalInSeconds());
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    }
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
  private void clearFlags() {
    Iterator it = cbeMap.values().iterator();
    while (it.hasNext()) {
      FileChangeDetector.CallBackEntry cbe =
      (FileChangeDetector.CallBackEntry) it.next();
      cbe.setChangeProcessed(false);
      cbe.setDeleteProcessed(false);
    }
  }
  
  private void processAssociatedCBEsForChange(File f) {
    Iterator it = cbeMap.values().iterator();
    while (it.hasNext()) {
      CallBackEntry cbe = (CallBackEntry)it.next();
      if (cbe.hasFile(f) && !cbe.isChangeProcessed()) {
        cbe.processChange();
      }
    }
  }
  
  private void processAssociatedCBEsForDelete(File f) {
    Iterator it = cbeMap.values().iterator();
    while (it.hasNext()) {
      CallBackEntry cbe = (CallBackEntry)it.next();
      if (cbe.hasFile(f) && !cbe.isDeleteProcessed()) {
        cbe.processDelete();
      }
    }
  }
  
  private class FileEntry {
    public File f;
    public long lm;
    private boolean everExisted = false; 
    
    FileEntry(File f) {
      this.f = f;
      this.lm = f.lastModified();
    }
    
    boolean isChanged() {
      boolean e = f.exists();
      if (e) everExisted = true;
      return everExisted && f.lastModified() > this.lm;
    }
    
    boolean isDeleted() {
      return everExisted && !f.exists();
    }
    
    void syncLM() {
      this.lm = f.lastModified();
    }
    
    void syncDel() {
      this.everExisted = false;
    }
  }
  
  private class CallBackEntry {
    FileChangeNotification fcn;
    File rf;
    Hashtable files;
    boolean changeProcessed = false;
    boolean deleteProcessed = false;
    
    CallBackEntry(FileChangeNotification fcn, File rf) {
      this.fcn = fcn;
      this.rf = rf;
      this.files = new Hashtable();
    }
    
    void addFile(File f) {
      files.put(f.getAbsolutePath(),".");
    }
    
    boolean hasFile(File f) {
      return files.get(f.getAbsolutePath()) != null;
    }
    
    boolean hasFile() {
      return files.size() > 0;
    }
    
    void removeFile(File f) {
      files.remove(f.getAbsolutePath());
    }
    
    void setChangeProcessed(boolean value) {
      changeProcessed = value;
    }
    
    boolean isChangeProcessed() {
      return changeProcessed;
    }
    
    void processChange() {
      changeProcessed = true;
      fcn.doChange(rf);
    }
    
    void setDeleteProcessed(boolean value) {
      deleteProcessed = value;
    }
    
    boolean isDeleteProcessed() {
      return deleteProcessed;
    }
    
    void processDelete() {
      deleteProcessed = true;
      fcn.doDelete(rf);
    }
    
  }

  
}
