/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.cache;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;


/** 
 * Responsible for providing temporary files on specified directory. 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: TempFileManager.java 51 2008-03-10 11:46:54Z erkan $
 */
public class TempFileManager implements Runnable {
  
  private Thread t;
  private boolean stop = false;
  private long unique = 0; 
  private File root = null;
  
  /** Creates new TempFileManager */
  public TempFileManager(File rootFile) {
    root = rootFile;
    if (!root.exists()) {
      root.mkdirs();
    }
    t = new Thread(this);
    t.setDaemon(true);
    t.start();
  }
  
  /**
   * Creates a temporary file on the specified directory and returns it. The temporary file 
   * name will have a prefix "yyyyMMddHHmmss" format in its name which is actually the time that
   * the file will be deleted after that time expires.
   *
   * @param duration The life duration of the file in seconds.
   */
  public File getTempFile(int duration) {
    File f = null;
    synchronized (this) {
      SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
      Date now = new Date();
      long t = now.getTime() + 1000L * duration;
      Date end = new Date();
      end.setTime(t);
      String name = format.format(end) + "_" + ++unique; 
      f = new File(root,name);
      f.deleteOnExit();
    }
    return f;
  }
  
  
  public File getTempFile(String fileName) {
    return new File(root,fileName);  
  }
  
  public void run() {
    while(!stop) {
      try {
        synchronized (this) {
          SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
          Date d = new Date();
          
          String prefix = "" + format.format(d);
          
          TempFilesFilter f = new TempFilesFilter(prefix);
          if (f != null) {
            File[] files = root.listFiles(f);
            if (files != null) {
              for (int i = 0; i < files.length; i++) {
                delete(files[i]);
              }
            }
          }
        }
        t.sleep(1000L * 60 * 5);  // sleep for 5 minutes 
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    }
  }
  
  public void stop() {
    this.stop = true;
  }
  
  public void finalize() {
    stop();
  }
  
  public static boolean delete(File file) {
    
    if (!file.isDirectory()) {
      return file.delete();
    }
    
    File[] files = file.listFiles();
    for (int i = 0; i < files.length; i++) {
      boolean success = delete(files[i]);
      if (!success) return false;
    }
    
    return file.delete();
    
  }  
  
  class TempFilesFilter implements java.io.FileFilter {
    private String prefix = null;
    
    TempFilesFilter(String prefix) {
      this.prefix = prefix;
    }
    
    public boolean accept(File file) {
      String name = file.getName();    
      if (name.length() <= prefix.length()) return false;
      String filePrefix = name.substring(0,prefix.length());
      int result = filePrefix.compareTo(prefix);
      if ( result < 0) {
        return true;
      } else {
        return false;
      }
    }
  }
  
}
