/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ConfigParameter.java
 *
 */

package org.moremotion.config;

import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Encryption;


/**
 * Represents a Configuration Parameter and provides methods to convert the value of the
 * parameter to String, Password String, {@link org.moremotion.evaluator.Expression Expression}, int, double, boolan and char. If the value of this parameter contains
 * embedded MScript functions {@link #resolve resolve()} method can be used to obtain a ConfigParameter object with a resolved value.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ConfigParameter.java 66 2008-03-14 18:06:09Z erkan $
 */
public class ConfigParameter {
  
  private String name;
  private String strValue;
  private Object object;
  
  private boolean hasMScript = false;
  
  private ConfigParameter() {}
  
  public ConfigParameter(String name, String value) {
    this.name = name;
    this.strValue = value;
    if (value == null) return;
    if (value.length() == 0 || value.trim().length() == 0) {
      this.strValue = null;
      return;
    }
    hasMScript = MScriptResolver.hasFunctions(value);
  }
  
  public String toString() {
    return this.strValue;
  }
  
  public boolean isEmpty() {
    return this.strValue == null;
  }
  
  public boolean hasValue() {
    return this.strValue != null;
  }
  
  public boolean hasMScript() {
    return this.hasMScript;
  }
  
  /**
   * If the parameter value does not contain embedded MScript funcions this method simply return this object.
   * If it does, it resolves the MScript functions in the strValue and creates a new copy of this object and returns it
   * @param request The MoreMotionRequest object required to resolve the MScript functions.
   */
  public ConfigParameter resolve(MoreMotionRequest request) throws ConfigParameterException {
    if (!hasMScript) {
      return this;
    }
    ConfigParameter copy = new ConfigParameter();
    copy.name = name;
    copy.strValue = resolvedValue(request);
    copy.object = object;
    return copy;
  }
  
  private String resolvedValue(MoreMotionRequest request) throws ConfigParameterException {
    try {
      return request.resolve(strValue);
    } catch (Exception ex) {
      throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name,strValue},ex);
    }
  }
  
  public String stringValue() throws ConfigParameterException {
    if (strValue == null) {
      throw new ConfigParameterException("MISSING_CONFIGURATION_PARAMETER",new String[]{name},null);
    }
    return strValue;
  }
  
  public String stringValue(String defValue) {
    return strValue == null || strValue.length() == 0 ? defValue : strValue;
  }
  
  
  /*
   * Password Parameter Handling
   */
  private String parsePassword(String str) {
    if (!Encryption.isEncrypted(str,10,52845,11719,133)) {
      return str;
    }
    return Encryption.decrypt(str,10,52845,11719,133);
  }
  
  /**
   * Decrypts and returns the value
   */
  public String passwordValue() throws ConfigParameterException  {
    
    if (object == null) object = parsePassword(strValue);
    
    return (String)object;
  }
  
  /**
   * Decrypts and returns the value. Parameter has no value then defValue is returned.
   * @param defValue The defult value to return in case of the parameter has no value.
   */
  public String passwordValue(String defValue) throws ConfigParameterException  {
    
    if (object != null) {
      return (String)object;
    }
    
    if (strValue == null || strValue.length() == 0) {
      return defValue;
    }     
    
    object = parsePassword(strValue);
    
    return (String)object;
  }
  
  

  /*
   * Expression Parameter Handling
   */
  private Expression parseExpressionObject(String str) {
    return new Expression(str);
  }
  
  public Expression expressionValue() throws ConfigParameterException  {
    
    if (object == null || !(object instanceof Expression)) object = parseExpressionObject(strValue);
    
    return (Expression)object;
  }
  
  public Expression expressionValue(Expression defValue) {
    
    if (object != null && (object instanceof Expression)) {
      return (Expression)object;
    }
    
    if (strValue == null || strValue.length() == 0) {
      return defValue;
    }
    
    object = parseExpressionObject(strValue);
    
    return (Expression)object;
  }
  
  /*
   * Integer Parameter Handling
   */
  private Integer parseIntegerObject(String str) throws ConfigParameterException {
    try {
      return new Integer(str);
    } catch (Exception e) {
      throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name,strValue},e);
    }
  }
  
  /**
   * Returns the value of the parameter as integer. If the value cannot be converted to a integer then
   * ConfigParameterException is raised.
   */
  public int intValue() throws ConfigParameterException  {
    
    if (object == null || !(object instanceof Integer)) object = parseIntegerObject(strValue);
    
    return ((Integer)object).intValue();
  }
  
  
  /**
   * Returns the value of the parameter as integer. If parameter has no value then defValue is returned.
   * If parameter has a vallue but the value cannot be converted to a integer then
   * ConfigParameterException is raised.
   * @param defValue The defult value to return in case of the parameter has no value.
   */
  public int intValue(int defValue) throws ConfigParameterException  {
    
    if (object != null && (object instanceof Integer)) {
       return ((Integer)object).intValue();
    }
    
    if (strValue == null || strValue.length() == 0) {
      return defValue;
    }    
    
    object = parseIntegerObject(strValue);
    
    return ((Integer)object).intValue();
  }
  
  /*
   * Double Parameter Handling
   */
  
  private Double parseDoubleObject(String str) throws ConfigParameterException {
    try {
      return new Double(str);
    } catch (Exception e) {
      throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name,strValue},e);
    }
  }
  
  public double doubleValue() throws ConfigParameterException  {
    
    if (object == null || !(object instanceof Double)) object = parseDoubleObject(strValue);
    
    return ((Double)object).doubleValue();
  }
  
  public double doubleValue(double defValue) throws ConfigParameterException  {
    
    if (object != null && (object instanceof Double)) {
       return ((Double)object).doubleValue();
    }
    
    if (strValue == null || strValue.length() == 0) {
      return defValue;
    }    
    
    object = parseDoubleObject(strValue);
    
    return ((Double)object).doubleValue();
  }
  
  /*
   * Boolean Parameter Handling
   */
  
  private Boolean parseBooleanObject(String str) throws ConfigParameterException {
    try {
      return new Boolean(str);
    } catch (Exception e) {
      throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name,strValue},e);
    }
  }
  
  public boolean booleanValue() throws ConfigParameterException  {
    
    if (object == null || !(object instanceof Boolean)) object = parseBooleanObject(strValue);
    
    return ((Boolean)object).booleanValue();
  }
  
  public boolean booleanValue(boolean defValue) throws ConfigParameterException  {
    
    if (object != null && (object instanceof Boolean)) {
       return ((Boolean)object).booleanValue();
    }
    
    if (strValue == null || strValue.length() == 0) {
      return defValue;
    }
    
    object = parseBooleanObject(strValue);
    
    return ((Boolean)object).booleanValue();
  }
  
  /*
   * Char Parameter Handling
   */
  
  private Char parseCharObject(String str) throws ConfigParameterException {
    try {
      return new Char(str);
    } catch (Exception e) {
      throw new ConfigParameterException("INVALID_CONFIGURATION_PARAMETER_VALUE",new String[]{name,strValue},e);
    }
  }
  
  public char charValue() throws ConfigParameterException  {
    
    if (object == null || !(object instanceof Char)) object = parseCharObject(strValue);
    
    return ((Char)object).charValue();
  }
  
  public char charValue(char defValue) throws ConfigParameterException  {
    
    if (strValue == null || strValue.length() == 0) return defValue;
    if (object == null || !(object instanceof Char)) object = parseCharObject(strValue);
    
    return ((Char)object).charValue();
  }

  public String toString(Object object) {
    return null;
  }
  
  
}

class Char extends Object {
  
  private char value = ' ';
  
  Char(String s) throws Exception {
    if (s != null && s.length() > 0) {
      value = s.charAt(0);
    }
  }
  
  static char parseChar(String s) throws Exception {
    if (s != null && s.length() > 0) {
      return s.charAt(0);
    }
    return ' ';
  }
  
  char charValue() {
    return value;
  }
  
}

