/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Paging.java
 *
 * Created on 25 July 2004 Monday, 18:45
 */

package org.moremotion.datasrc;

import javax.servlet.http.HttpSession;
import java.util.Hashtable;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;


/**
 * Objects of this class are created by the PageGenerator and passed to DataService classes to let them
 * generate only the requested part of their data.
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Paging.java 51 2008-03-10 11:46:54Z erkan $
 */

public class Paging {
  
  private MoreMotionRequest request = null;
  private String adomName = null;
  private Hashtable bag = null;
  private PagingEntry pen = null;
  private int totalNumberOfItems = -1;
  private int itemsPerPage_configValue = -1;
  private int pagesPerBlock_configValue = -1;
  private String bagKey = "";
  
  
  /**
   * Creates a new instance of Paging
   * @param request MoreMotionRequest object
   * @param adomName ADOM Name.
   * @param dsconfig DataSourceConfig object this Paging object linked.
   */
  public Paging(MoreMotionRequest request, String adomName, DataSourceConfig dsconfig) throws ConfigurationException {
    
    this.request = request;
    this.adomName = adomName;
    
    HttpSession session = request.getSession();
    bag = (Hashtable)session.getAttribute("moremotion.datasource.paging.bag");
    if (bag == null)  {
      bag = new Hashtable();
      session.setAttribute("moremotion.datasource.paging.bag", bag);
    }
    
    if (dsconfig != null) {
      itemsPerPage_configValue = dsconfig.getItemsPerPage(request);
      pagesPerBlock_configValue = dsconfig.getPagesPerBlock(request);
    }
    
    bagKey = adomName;
    String pageName = dsconfig.getConfigUnit().getPageName();
    if (pageName != null) {
      bagKey = pageName + "." + adomName;
    }
    
    String penStr = (String)bag.get(bagKey);
    if (penStr == null) {
      pen = new PagingEntry();
      pen.itemsPerPage = itemsPerPage_configValue;
      pen.pagesPerBlock = pagesPerBlock_configValue;
      savePen();
      
    } else {
      pen = new PagingEntry(penStr);
    }
    
  }
  
  /**
   * Sets the current page number.
   */
  public void setCurrentPage(int value) {
    if (value > 0) {
      /*
      if (value != 999999 && value > getPageCount()) {
        value = getPageCount();
      }
      */
      pen.currentPage = value;
      savePen();
    }
  }
  
  /**
   * Returns the current page number
   */
  public int getCurrentPage() {
    return pen.currentPage;
  }
  
  private void savePen() {
    bag.put(bagKey,pen.toString());
  }
  
  /**
   * Sets the items-per-page value. 
   * items-per-page indicates how many item will be included in the page XML for a page.
   */
  public void setItemsPerPage(int value) {
    if (value > 0) {
      
      // Adjust new current page
      if (pen.itemsPerPage > 0) {
        int cp = ((pen.itemsPerPage * (pen.currentPage - 1) + 1) / value) + 1;
        pen.currentPage = cp;
      }
      
      pen.itemsPerPage = value;
      
      savePen();
    }
    
  }
  
  /**
   * Returns the items-per-page value. 
   * items-per-page indicates how many items will be included in the page XML for a page.
   */
  public int getItemsPerPage() {
    return pen.itemsPerPage;
  }
  
  public void setPagesPerBlock(int value) {
    if (value > 0) {
      if (value > getPageCount()) {
        value = getPageCount();
      }
      pen.pagesPerBlock = value;
      savePen();
    }
  }
  
  /**
   * Returns pages-per-block value.
   */
  public int getPagesPerBlock() {
    return pen.pagesPerBlock;
  }
  
  /**
   * Sets the total number of items available in the datasource.
   */
  public void setNumberOfItems(int value) {
    pen.numberOfItems = value;
    if (pen.currentPage > getPageCount() ) {
      pen.currentPage = getPageCount();
      if (pen.currentPage < 1) pen.currentPage = 1;
    }
    
    if (pen.numberOfItems > (pen.currentPage * pen.itemsPerPage)) {
      pen.morePages = true;
      pen.lastPage = false;
    } else {
      pen.morePages = false;
      if (pen.numberOfItems > ((pen.currentPage - 1 ) * pen.itemsPerPage)) {
        pen.lastPage = true;
      } else {
        pen.lastPage = false;
      } 
    }
    
    savePen();
  }
  
  public void setTotalNumberOfItems(int value) {
    totalNumberOfItems = value;
  }
  /**
   * Returns the number of items available in the datasource.
   */
  public int getNumberOfItems() {
    return pen.numberOfItems;
  }
  
  /**
   * Returns the total number of items available in the datasource.
   * If all the items not yet processed then returns -1.
   */
  public int getTotalNumberOfItems() {
    return totalNumberOfItems;
  }
  
  /**
   * Sets the new-page-required flag.
   */
  public void setNeedsRefresh(boolean value) {
    pen.needsRefresh = value;
    savePen();
  }
  
  /**
   * Returns the value of the new-page-required flag.
   */
  public boolean getNeedsRefresh() {
    return pen.needsRefresh;
  }
  
  /**
   * Sets the more-pages flag.
   */
  public void setMorePages(boolean value) {
    pen.morePages = value;
    savePen();
  }
  
  /**
   * Returns the value of the more-pages flag.
   */
  public boolean getMorePages() {
    return pen.morePages;
  }
  
  /**
   * Sets the last-page flag.
   */
  public void setLastPage(boolean value) {
    pen.lastPage = value;
    savePen();
  }
  
  /**
   * Returns the value of the last-page flag.
   */
  public boolean getLastPage() {
    return pen.lastPage;
  }
  
  
  
  /**
   * Returns the total page count. The page count is calcuated by number-of-items / items-per-page formula.
   */
  public int getPageCount() {
    
    if (pen.itemsPerPage == 0) {
      pen.itemsPerPage = itemsPerPage_configValue;
      pen.pagesPerBlock = pagesPerBlock_configValue;
    }
    
    int result = 1;
    try {
      result = pen.numberOfItems / pen.itemsPerPage;
      if (pen.itemsPerPage * result < pen.numberOfItems) {
        result++;
      }
    } catch (Exception e) {}
    return result;
  }
  
  /**
   * Returns how many items to be skipped from the beginning to locate the first item to be included in page XML data.
   */
  public int getSkipAmount() {
    return (pen.currentPage - 1) * pen.itemsPerPage;
  }
  
//////  /**
//////   * Returns paging information as XML string
//////   */
//////  public String xtoXml() {
//////    
//////    int pageCount = getPageCount();
//////    
//////    StringBuffer output = new StringBuffer();
//////    output.append("<paging>");
//////    output.append("<number-of-items>" + pen.numberOfItems + "</number-of-items>");
//////    output.append("<items-per-page>" + pen.itemsPerPage + "</items-per-page>");
//////    output.append("<current-page>" + pen.currentPage + "</current-page>");
//////    output.append("<pages-per-block>" + pen.pagesPerBlock + "</pages-per-block>");
//////    
//////    output.append("<page-numbers>");
//////    
//////    int currentBlock = ( (int) (pen.currentPage - 1) / (pen.pagesPerBlock < 1 ? 10 : pen.pagesPerBlock) ) + 1;
//////    int i = (currentBlock  - 1) *  pen.pagesPerBlock + 1;
//////    int blockLimit = currentBlock * pen.pagesPerBlock;
//////    
//////    for(; i <= pageCount && i <= blockLimit; i++) {
//////      output.append("<page");
//////      if (i == pen.currentPage) {
//////        output.append(" current=\"true\"");
//////      }
//////      output.append(">" + i + "</page>");
//////    }
//////    
//////    output.append("</page-numbers>");
//////    output.append("</paging>");
//////    
//////    return output.toString();
//////  }
  
  /**
   * Adds the paging information of this object to the given ADOM object.
   */
  public void addPagingInfo(ADOM result) {
    
////    int pageCount = getPageCount();
////    
    ADOMNode rootNode = result;
////    rootNode.setNodeValue("paging", "");
////    rootNode.setNodeValue("paging.number-of-items", "" + pen.numberOfItems);
////    rootNode.setNodeValue("paging.items-per-page", "" + pen.itemsPerPage);
////    rootNode.setNodeValue("paging.current-page", "" + pen.currentPage);
////    rootNode.setNodeValue("paging.pages-per-block", "" + pen.pagesPerBlock);
////    rootNode.setNodeValue("paging.last-page", "" + pen.lastPage);
////    rootNode.setNodeValue("paging.more-pages", "" + pen.morePages);
////    
////    ADOMNode pageNumbersNode = rootNode.setNodeValue("paging.page-numbers", "");
////    
////    int currentBlock = ( (int) (pen.currentPage - 1) / (pen.pagesPerBlock < 1 ? 10 : pen.pagesPerBlock) ) + 1;
////    int i = (currentBlock  - 1) *  pen.pagesPerBlock + 1;
////    int blockLimit = currentBlock * pen.pagesPerBlock;

    rootNode.setNodeValue("paging-info", getPagingInfo());
    
////    for(; i <= pageCount && i <= blockLimit; i++) {
////      ADOMNode pageNode = new ADOMNode("page", "" + i);
////      if (i == pen.currentPage) {
////        pageNode.addAttribute("current", "true");
////      }
////      pageNumbersNode.addNode(pageNode);
////    }
    
  }
  
  public String getPagingInfo() {
    
    int currentBlock = ( (int) (pen.currentPage - 1) / (pen.pagesPerBlock < 1 ? 10 : pen.pagesPerBlock) ) + 1;
    int fromPage = (currentBlock  - 1) *  pen.pagesPerBlock + 1;
    int toPage = currentBlock * pen.pagesPerBlock;
    int pageCount = getPageCount();
    if (toPage > pageCount) toPage = pageCount;
    
    StringBuffer buf = new StringBuffer();
    buf.append("cp:"+pen.currentPage);
    buf.append(";noi:"+pen.numberOfItems);
    buf.append(";ipp:"+pen.itemsPerPage);
    buf.append(";ppb:"+pen.pagesPerBlock);
    buf.append(";lp:"+pen.lastPage);
    buf.append(";mp:"+pen.morePages);
    buf.append(";fp:"+fromPage);
    buf.append(";tp:"+toPage);
    
    return buf.toString();
  }
  
  
  private class PagingEntry {
    
    protected int currentPage = 1;
    protected int itemsPerPage = 1;
    protected int pagesPerBlock = 1;
    protected int numberOfItems = 0;
    protected boolean morePages = false;
    protected boolean lastPage = false;
    protected boolean needsRefresh = false;
    
    private PagingEntry() {
    }
    
    private PagingEntry(String penStr) {
      StringParser sp = new StringParser(penStr,';');
      while (!sp.eos()) {
        currentPage = sp.nextItemAsInt(1);
        itemsPerPage = sp.nextItemAsInt(0);
        pagesPerBlock = sp.nextItemAsInt(0);
        numberOfItems = sp.nextItemAsInt(0);
        morePages = sp.nextItemAsBoolean();
        lastPage = sp.nextItemAsBoolean();
        needsRefresh = sp.nextItemAsBoolean();
      }
    }
    
    public String toString() {
      StringBuffer s = new StringBuffer(30);
      s.append(currentPage);
      s.append(';');
      s.append(itemsPerPage);
      s.append(';');
      s.append(pagesPerBlock);
      s.append(';');
      s.append(numberOfItems);
      s.append(';');
      s.append(morePages);
      s.append(';');
      s.append(lastPage);
      s.append(';');
      s.append(needsRefresh);
      return s.toString();
    }
    
    
  }
  
}
