/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ADOMFormatter.java
 *
 */

package org.moremotion.page;

import java.util.Iterator;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.evaluator.Conventions;
import org.moremotion.resource.Resource;
import org.moremotion.resource.ResourceBundle;
import org.moremotion.resource.ResourceException;
import org.moremotion.resource.ResourceManager;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;

/**
 * Formats ADOMs into XML Strings.
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMFormatter.java 149 2008-06-11 13:35:46Z erkan $
 */
public class ADOMFormatter {
  
  private ADOM adom;
  private String rootTagName;
  private MoreMotionRequest request;
  private DataSourceConfig dsconfig = null;
  private PageDataRequest pdr = null;
  private String language = null;
  
  private boolean resolveMScipts = false;
  private boolean resourceADOM = false;
  private StringBuffer buffer = new StringBuffer(100);
  private String resBundleName = null;
  private ResourceBundle resBundle = null;
  private NodeSelector nodeSelector = null;
  
  
  /**
   * Creates a new ADOMFormatter.
   *
   * @param adom The ADOM object to be formatted.
   * @param request MoreMotionRequest object
   * @param pdr PageDataRequest object that specifies the data requested by the current page.
   */
  public ADOMFormatter(ADOM adom, MoreMotionRequest request, PageDataRequest pdr) {
    
    this.pdr = pdr;
    this.adom = adom;
    this.request = request;
    this.rootTagName = pdr.getName();
    
  }
  
  /**
   * Creates a new ADOMFormatter.
   *
   * This constructor is used when there is no PageDataRequest available for the ADOM
   * which means a Forced Data Source.
   *
   * @param adom The ADOM object to be formatted.
   * @param request MoreMotionRequest object
   * @param rootTagName the name of the root tag
   * @param resBundleName The name of the resource bundle
   */
  public ADOMFormatter(ADOM adom, MoreMotionRequest request, String rootTagName, String resBundleName) {
    
    this.adom = adom;
    this.request = request;
    this.rootTagName = rootTagName;
    
    this.resBundleName = resBundleName;
    
  }
  
  private void setResourceBundle() throws ADOMFormatingException {
    
    language = request.getLanguage();
    
    try {
      if (resBundleName != null && resBundleName.length() > 0) {
        resBundle = ResourceManager.getResourceBundle(resBundleName, language);
      }
    } catch (ResourceException re) {
      throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},re);
    }
    
  }
  
  private void prepare() throws ConfigurationException {
    
    this.dsconfig = DataSourceManager.getDataSourceConfig(request, pdr.getDataSourceName(request));
    resBundleName = adom.getResBundleName();
    
    if (dsconfig != null) {
      
      ConfigParameter cp;
      
      cp = dsconfig.getParameter("isResourceDS");
      this.resourceADOM = cp == null ? false : cp.resolve(request).booleanValue(false);
      
      cp = dsconfig.getParameter("resolveMScipts");
      this.resolveMScipts = cp == null ? false : cp.resolve(request).booleanValue(false);
    }
    this.nodeSelector = pdr.getNodeSelector(request);
    
  }
  
  
  /**
   * Formats the ADOM and returns the formatted string
   */
  public String format() throws ADOMFormatingException {
    
    try {
      
      if (pdr != null) {
        prepare();
      }
      
      setResourceBundle();
      
      if (adom.hasXmlExplicit()) {
        
        buffer.append("<" + rootTagName + ">");
        
        appendExtraItems(buffer,true);
        
        if (resBundle != null) {
          String s = replaceResourceMessages(adom.getXmlExplicit());
          buffer.append(s);
          
        } else {
          buffer.append(adom.getXmlExplicit());
        }
        
        appendExtraItems(buffer,true);
        
        buffer.append("</" + rootTagName + ">");
        
        return buffer.toString();
        
      } else if (resourceADOM) {
        return formatResourceADOM();
        
      } else {
        buffer.append("<" + rootTagName + ">");
        
        appendExtraItems(buffer,true);
        
        Iterator it = adom.getNodes();
        while(it.hasNext()) {
          ADOMNode node = (ADOMNode) it.next();
          buffer.append(nodeToXml(node));
        }
        
        appendExtraItems(buffer,false);
        
        buffer.append("</" + rootTagName + ">");
        
        return buffer.toString();
      }
      
    } catch (ConfigurationException e) {
      throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},e);
    }
    
    
  }
  
  private void appendExtraItems(StringBuffer buf, boolean before) throws ConfigParameterException {
    if (pdr == null) return;
    int amount = 0;
    if (before) amount = pdr.getExtraItemsBefore(request);
    else amount = pdr.getExtraItemsAfter(request);
    
    if (amount < 1) return;
    for (int i = 0; i < amount; i++) {
      if (before) buf.append("<item extra=\"before\"/>");
      else buf.append("<item extra=\"after\"/>");
    }
  }
 
  private String nodeToXml(ADOMNode node) throws ADOMFormatingException {
    
    StringBuffer buf = new StringBuffer();
    
    ADOMNode nood;
    try {
      nood = nodeSelector == null || node.getName().equals("item") ? node : nodeSelector.processNode(node);
    } catch (ConfigParameterException e) {
      throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},e);
    }
    
    if (nood == null) {
      return "";
    }
    
    String nodeName = Conventions.safeXmlNodeName( nood.getName());
    
    buf.append("<" + nodeName);
    
    String value = nood.getValueForXML();
    
    // attributes
    if (node.getAttributes() != null) {
      Iterator it = node.getAttributes().keySet().iterator();
      while(it.hasNext()) {
        buf.append(" ");
        String name = (String) it.next();
        
        buf.append(name + "=\"" +
            PageUtils.encodeXMLAttributeValue(
            replaceResourceMessages( node.getAttribute(name) )
            ) + "\""
            );
        
      }
    }
    
    if (value != null && value.startsWith("<![CDATA[")) {
      buf.append(" cdata=\"true\"");
    }
    
    boolean emptyValue = (value == null) || (value.length() == 0);
    
    // return if has no value no children
    Iterator children = node.getNodes();
    if (emptyValue && (children == null || !children.hasNext())) {
      buf.append("/>");
      return buf.toString();
    }
    
    buf.append(">");
    
    // text
    if (!emptyValue) {
      if (resolveMScipts) {
        try {
          value = request.resolve(value);
        } catch (Exception e) {
          throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},e);
        }
      }
      buf.append( PageUtils.encodeXMLElementValue( replaceResourceMessages(value) ) );
    }
    
    // children
    Iterator it = node.getNodes();
    while(it.hasNext()) {
      ADOMNode an = (ADOMNode) it.next();
      buf.append(nodeToXml(an));
    }
    
    buf.append("</" + nodeName + ">");
    return buf.toString();
  }
  
  private String formatResourceADOM() throws ADOMFormatingException {
    
    buffer.append("<" + rootTagName + ">");
    Iterator itr = adom.getNodes();
    
    while (itr.hasNext()) {
      ADOMNode an = (ADOMNode)itr.next();
      if (an.getName().equals("item")) {
        String av = an.getAttribute("id");
        if (av != null) {
          an = new ADOMNode(av,an.getValue());
          
          ADOMNode nan;
          try {
            nan = nodeSelector == null ? an : nodeSelector.processNode(an);
          } catch (ConfigParameterException e) {
            throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},e);
          }
          
          if (nan != null) {
            buffer.append("<" + nan.getName() + ">");
            buffer.append( PageUtils.encodeXMLElementValue( nan.getValue() ) );
            buffer.append("</" + nan.getName() + ">");
          }
        }
      }
      
    }
    
    buffer.append("</" + rootTagName + ">");
    
    return buffer.toString();
  }
  
  
  private String replaceResourceMessages(String str) throws ADOMFormatingException {
    
    if (str == null) {
      return "";
    }
    
    if (resBundleName == null) return str;
    int start = str.lastIndexOf(Resource.RES);
    if (start == -1) {
      return str;
    }
    int end = 0;
    while(start != -1) {
      
      end = StringParser.locateChar(str,start + Resource.RES.length(), ')');
      
      if (end < 0) {
        String def;
        if (str.length() - start > 100)  {
          def = str.substring(start, start + 100);
        } else {
          def = str.substring(start);
        }
        
        throw new ADOMFormatingException("INVALID_RES_FUNCTION:MISSING_RIGTH_PARANTHESIS", new String[]{def},null);
      }
      end++;
      String resmsg = str.substring(start,end);
      
      Resource msg;
      try {
        msg = Resource.parse(resmsg);
      } catch (ResourceException re) {
        throw new ADOMFormatingException(re);
      }
      
      if (!msg.bundleAssigned()) {
        msg.setBundleName(resBundle.getName());
      }
      String result = null;
      
      try {
        result = msg.format(this.language);
      } catch (ResourceException re) {
        throw new ADOMFormatingException("ADOM_FORMATTING_ERROR",new String[]{rootTagName},re);
      }
      
      str = str.substring(0,start) + result + str.substring(end);
      
      start = str.lastIndexOf(Resource.RES, --start);
      end = 0;
    }
    
    return str;
  }
  
  
  
  
}