/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ExecutionManager.java
 *
 */
package org.moremotion.process;

import java.util.ArrayList;
import java.util.HashMap;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.evaluator.EvaluationException;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.SymbolResolvingException;
import org.moremotion.security.AccessControlDefinition;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;
import org.moremotion.security.SecurityManager;

/**
 * Responsible to execute the process templates
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ExecutionManager.java 157 2008-06-28 11:36:38Z erkan $
 */
public class ExecutionManager {

  private MoreMotionRequest request;
  private ProcessTemplateCallDefinition callDef;
  private MMSymbolResolver mmsr;
  private Logger logger;

  /**
   * Creates a new instance of ExecutionManager
   */
  private ExecutionManager(MoreMotionRequest request) {
    this.request = request;
    this.mmsr = request.getSymbolResolver();
  }

  /**
   * Creates and returns a Execution Manager for the specified process template.
   * @param templateName The name of the process template that exist is the process configuration
   * @param callerPage The name of the caller page. Required to resolve relative references to processes and templates.
   * @param request MoreMotionRequest object
   */
  public static ExecutionManager getExecutionManager(String templateName, String callerPage, MoreMotionRequest request, int debugLevel) throws ProcessManagingException {

    try {
      ExecutionManager em = new ExecutionManager(request);

      ConfigUnit unit = null;

      if (templateName.length() > 0 && templateName.charAt(0) != '/' && callerPage != null) {
        unit = ConfigCache.getUnit("processTemplate", callerPage + '.' + templateName);
      }

      if (unit == null) {
        unit = ConfigCache.getUnit("processTemplate", templateName);
      }

      if (unit == null) {
        throw new ProcessManagingException("PROCESS_TEMPLATE_MISSING", new String[]{templateName}, null);
      }
      
      if (unit.isNew()) {
        synchronized (unit.getSyncObject()) {
          new ProcessTemplateConfig(unit);
        }
      }

      ProcessTemplateConfig ptc = (ProcessTemplateConfig) unit.getDataObject();

      em.callDef = new ProcessTemplateCallDefinition(null,null);
      em.callDef.setTemplateConfig(ptc);
      em.logger = new Logger(templateName, debugLevel);
      em.logger.setResourceBundleName("moremotion");

      return em;
      
    } catch (ConfigurationException ex) {
      throw new ProcessManagingException(ex);
    }
  }

  private void flushCompletionCode(CallInstance ci) {
    String stepName = ci.callDef.getPureStepName();
    if (stepName != null) {
      request.getVPool().put("_" + stepName + "_cc", "" + ci.completionCode);
    }
  }

  public int execute() throws ProcessManagingException {
    CallInstance ci = null;
    try {
      ci = new CallInstance(null, callDef, request);
    } catch (ConfigParameterException ex) {
      createMessage(ex,ci);
      throw new ProcessManagingException(ex);
    }
    ci.logger = logger;
    mmsr.setProcessCallInstance(ci);

    long t1 = System.currentTimeMillis();
    executeCallInstance(ci, null);
    finalize(ci,99,true);
    long t2 = System.currentTimeMillis();

    mmsr.setProcessCallInstance(null);

    if (logger.LEVEL1) {
      logger.printres("TEMPLATE_EXECUTION_COMPLETED", new String[]{callDef.getTargetName(), (t2 - t1) + "", ci.completionCode + ""});
    }

    return ci.completionCode;
  }

  public void createMessage(Exception ex, CallInstance ci) {
    if (ci.process != null) {
      ci.process.createMessage(ex);
    } else {
      request.createMessage(ex);
    }
  }

  private HashMap checkedTemplates = null;
  
  private void checkTemplateAccess(ProcessTemplateConfig ptconfig) throws ProcessManagingException {
    
    if (request.getSession().getAttribute("temporary.loggedin") != null) return;

    if (checkedTemplates == null) {
      checkedTemplates = new HashMap();
    } else {
      if (checkedTemplates.get(ptconfig) != null) return;
    }

    AccessControlDefinition acd;
    try {
      acd = ptconfig.getAccessControlDefinition(request);
      if (acd != null) {
        boolean hasAccess = false;
        try {
          hasAccess = SecurityManager.checkAccessAuthority(request, acd);
        } catch (Exception e) {
          throw new ProcessManagingException(e);
        }
        if (!hasAccess) {
          throw new ProcessManagingException("NOT_AUTHORIZED_TO_ACCESS", new String[]{acd.getCheckPoint()}, null);
        }
      }
    } catch (ConfigParameterException ex) {
      throw new ProcessManagingException(ex);
    }

    checkedTemplates.put(ptconfig,"*");
  }

  private void setProcessBlock(CallInstance ci) throws ProcessManagingException {

    String blockName = null;
    try {
      blockName = ci.callDef.getBlockName(request);
    } catch (ConfigParameterException ex) {
      throw new ProcessManagingException("PROCESS_BLOCK_ERROR", new String[]{blockName}, ex);
    }

    ProcessBlock pb = null;

    if (blockName == null || blockName.length() == 0) {
      if (ci.level < 2) {
        if (logger.LEVEL2) {
          logger.printres("USING_DUMMY_BLOCK", ci.callDef.getStepName());
        }
        pb = ProcessBlockPool.getDummyBlock();
      }

    } else {
      pb = ProcessBlockPool.getPool(request).getBlock(blockName);
      if (logger.LEVEL2) {
        logger.printres("USING_PROCESS_BLOCK", ci.callDef.getStepName(), blockName, pb.getRecordCount() + "");
      }
    }

    ci.blockName = blockName;
    if (pb != null) {
      ci.processBlock = pb;
      ci.processRecord = new ProcessRecord(pb);
    }

    ci.recordFilter = ci.callDef.getRecordFilter();

    if (ci.recordFilter != null && logger.LEVEL2) {
      logger.printres("RECORD_FILTER", ci.callDef.getStepName(), ci.recordFilter + "");
    }

  }

  private boolean precondSatisfied(CallDefinition callDef) throws ConditionEvaluationException {

    boolean satisfied = false;
    try {
      satisfied = callDef.precondSatisfied(request);
    } catch (ConfigParameterException ex) {
      throw new ConditionEvaluationException(ex);
    }

    if (logger.LEVEL3) {
      if (callDef.getCondition() != null) {
        String s0 = callDef.getCondition().toString();
        String s1 = callDef.getStepName();
        if (satisfied) {
          if (!callDef.getCondition().toString().equals("_maxlcc=0")) {
            logger.printres("PRECONDITION_EVAL_RESULT_TRUE", s0, s1);
          }
        } else {
          logger.printres("PRECONDITION_EVAL_RESULT_FALSE", s0, s1);
        }
      }
    }

    return satisfied;
  }

  private boolean isThroughRecordFilter(CallInstance ci, int recnum) throws ConditionEvaluationException {

    try {
      if (ci.recordFilter == null || ci.recordFilter.evalBool(mmsr)) {
        if (logger.LEVEL3) {
          if (ci.recordFilter != null) {
            logger.printres("CALLING_FOR_RECORD_THROUGH", ci.callDef.getStepName(), recnum + "");
          } else {
            logger.printres("CALLING_FOR_RECORD", ci.callDef.getStepName(), recnum + "");
          }
        }
        return true;
      }

      return false;

    } catch (Exception ex) {
      throw new ConditionEvaluationException("ERROR_WHEN_TESTING_CONDITION", new String[]{ci.recordFilter.toString()}, ex);
    }

  }

  private void callInstanceForRecord(CallInstance ci, ProcessRecord prec) throws ProcessException, ConfigParameterException, ProcessManagingException, EvaluationException, ConditionEvaluationException {

    if (ci.callDef instanceof ProcessCallDefinition) {
      processRecord(ci, prec);

    } else if (ci.callDef instanceof QueryProcessCallDefinition) {
      logger.incrIndent();
      processQueryRecord(ci, prec);
      logger.decrIndent();

    } else {
      logger.incrIndent();
      processTemplateRecord(ci, prec);
      logger.decrIndent();

    }

  }

  private void executeCallInstance(CallInstance ci, ProcessRecord ownerRecord) throws ProcessManagingException {

    if (!ci.prepared) {
      setProcessBlock(ci);
      prepareCallList(ci);
      if (!(ci.callDef instanceof ProcessTemplateCallDefinition)) {
        prepareProcess(ci, logger.getInnerIndent());
      }
      ci.prepared = true;
    }

    try {

      ProcessBlock pb = ci.processBlock;

      if (ci.blockName == null) {

        if (isThroughRecordFilter(ci, 0)) {
          ProcessRecord prec = ownerRecord != null ? ownerRecord : ci.processRecord;
          callInstanceForRecord(ci, prec);
        }

      } else {

        ProcessRecord prec = ci.processRecord;

        for (int i = 0; i < pb.getRecordCount() && ci.exception == null; i++) {

          prec.setIndex(i);
          mmsr.pushProcessRecord(prec);

          if (isThroughRecordFilter(ci, i)) {
            callInstanceForRecord(ci, prec);
          }

          mmsr.popProcessRecord();
          
          finalizeChildren(ci,0);
        }

      }

      finalize(ci,0, false);

    } catch (Exception ex) {
      if (ex instanceof ProcessManagingException) {
        createMessage(ex, ci);
        throw (ProcessManagingException) ex;
      }
      if (ex instanceof ConditionEvaluationException) {
        createMessage(ex, ci);
        throw new ProcessManagingException(ex);
      }
      ex.printStackTrace();
      throw new ProcessManagingException("PROCESS_EXECUTION_ERROR", new String[]{ci.process.getClass().getName()}, ex);
    }

  }

  private void processRecord(CallInstance ci, ProcessRecord prec) throws ProcessException {

    try {
      ci.process.processRecord(prec);

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
    }

  }

  private void processQueryRecord(CallInstance ci, ProcessRecord prec) throws ConfigParameterException, ProcessManagingException, EvaluationException, ConditionEvaluationException {

    QueryProcess process = (QueryProcess) ci.process;

    try {
      process.processRecord(prec);

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
      return;
    }

    if (ci.children.length == 0) {
      /*
       * There is no child process. If outputTo is specified,
       * create a MemoryProcessBlock and fill it.
       *
       * Note that if query process has more than one input records then
       * this will also be called more than once. Only by the
       * first call a new MemoryProcessBlock will be created and
       */
      prepareOutputTo(ci, process);

      return;
    }

    MemoryProcessBlock pb = null;
    ProcessRecord subrec = null;

    MemoryProcessRecord record;
    try {
      record = process.getNextResultRecord();

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
      return;
    }

    int count = 0;

    while (record != null && ci.exception == null) {

      count++;

      if (pb == null) {

        String[] fieldNames;
        try {
          fieldNames = process.getFieldNames();

        } catch (ProcessException be) {
          ci.exception = be;
          ci.setCompletionCode(20);
          createMessage(be, ci);
          return;
        }

        if (logger.LEVEL2) {
          logger.printres("RETRIEVING_OUTPUT_RECORDS", ci.callDef.getStepName());
          logger.printres("FIELD_NAMES", printFieldNames(fieldNames));
        }

        pb = new MemoryProcessBlock(fieldNames);
        subrec = new ProcessRecord(pb);
        subrec.setIndex(0);
      }

      mmsr.pushProcessRecord(subrec);

      pb.clearRecords();
      pb.addRecord(record);

      for (int i = 0; i < ci.children.length && ci.exception == null; i++) {

        CallInstance child = ci.children[i];

        if (precondSatisfied(child.callDef)) {
          executeCallInstance(child, subrec);
        }

      }

      mmsr.popProcessRecord();


      if (ci.exception == null) {
        try {
          record = process.getNextResultRecord();

        } catch (ProcessException be) {
          ci.exception = be;
          ci.setCompletionCode(20);
          createMessage(be, ci);
        }

      }

    }

  }

  private void processTemplateRecord(CallInstance ci, ProcessRecord prec) throws ConfigParameterException, ProcessManagingException, EvaluationException, ConditionEvaluationException {

    mmsr.pushTCall(((ProcessTemplateCallDefinition) ci.callDef).getTemplateCallParams());

    for (int i = 0; i < ci.children.length && ci.exception == null; i++) {

      CallInstance child = ci.children[i];

      if (precondSatisfied(child.callDef)) {
        mmsr.setProcessCallInstance(child);
        executeCallInstance(child, prec);
        mmsr.setProcessCallInstance(ci);
      }

    }

    mmsr.popTCall();

  }

  private void prepareOutputTo(CallInstance ci, QueryProcess process) throws ConfigParameterException, ProcessManagingException {

    String outputTo = ((QueryProcessCallDefinition) ci.callDef).getOutputTo(request);
    if (outputTo == null) {
      return;
    }

    MemoryProcessBlock pb = null;
    ADOM adom = null;

    String targetName = null;
    int targetType = -1;

    if (outputTo.startsWith("ADOM:")) {
      targetName = outputTo.substring(5);
      targetType = 2;

    } else if (outputTo.indexOf(':') == -1) {
      targetName = outputTo;
      targetType = 1;

    } else {
      targetName = outputTo;
      targetType = 3;
    }

    String[] fieldNames;

    try {
      fieldNames = process.getFieldNames();

      if (fieldNames == null) {
        return;
      }

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
      return;
    }

    if (logger.LEVEL2) {

      if (targetType == 2) {
        logger.printres("CREATING_REQUEST_ADOM_WITH_FIELDS", printFieldNames(fieldNames));
      } else if (targetType == 1) {
        logger.printres("CREATING_MEMORYPROCESSBLOCK_WITH_FIELDS", printFieldNames(fieldNames));
      } else if (targetType == 3) {
        logger.printres("CREATING_XMLSTRING_WITH_FIELDS", printFieldNames(fieldNames));
      }

    }

    if (targetType == 1) {

      pb = new MemoryProcessBlock(fieldNames);
      ci.processBlock = pb;
      ProcessBlockPool.getPool(request).addBlock(targetName, pb);

    } else {
      adom = request.newRequestADOM(targetName);

    }

    int count = 0;
    MemoryProcessRecord record;

    try {
      record = process.getNextResultRecord();

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
      return;
    }

    while (record != null) {

      if (targetType == 1) {
        pb.addRecord(record);

      } else {
        addRecordToADOM(adom, fieldNames, record.fields);

      }

      count++;

      try {
        record = process.getNextResultRecord();

      } catch (ProcessException be) {
        ci.exception = be;
        ci.setCompletionCode(20);
        createMessage(be, ci);
        return;
      }

    }

    if (targetType == 2 && logger.LEVEL2) {
      logger.printres("NUMBER_OF_ADDED_RECORDS_ADOM", count + "");
    } else if (targetType == 1 && logger.LEVEL2) {
      logger.printres("NUMBER_OF_ADDED_RECORDS_MEMORYPROCESSBLOCK", count + "");
    } else if (targetType == 3) {

      String xml = adom.toXml(null);

      if (logger.LEVEL3) {
        logger.printres("XML_SET_TO_SYMBOL_WITH_VALUE", targetName, xml);
      } else if (logger.LEVEL2) {
        logger.printres("XML_SET_TO_SYMBOL", targetName);
      }

      try {
        request.getSymbolResolver().setValue(targetName, xml);

      } catch (SymbolResolvingException ex) {
        throw new ProcessManagingException(ex);
      }

    }

  }

  private void addRecordToADOM(ADOM adom, String[] fieldNames, String[] record) throws ProcessManagingException {

    if (fieldNames.length != record.length) {
      throw new ProcessManagingException("INVALID_PROCESS_RECORD_ADDING", null, null);
    }

    ADOMNode item = new ADOMNode("item");
    adom.addNode(item);
    for (int i = 0; i < fieldNames.length; i++) {
      item.setNodeValue(fieldNames[i], record[i]);
    }

  }

  private String printFieldNames(String[] fieldNames) {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < fieldNames.length; i++) {
      if (buf.length() > 0) {
        buf.append(',');
      }
      buf.append(fieldNames[i]);
    }
    return buf.toString();
  }

  private HashMap cachedConfigs = null;
  
  private void prepareCallList(CallInstance ci) throws ProcessManagingException {
  
    if (ci.children == null) {    
    
      ConditionalList callList = null;

      if ( ci.callDef instanceof QueryProcessCallDefinition ) {
        callList = ((QueryProcessCallDefinition)ci.callDef).getCallList();

      } else if ( ci.callDef instanceof ProcessTemplateCallDefinition ) {
        
//        CallInstance xci = ci.parent;
//        while (xci != null) {
//          if (xci.callDef == ci.callDef) {
//            ProcessTemplateCallDefinition ptcdef = (ProcessTemplateCallDefinition) ci.callDef;
//            try {
//              if (ptcdef.getReset(request) == null) {
//                callList = ptcdef.getTemplateConfig().callList;
//                break;
//              }
//            } catch (ConfigParameterException ex) {
//              throw new ProcessManagingException(ex);
//            }
//          }
//          xci = xci.parent;  
//        }
        
        if (callList == null) {
          
          try {
            if (cachedConfigs != null) {
              ProcessTemplateConfig config = (ProcessTemplateConfig)cachedConfigs.get(ci.callDef);
              if (config != null) {
                callList = config.callList;  
              }
            }
            if (callList == null) {
              ProcessTemplateConfig ptconfig = ((ProcessTemplateCallDefinition)ci.callDef).getTemplateConfig(request);
              checkTemplateAccess(ptconfig);
              callList = ptconfig.callList;
              if (ci.resetDepth != 0) {
                if (cachedConfigs == null) cachedConfigs = new HashMap();
                cachedConfigs.put(ci.callDef,ptconfig);
              }
            }

          } catch (ConfigurationException ex) {
            createMessage(ex, ci);
            throw new ProcessManagingException(ex);
          }
        }
      }    

      if (callList == null) {
        ci.children = new CallInstance[0];

      } else {

        ArrayList list = callList.getList();
        ArrayList children = new ArrayList();

        try {
          for (int i = 0; i < list.size(); i++) {
            CallInstance child = new CallInstance(ci, (CallDefinition) list.get(i), request);
            children.add(child);
          }
        } catch (ConfigParameterException ex) {
          createMessage(ex,ci);
          throw new ProcessManagingException(ex);
        }

        ci.children = new CallInstance[children.size()];
        children.toArray(ci.children);
      }    
    }
    
  }
  
  private void prepareProcess(CallInstance ci, int indent) throws ProcessManagingException {

    if (ci.process == null) {

      if (logger.LEVEL3) {
        logger.printres("PREPARING_PROCESS", ci.callDef.getStepName());
      }

      String className = ci.callDef.getProcessConfig().getClassName();
      Process process;
      try {
        process = (Process) Class.forName(className).newInstance();
      } catch (Exception ex) {
        ci.exception = ex;
        throw new ProcessManagingException("UNABLE_TO_LOAD_PROCESS_CLASS", new String[]{className}, ex);
      }

      /*
       * Initialize the process
       */
      process.request = request;
      process.callInstance = ci;
      process.callDef = ci.callDef;
      process.processConfig = ci.callDef.getProcessConfig();

      process.logger = new Logger(ci.callDef.getStepName());
      try {
        process.logger.setLevel(ci.callDef.getDebugLevel(request));
        process.logger.setResourceBundleName(ci.callDef.getProcessConfig().getUnitName());
      } catch (ConfigParameterException ex) {
        throw new ProcessManagingException(ex);
      }
      process.logger.setIndent(indent);

      ci.process = process;
    }
    

    /*
     * Call its prepare() method
     */
    try {
      ci.process.prepare();
      
    } catch (ProcessException ex) {
      createMessage(ex, ci);
      throw new ProcessManagingException("PROCESS_PREPARE_ERROR", new String[]{ci.callDef.getProcessConfig().getClassName()}, ex);
    }

  }

//  private void resetCallList(CallInstance ci) {
//    if (ci.children != null && ci.callDef instanceof ProcessTemplateCallDefinition) {
//      if (((ProcessTemplateCallDefinition)ci.callDef).isTargetDynamic()) {
//        ci.children = null;
//      }
//    }
//  }
  
  
  private void finalizeChildren(CallInstance ci, int depth) throws ProcessManagingException {
    
    if (ci.children != null) {
      for (int i = 0; i < ci.children.length; i++) {
        finalize(ci.children[i], depth+1, false);
      }
    }
    
  }
  
  
  private void finalize(CallInstance ci, int depth, boolean force) throws ProcessManagingException {

    boolean reset = ci.prepared && ci.resetDepth <= depth;
    if (reset) {
      force = reset;
    }
        
    if (ci.children != null) {
      for (int i = 0; i < ci.children.length; i++) {
        finalize(ci.children[i], depth+1, force || reset);
      }
    }

    try {

      if (reset || (ci.prepared && force && ci.resetDepth == 200)) {
        if (ci.process != null) {
          if (logger.LEVEL2) {
            ci.getLogger(logger).printres("NOW_FINALIZING", ci.callDef.getStepName());
          }
          ci.process.finalizeProcess();
        }
        ci.prepared = false;
        
        if (ci.children != null && ci.callDef instanceof ProcessTemplateCallDefinition) {
          if (((ProcessTemplateCallDefinition)ci.callDef).isTargetDynamic()) {
            ci.children = null;
          }
        }
      }

      flushCompletionCode(ci);

    } catch (ProcessException be) {
      ci.exception = be;
      ci.setCompletionCode(20);
      createMessage(be, ci);
    }

  }
}
