/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MemoryProcessBlock.java
 *
 * Created on October 16, 2003, 7:47 PM
 */

package org.moremotion.process;

import java.util.HashMap;
import java.util.ArrayList;

/**
 * Represents a Process Block that includes one or more {@link org.moremotion.process.ProcessRecord Process Records}.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MemoryProcessBlock.java 63 2008-03-14 16:29:03Z erkan $
 */
public class MemoryProcessBlock implements ProcessBlock {
  
  private String[] fieldNames;
  private ArrayList records = new ArrayList();
  private HashMap customFields;
  private HashMap objects;
  
  /**
   * Creates new FormProcessBlock.
   */
  protected MemoryProcessBlock(String[] fieldNames) {
    this.fieldNames = fieldNames;
    records = new ArrayList();
  }
  
  /**
   * Adds a process record to this process block.
   * @param record {@link MemoryProcessRecord} object to add to this block
   */
  public void addRecord(MemoryProcessRecord record) throws ProcessManagingException {
    if (fieldNames.length != record.fields.length) {
      throw new ProcessManagingException("INVALID_PROCESS_RECORD_ADDING",null,null);
    }
    records.add(record);
  }
  
 
  /**
   * Clears all the process records of this process block.
   */
  public void clearRecords() {
    records.clear();
    customFields = null;
    objects = null;
  }
  
  /**
   * Returns the number of process records available in this block.
   */
  public int getRecordCount() {
    return records.size();
  }
  
  
  /**
   * Returns the process record that has the specified index number.
   */
  public ProcessRecord getRecord(int index) {
    return (ProcessRecord) records.get(index);
  }
  
  /**
   * Returns the names of the block fields.
   */
  public String[] getFieldNames() {
    return fieldNames;
  }
  
  private int getFieldIndex(String name) {
    for (int i = 0; i < fieldNames.length; i++) {
      if (name.equals(fieldNames[i])) {
        return i;
      }
    }
    return -1;
  }
  
  public String getField(String fieldName, int index) {
    
    if (fieldName == null) return null;
    
    MemoryProcessRecord record = (MemoryProcessRecord) records.get(index);
    
    if (fieldName.equals("_selected")) {
      return record.status.isSelected() ? "true" : "false";
    }
    
    if (fieldName.equals("_modified")) {
      return record.status.isModified() ? "true" : "false";
    }
    
    if (fieldName.equals("_modified_fields")) {
      return record.status.getModifiedFieldsAsString();
    }
    
    if (customFields != null) {
      String key = fieldName + " " + index;
      String s = (String)customFields.get(key);
      if (s != null) return s;
    }
    
    int fix = getFieldIndex(fieldName);
    
    if (fix > -1) {
      return record.fields[fix];
    }
    
    return null;
  }
  
  public boolean isSelected(int index) {
    MemoryProcessRecord record = (MemoryProcessRecord) records.get(index);
    return record.status.isSelected();
  }
  
  public boolean isModified(int index) {
    MemoryProcessRecord record = (MemoryProcessRecord) records.get(index);
    return record.status.isModified();
  }
  
  public boolean isFieldModified(String fieldName, int index) {
    MemoryProcessRecord record = (MemoryProcessRecord) records.get(index);
    return record.status.isFieldModified(fieldName);
  }
  
  public void setField(String fieldName, String value, int index) {
    String key = fieldName + " " + index;
    if (customFields == null) {
      customFields = new HashMap();
    }
    customFields.put(key, value);
  }
  
  public Object getObject(String fieldName, int index) {
    String key = fieldName + " " + index;
    return objects.get(key);
  }
  
  public void setObject(String fieldName, Object object, int index) {
    String key = fieldName + " " + index;
    if (objects == null) objects = new HashMap();
    objects.put(key,object);
  }
  
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("Memory PB");
    buf.append("; record Q: " + (records == null ? "null" : "" + records.size() ) );
    buf.append("; field Names : [");
    if (fieldNames != null) {
      for (int i = 0; i < fieldNames.length; i++) {
        if (i > 0) buf.append(',');
        buf.append(fieldNames[i]);
      }
    }
    buf.append("]");
    
    return buf.toString();
  }
  
}
