/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.process;

import org.moremotion.adom.ADOMNode;
import org.moremotion.evaluator.MScriptException;
import org.moremotion.exception.MoreMotionException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.Logger;

/**
 * 
 * <P>The process manager will call the interface methods of this abstract class as in the following example.
 * <PRE>
 * pro.prepare(pbc);
 * for (int i = 0; i &lt; pb.recordCount(); i++) {
 *   ProcessRecord prec = pb.getRecord(i);
 *   if (isThroughRecordFilter(prec)) {
 *     <b>pro.processRecord(prec)</b>;
 *   }
 * }
 * int cc = <b>pro.finalizeProcess()</b>;
 * </PRE>
 * Please note that the process classes that extends this interface can have private data members safely
 * since the process manager will create a new instance of them before calling their methods.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Process.java 67 2008-03-17 07:03:46Z erkan $
 */

abstract public class Process {
  
  protected MoreMotionRequest request;
  protected ProcessConfig processConfig;
  protected CallDefinition callDef;
  protected CallInstance callInstance;
  
  protected Logger logger;
  
  /**
   * The Process should prepare to be called for each process records existing 
   * in the configured process block. The preparation can be accessing to the process configuration,
   * establishing any necessary database connection and store the results of this preparation in private data members.
   * 
   * @throws ProcessException indicates that the preparation failed and means 
   * other methods of this process instance will not be called and the overall completion code of the 
   * process execution will be set to 20.
   */
  abstract public void prepare() throws ProcessException;
  
  
  /**
   * This method is called by the process manager for each process record in the configured process block that
   * passes through the configured record filter. For example if the call definition of the process is as follows <PRE>
   * &lt;callProcess name="x" blockName="PBx" recordFilter="PRICE &lt; 100" ... </PRE> then the process manager will 
   * call the processRecord() method for each record in the process block "PBx" that satisfies the "PRICE &lt; 100"
   * pre-condition.
   * 
   * @param prec The ProcessRecord Object that contains process fields 
   * @throws ProcessException indicates that there is an error and 
   * the process cannot continue. This exception will set the overall completion code to 20.
   */ 
  abstract public void processRecord(ProcessRecord prec) throws ProcessException;
  
  /**
   * This method is called by the process manager after there is no more records to process.
   * This is the last call to the process instance and therefore 
   * the process should finalize its tasks.
   * <p>Note that this method is called regardless of the completion code 
   * if the prepare method did not throw an exception.
   * <p>
   * The method should take care of the current completion code and act accordingly
   * <PRE>
   *  public void finalizeProcess() throws ProcessException {
   *    if (getCompletionCode() <= 4) {
   *      // successfull completion
   *    } else {
   *      // unsuccessfull completion
   *    }
   *   }
   * </PRE> 
   * 
   * @throws ProcessException that indicates that finalization was in an unrecoverable error. 
   * This exception will set the completion code to 20.
   */
  abstract public void finalizeProcess() throws ProcessException;  

  
  /**
   * Resolves the MScript functions existing in the given string and returns the resolved string.
   */
  public String resolve(String str) throws MScriptException {
    return request.resolve(str);
  }
  
  /**
   * Sets the completion code of the current step.
   * 
   * @param cc the completion code
   * 
   * <P>The value 0, which is the default, means everything was fine . A value between 1 and 4 means 
   * there were warnings but that's OK. A value between 5 and 8 means there were errors which should be 
   * interpretted as user errors and finally a value greater than 8 means process error.
   * 
   * <P>It should be kept in mind that the process manager will not change the execution plan 
   * depending on the process completion codes. This is valid even if the process throws a
   * ProcessException. in that case the process manager will catch the exception and set the 
   * completion code of the step to 20.
   * 
   * <P>The process manager will attempt to execute all the processes in the processTemplate as long as the 
   * execution precondition (the expression defined in the "if" attribute) of the invocation elements, 
   * such as callProcess, callQueryProcess and callTemplate is satisfied.
   * 
   * <P>Once the execution of process classes completed, the process manager decides what to do;
   * If the maximum completion code is between 0 and 4, this means
   * "success", the PM displays the configured "nextpage"; if it is between 5 and 8, this means "user-error" 
   * and the PM re-displays the "current page" to let the user to correct his/her errors; 
   * and if the code is greater than 8, the PM displays the configured "errorpage".
   */ 
  public void setCompletionCode(int cc) {
    callInstance.setCompletionCode(cc);
  }
  
  /**
   * Returns the completion code of the current step.
   */ 
  public int getCompletionCode() {
    return callInstance.completionCode;
  }
  
  /**
   * Creates a new Request ADOM for the message if not already created
   * and adds the given message in it.
   * <P>This method will use the <CODE><i>unitname</i></CODE> definition made in the Proceces Definition element
   * to name the data source to be created and the resource associated with it.
   * </P>
   * 
   * @param messageId Either a message text or a resource ID. The message will be taken from the resource file <CODE><i>unitname</i>.res</CODE> with this ID. If no resource
   * is found with the given id in the resource file then <CODE>messageId</CODE> is used as the message text.<br>
   * @param arguments the message arguments which will be used to replace argument symbols (%0, %1, etc.) existing in the message text.<br>
   * @param details Either an <CODE>Exception</CODE> object that keeps the original exception or a <CODE>String</CODE> object that keeps the<br> message details.
   * @return created <CODE>ADOMNode</CODE> object to let you add extra information to the message created.
   * 
   * <P> The XML representation of the created message ADOM is as follows:
   * <PRE>
   * &lt;<i>unitname</i>_messages&gt;
   *    &lt;message id=&quot;messageId&quot;&gt;
   *      &lt;text&gt;Message text taken from the resource file with the given messageId&lt;/text&gt;
   *      &lt;unitname&gt;Given unit name&lt;/unitname&gt;
   *      &lt;details&gt;Message Details. Either the Stack Trace of the orginal exception or the given details as String&lt;/details&gt;
   *    &lt;/message&gt;
   * &lt;/<i>unitname</i>_messages&gt;
   * </PRE>
   * </P>
   * <P><B>Example:</B><PRE>
   *  ADOMNode msg = createMessage(MISSING_TABLE",new String[]{databaseName, tableName},e);
   *  msg.setNodeValue("extrainfo","extra info .....");
   *  p.setCC(12); // That will tell the Processs Manager the execution of the processes was not OK.
   * </PRE>
   * </P>
   */
  public ADOMNode createMessage(String messageId, String[] arguments, Object details) {
    
    String unitName = "";
    try {
      if (details instanceof ProcessException) {
        unitName = ((ProcessException)details).getResBundleName();
      }
      if (unitName == null || unitName.length() == 0) {
        unitName = callDef.getProcessConfig().getUnitName();
      }
    } catch(Exception e){}
    
    ADOMNode msgNode = this.request.createMessage(unitName,messageId,arguments, details);
    
    msgNode.setNodeValue("stepname", callDef.getStepCaption() );
    msgNode.setNodeValue("processname", callDef.getTargetName() );
    
    return msgNode;
  }
  
  /**
   * Creates a new Request ADOM for the message if not already created, composes an message entry from the
   * given Exception object and adds in it. If the Exception object is an instance of a
   * {@link org.moremotion.exception.MoreMotionException MoreMotionException} or
   * a {@link org.moremotion.process.ProcessException ProcessException} this
   * method can extract the message id, arguments and details from it.
   * 
   * <P>
   * If the given exception is or contains a <CODE>java.sql.SQLException</CODE> the returned ADOMNode object
   * contains two additional entries athat are "jdbc-error-code" and "jdbc-sql-state".
   * <P>
   * @see #createMessage(String messageID, String[] arguments, Object details)
   */
  public ADOMNode createMessage(Exception exception) {
    
    java.sql.SQLException se = null;
    String unitName = null;
    String messageId;
    String[] args = null;
    Object details = null;
    
    if (exception instanceof MoreMotionException) {
      MoreMotionException e = (MoreMotionException)exception;
      unitName = e.getResBundleName();
      messageId = e.getMessage();
      args = e.getArguments();
      //      details = e.getRootCause();
      details = e;
      
      if (unitName == null || unitName.length() == 0) {
        unitName = callDef.getProcessConfig().getUnitName();
      }
      
    } else {
      messageId = exception.getMessage();
      if (exception instanceof java.sql.SQLException) {
        se = (java.sql.SQLException)exception;
      }
      details = exception;
    }
    
    ADOMNode msgNode = this.request.createMessage(unitName, messageId, args, details);
    
    msgNode.setNodeValue("stepname", callDef.getStepCaption() );
    msgNode.setNodeValue("processname", callDef.getTargetName() );
    
    if (se != null) {
      msgNode.setNodeValue("jdbc-error-code",se.getErrorCode() + "");
      msgNode.setNodeValue("jdbc-sql-state",se.getSQLState());
    }
    
    return msgNode;
  }  
  
  
}
