/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.process;

/**
 * The QueryProcess Interface is defined for the processes that
 * can create output records from the given input record. The term "Query" here does not imply
 * a database query. It means in general a query that returns 0 or more results.
 * <P>The process manager will call the methods of this interface as in the following example.
 * <PRE>
 * <b>pro.prepare(pbc)</b>;
 * ProcessRecord prec = new ProcessRecord(pb);
 * for (int i = 0; i &lt; pb.getRecordCount(); i++) {
 *   prec.setIndex(i);
 *   if (isThroughRecordFilter(prec)) {
 *     <b>pro.processRecord(prec)</b>;
 *     MemoryProcessBlock outblock = new MemoryProcessBlock( <b>pro.getFieldNames()</b> );
 *     ProcessRecord outrec = new ProcessRecord( outblock );
 *     while (<b>pro.hasMoreResultRecords()</b>) {
 *       outblock.addRecord( <b>pro.getNextResultRecord()</b> );
 *       outrec.setIndex(1);
 *       // Call subprocesses with outblock
 * 
 *       subpro.processRecord(outrec);
 *     }
 *   }
 * }
 * int cc = <b>pro.finalizeProcess()</b>;
 * </PRE>
 * Please note that the process classes that implements this interface can have private data members safely
 * since the process manager will create a new instance of them before calling their methods.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: QueryProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
  abstract public class QueryProcess extends Process {
  
  /**
   * The process should return the names of the fields as a String array
   */
  abstract public String[] getFieldNames() throws ProcessException;
  
  /**
   * The query process should create and return the next result record as a MemoryProcessRecord.
   * If there is no more records to return then the method should return null.
   */
  abstract public MemoryProcessRecord getNextResultRecord() throws ProcessException;
  
  
}
