/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Resource.java
 *
 */

package org.moremotion.resource;

import java.util.ArrayList;
import org.moremotion.util.StringParser;


/**
 * Represents a multi-lingual resource. A resource has a resource Id, arguments and the name of the Resource Bundle
 * where the resource definition exist.
 *
 * <P><B>Example:</B><PRE>
 * String tableName = "table_x";
 * String dbName = "products";
 *
 * Resource res = new Resource("TABLE_NOT_FOUND", new String[]{tableName,dbName}, "myres");
 *
 * ADOM myds = request.newRequestADOM("myds");
 * myds.setNodeValue("msg", res.toString());
 * </PRE><CODE>toString()</CODE> Method returns a specially formatted string. E.g: <PRE>
 *   $res(TABLE_NOT_FOUND#myres,table_x,products)
 * </PRE>
 * Such <CODE>$res()</CODE> definitions existing in an ADOM are resolved with the actual resource text in the language
 * of the page to be displayed next. The actual text of a resource i.e. <CODE>TABLE_NOT_FOUND</CODE> is taken
 * from the resource bundle i.e. <CODE>myres</CODE> and the <CODE>%0, %1 </CODE>symbols in the resource text are
 * replaced with arguments i.e. <CODE>table_x, products</CODE>.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Resource.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Resource {
  
  public static final String RES = "$res(";
  private String bundleName = "no_resource";
  private String id = null;
  private String[] arguments = null;
  
  /**
   * Creates a new resource.
   * @param id The resource Id
   */
  public Resource(String id) {
    this.id = id;
  }
  
  /**
   * Creates a new resource.
   * @param id The resource Id
   * @param arguments The String array each element of it positionally corresponding to a resource argument
   */
  public Resource(String id, String[] arguments) {
    this.id = id;
    this.arguments = arguments;
  }
  
  /**
   * Creates a new resource.
   * @param id The resource Id
   * @param arguments The String array each element of it positionally corresponding to a resource argument
   * @param resBundleName The name of the resource bundle where the actual resource text of this resource is kept
   */
  public Resource(String id, String[] arguments, String resBundleName) {
    this.id = id;
    this.arguments = arguments;
    this.bundleName = resBundleName;
  }
  
  /**
   * Sets the name of the resource bundle
   */
  public void setBundleName(String bundleName) {
    this.bundleName = bundleName;
  }
  
  /**
   * Returns the name of the resource bundle
   */
  public String getBundleName() {
    return this.bundleName;
  }
  
  /**
   * Returns 'true' is a bundle name is assigned to resource 'false' otherwise
   */
  public boolean bundleAssigned() {
    return !this.bundleName.equals("no_resource");
  }
  
  /**
   * Returns the resource Id.
   */
  public String getId() {
    return this.id;
  }
  
  /**
   * Returns the resource arguments.
   */
  public String[] getArguments() {
    return this.arguments;
  }
  
  /**
   * Sets the resource arguments.
   */
  public void setArguments(String[] value) {
    this.arguments = value;
  }
  
  private static String makeResFunction(String message, String[] arguments, String bundleName) {
    StringBuffer buffer = new StringBuffer();
    buffer.append(RES);
    if (bundleName != null) {
      buffer.append("#");
      buffer.append(bundleName);
      buffer.append(",");
    }
    if (message == null) message = "";
    buffer.append(message.replace(',','\4').replace('\'','\5'));
    if (arguments != null && arguments.length > 0) {
      for (int i = 0; i < arguments.length; i++) {
        String arg = arguments[i];
        if (arg != null) {
          buffer.append(",");
          
          
          int j = arg.indexOf(Resource.RES);
          if (j > -1) {
            StringBuffer sb = new StringBuffer(arg.length());
            int k = 0;
            while (j != -1) {
              k = StringParser.locateChar(arg, j + Resource.RES.length(), ')');
              if (k == -1) k = arg.length(); else k++;
              sb.append(escape(arg.substring(0,j)));
              sb.append(arg.substring(j,k));
              j = arg.indexOf(Resource.RES,k);
            }
            sb.append(escape(arg.substring(k)));
            buffer.append( sb.toString() );
          } else {
            buffer.append( escape(arg) );
          }
        }
      }
    }
    buffer.append(")");
    return buffer.toString();
  }
  
  
  private static String escape(String s) {
    return s.replace(',','\4').replace('\'','\5').replace('(','\6').replace(')','\7');
  }
  
  private static String unescape(String s) {
    return s.replace('\4',',').replace('\5','\'').replace('\6','(').replace('\7',')');
  }
  
  /**
   * Returns the string respresentation of this resource as $res() function.
   * <CODE>$res(RESOURCE_ID#bundle_name,res_arg1,res_arg2,..)</CODE>
   */
  public String toString() {
    return makeResFunction(this.id, this.arguments, this.bundleName);
  }
  
  /**
   * Formats this resource by retrieving the actual text in specified language from the resource bundle and
   * replacing the resource argument symbols in it.
   * @param language The language code i.e. en, de, fr, tr, etc.
   * @return The formatted string
   * @throws ResourceException if retrieval of the actual text fails.
   */
  public String format(String language) throws ResourceException {
    String msgText;
    if (this.bundleName != null) {
      ResourceBundle bundle = ResourceManager.getResourceBundle(this.bundleName,language);
      msgText = bundle.getResourceText(this.id);
      if (msgText.equals(this.id)) {
        return formatWithoutResource(msgText,arguments);
      }
    } else {
      msgText = this.id;
      return formatWithoutResource(msgText,arguments);
    }
    int index = -1;
    if (arguments != null) {
      for (int i = 0; i < arguments.length; i++) {
        Object o = arguments[i];
        index++;
        if (o != null) {
          String sym = "%" + i;
          int apos = msgText.indexOf(sym);
          while (apos > -1) {
            String symval = o.toString();
            msgText = msgText.substring(0,apos) + symval + msgText.substring(apos + sym.length());
            apos = msgText.indexOf(sym);
          }
        }
      }
    }
    return msgText;
  }
  
  
  private static String formatWithoutResource(String str, String[] arguments) {
    StringBuffer buf = new StringBuffer();
    buf.append(str);
    if (arguments != null) {
      for (int i = 0; i < arguments.length; i++) {
        Object o = arguments[i];
        buf.append(", " + o);
      }
    }
    return buf.toString();
  }
  
  //////////////////////////////////////////////////////////////////////////////
  // static
  /** Turns message to java message format. Converts %i to {i} [i=0, 1, 2, ... n].
   * Takes care of '{' and '}' if used in message text.
   */
  /*
  private static String toJMF(String message) {
    if (message == null) {
      throw new IllegalStateException("Message identifier of a Resource cannot be null at this moment!");
    }
    int len = message.length();
    StringBuffer outBuffer = new StringBuffer(len*2);
    for(int x=0; x<len; x++) {
      char aChar = message.charAt(x);
      switch(aChar) {
        case '%':
          if ( x + 1 < len) {
            char nextChar = message.charAt(x + 1);
            if (Character.isDigit(nextChar)) {
              outBuffer.append('{');
              do {
                outBuffer.append(message.charAt(++x));
              } while( (x + 1 < len) && (Character.isDigit(message.charAt(x + 1))) );
              outBuffer.append('}');
            } else
              if (nextChar == '%') {
                do {
                  outBuffer.append(message.charAt(++x));
                } while( (x + 1 < len) && ('%' == message.charAt(x + 1)));
              } else {
                throw new IllegalStateException("'%' should be succeeded by either '%' or a digit! [msg: " +
                message + ", pos: " + (x + 2) + "]");
              }
          } else {
            throw new IllegalStateException("Resource messages cannot end *ONLY* with '%'! [msg: " + message + "]");
          }
          break;
        case '{':outBuffer.append('\''); outBuffer.append(aChar); outBuffer.append('\'');
        break;
        case '}':outBuffer.append('\''); outBuffer.append(aChar); outBuffer.append('\'');
        break;
        default:
          outBuffer.append(aChar);
      }
    }
    return outBuffer.toString();
  }
   */
  
  /**
   * Parses the resource function in the specified string and returns a resource object.
   */
  public static Resource parse(String resfunc) throws ResourceException {
    if (resfunc == null) {
      throw new ResourceException("PARSING_RES_FUNCTION_FAILED:NULL_VALUE");
    }
    if (!resfunc.startsWith("$res(")) {
      throw new ResourceException("PARSING_RES_FUNCTION_FAILED:MISSING_RES_SYMBOL", new String[]{resfunc}, null);
    }
    if (!resfunc.endsWith(")")) {
      throw new ResourceException("PARSING_RES_FUNCTION_FAILED:MISSING_RIGTH_PARANTHESIS", new String[]{resfunc}, null);
    }
    
    String content = resfunc.substring(RES.length(), resfunc.length() - 1);
    StringParser sp = new StringParser(content,',');
    sp.trimmed();
    
    String resname = null;
    String msg = sp.nextItem();
    
    if (msg.startsWith("#")) {
      resname = msg.substring(1);
      
      msg = unescape(sp.nextItem());
      
    } else {
      msg = unescape(msg);
    }
    
    String[] args = null;
    ArrayList argList;
    if (sp.hasMore()) {
      argList = new ArrayList();
      while (sp.hasMore()) {
        String arg = unescape(sp.nextItem());
        argList.add(arg);
      }
      args = new String[argList.size()];
      argList.toArray(args);
    }
    
    if (msg.equals("")) {
      throw new ResourceException("PARSING_RES_FUNCTION_FAILED:RESOURCE_ID_MISSING", new String[]{resfunc}, null);
    }
    
    Resource result = new Resource(msg,args);
    if (resname != null) result.setBundleName(resname);
    return result;
  }
  
  //  /** Extracts resource messages from a given string.
  //   * @param inBuffer Where resource messages are searched.
  //   * @return Resource array with 0 length, if no resource message is found.
  //   */
  //
  //
  //  public static Resource[] extractResources(String inBuffer) {
  //    if (inBuffer == null) {
  //      throw new IllegalArgumentException("Input cannot be null at this moment!");
  //    }
  //    ArrayList rmList = new ArrayList();
  //    if (inBuffer.indexOf(RES) == -1) {
  //      Resource[] result = new Resource[rmList.size()];
  //      rmList.toArray(result);
  //      return result;
  //    }
  //
  //    int len = inBuffer.length();
  //    int end = 0;
  //
  //    while(true) {
  //      int start = inBuffer.indexOf(RES, end);
  //      if (start == -1) {
  //        break;
  //      }
  //      end = StringParser.locateClosingChar(inBuffer, start + RES.length() ,')');
  //      if (end != -1) {
  //        end++;
  //        rmList.add(parse(inBuffer.substring(start,end)));
  //      }
  //    }
  //
  //    Resource[] result = new Resource[rmList.size()];
  //    rmList.toArray(result);
  //    return result;
  //  }
  
  //////////////////////////////////////////////////////////////////////////////
  // Main
  /** Test method.
   */
  public static void  main(String[] args) throws Exception {
    //testToString();
    testParse();
    //testExtract();
  }
  
  
  private static void testExtract() {
    //String str = "bla bla bla $res(User not found!) bla bla";
    //String str = "bla bla bla $res(User not found!)";
    String str = "$res(User not found!) bla bla bla $res((ABC((()))CDE), (,,,,,), qwer)";
    //String str = "$res((User not found!{Consider this: %1, %2}), (ABC, DEF), XYZ) a (( )) a $res(User %1 not found!, 123)";
    
    //   Resource[] found = extractResources(str);
    //    for (int i=0; i < found.length; i++) {
    //      System.out.println("#" + (i+1) + ": " + found[i]);
    //    }
  }
  
  private static void testParse() {
    try {
      String str = "$res( User not found! [username %1], Franklin, Benjamin)";
      System.out.println("#1: " + Resource.parse(str).toString());
      
      str = "$res((User not found! [username %1]),(Franklin, Benjamin))";
      System.out.println("#2: " + Resource.parse(str));
      
      str = "$res( User not found! [username %1])";
      System.out.println("#3: " + Resource.parse(str));
      
      str = "$res((I like these animals: %1, NOT this: %2), (cat, dog, dove), donkey)";
      System.out.println("#4: " + Resource.parse(str));
      
      str = "$res(I like %1, banana)";
      System.out.println("#5: " + Resource.parse(str));
    } catch (Exception e){}
  }
  
  // test toString() method
  private static void testToString() {
    // no variant
    System.out.println("#1: " + new Resource("User is %%20 %%% invalid {yep} [username: %0]"));
    // with arguments
    String[] xxx = {"ABC"};
    System.out.println("#3: " + new Resource("User is invalid {yep} [username: %1]", xxx));
    String[] yyy = {"ABC", "DEF", "XYZ"};
    System.out.println("#4: " + new Resource("User is invalid {yep} [username: %1]", yyy));
    // end-points
    System.out.println("#5: " + new Resource("abc xyz %20"));
    System.out.println("#6: " + new Resource("abc xyz %%20"));
    System.out.println("#7: " + new Resource("abc xyz %%"));
    System.out.println("#8: " + new Resource("abc xyz 5%%"));
    // exceptional cases
    try {
      System.out.println("#9: " + new Resource("abc xyz 5%"));
    } catch(Exception e) {
      e.printStackTrace();
    }
    try {
      System.out.println("#10: " + new Resource("abc xyz %A"));
    } catch(Exception e) {
      e.printStackTrace();
    }
    try {
      System.out.println("#11: " + new Resource("%A abc xyz"));
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
}