/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.resource;

import java.util.Properties;
import java.util.Hashtable;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.StringParser;
import org.moremotion.cache.FileChangeDetector;
import org.moremotion.cache.FileChangeNotification;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/** Responsible for accessing resources available in the current moremotion installation.
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ResourceManager.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ResourceManager implements FileChangeNotification {

  private static ResourceManager Instance = null;
  private static Hashtable cache = new Hashtable();

  private ResourceManager() {
  }

  public void doChange(File f) {
    cache = new Hashtable();
    SystemContext.logger().printres("RELOADING_RESOURCES", f.getName());
  }

  public void doDelete(File f) {
    cache = new Hashtable();
    SystemContext.logger().printres("RELOADING_RESOURCES_FILE_DELETED", f.getName());
  }

  /**
   * Returns a ResourceBundle object by the given resource identifier and language code. If the specified bundle is not in cache
   * or the resource file newer than the cached bundle, this methos (re)loads it from the resource file located under
   * "/WEB-INF/MM-INF/resources" folder.
   * <P>If a resource file, i.e. "myres" is multi-lingual it should exists in "WEB-INF/MM-INF/resources/myres.res" file
   * in the following format:<PRE>
   * &lt;root&gt;
   *   &lt;item id="RES1"&gt;
   *     &lt;en&gt;English Text&lt;/en&gt;
   *     &lt;de&gt;Deutscher Text&lt;/de&gt;
   *   &lt;/item&gt;
   *   &lt;item id="RES2"&gt;
   *     ...
   * &lt;/root&gt; 
   * </PRE>
   * It is also possible to provide a separate resource file per language as follows. <PRE>
   * English resource file "WEB-INF/MM-INF/resources/en/myres.res"
   * &lt;root&gt;
   *   &lt;item id="RES1"&gt;English Text&lt;/item&gt;
   *   &lt;item id="RES2"&gt; ...
   * &lt;/root&gt; 
   *
   * German resource file "WEB-INF/MM-INF/resources/de/myres.res"
   * &lt;root&gt;
   *   &lt;item id="RES1"&gt;Deutscher Text&lt;/item&gt;
   *   &lt;item id="RES2"&gt; ...
   * &lt;/root&gt; 
   * </PRE>
   *</P>
   * @param bundleName The name of the resource bundle
   * @param language Language code as: <CODE>tr, en, de, fr, etc.</CODE>.
   * @return ResourceBundle object.
   * @throws ResourceException if resource file is not found or is invalid.
   */
  public static ResourceBundle getResourceBundle(String bundleName, String language) throws ResourceException {

    if (Instance == null) {
      Instance = new ResourceManager();
    }
    if (bundleName == null) {
      return emptyResourceBundle();
    }

    if (bundleName.startsWith("/") && bundleName.indexOf(".") == -1) {
      // Data Source Resource
      String name = bundleName.substring(1);
      return new ResourceBundle(name, SystemContext.servletContext(), language);
    }

    String key = bundleName + "@" + language;

    ResourceBundle bundle = (ResourceBundle) cache.get(key);
    if (bundle != null) {
      return bundle;
    }

    synchronized (key) {

      File f = new File(SystemConfig.mmHomeFile(), "resources" + File.separator + bundleName + ".res");
      File lf = new File(SystemConfig.mmHomeFile(), "resources" + File.separator + language + File.separator + bundleName + ".res");

      if (lf.exists()) {
        bundle = getResourceBundle(lf, null);
      } else {
        bundle = getResourceBundle(f, language);
      }
      bundle.setName(bundleName);

      if (bundle == null) {
        return emptyResourceBundle();
      }
      cache.put(key, bundle);

    }

    return bundle;
  }

  /**
   * Returns the ResourceBundle object of MoreMotion.
   * @param language Language code as: <CODE>tr, en, de, fr, etc.</CODE>.
   * @return ResourceBundle object.
   */
  public static ResourceBundle getSystemResourceBundle(String language) throws ResourceException {
    if (language == null) {
      language = SystemConfig.applicationLanguage();
    }
    return getResourceBundle("moremotion", language);
  }

  /**
   * Resolves the preformatted $res() definitions existing in the given string using the given language.
   * @param inBuffer The input string that contains preformatted $res() definitions.
   * @param language The language code to use in resolving the $res() definitions.
   */
  public static String replaceResDefinitions(String inBuffer, String language) throws ResourceException {

    if (inBuffer == null) {
      return "";
    }

    int start = inBuffer.indexOf(Resource.RES);
    if (start == -1) {
      return inBuffer;
    }

    StringBuffer outBuffer = new StringBuffer(inBuffer.length() + (inBuffer.length() / 2));
    int len = inBuffer.length();
    int end = 0;

    while (start != -1) {

      outBuffer.append(inBuffer.substring(end, start));

      end = StringParser.locateChar(inBuffer, start + Resource.RES.length(), ')');

      if (end == -1) {
        String def;
        int sl = inBuffer.length();
        if (sl - start > 100) {
          def = inBuffer.substring(start, start + 100);
        } else {
          def = inBuffer.substring(start);
        }
        throw new ResourceException("INVALID_RES_FUNCTION:MISSING_RIGTH_PARANTHESIS", new String[]{def}, null);
      }
      end++;
      String resmsg = inBuffer.substring(start, end);

      Resource msg = Resource.parse(resmsg);
      String result = msg.format(language);
      outBuffer.append(result);

      start = inBuffer.indexOf(Resource.RES, end);
    }

    outBuffer.append(inBuffer.substring(end));
    return outBuffer.toString();
  }

  /**
   * Returns an empty resource
   */
  private static ResourceBundle emptyResourceBundle() {
    return new ResourceBundle(new Properties(), -1);
  }

  private static ResourceBundle getResourceBundle(File f, String language) throws ResourceException {

    ResourceBundle result = null;
    if (!f.exists()) {
      return emptyResourceBundle();
    }

    FileChangeDetector.getInstance().addFile(f, Instance);

    try {
      Properties source = parseResourceFile(f, language);
      result = new ResourceBundle(source, f.lastModified());
    } catch (Exception ex) {
      throw new ResourceException(ex);
    }

    return result;
  }

  private static Properties parseResourceFile(File f, String language) throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    Document document = builder.parse(f);

    Properties result = new Properties();
    Node rootNode = document.getElementsByTagName("root").item(0);
    processElement(result, rootNode, language);
    return result;
  }

  private static String getValue(Node node) {
    Node n = node.getFirstChild();
    if (n == null) {
      return "";
    }
    return n.getNodeValue();  
  }

  private static Element getChildElement(Element elm, String tagName) {
    NodeList nlist = elm.getElementsByTagName(tagName);
    return (Element) nlist.item(0);
  }

  private static void processElement(Properties result, Node root, String language) {

    NodeList nlist = root.getChildNodes();

    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        Element currElm = (Element) node;
        if (currElm.getNodeName().equals("item")) {
          String id = currElm.getAttribute("id");
          Element theElement = currElm;
          if (language != null) {
            theElement = getChildElement(currElm, language);
          }

          if (theElement != null) {
            String value = getValue(theElement);
            value = value.trim();
            if (value.startsWith(".\n")) {
              value = value.substring(2);
            }

            result.setProperty(id, value);
          }
        }
      }

    }

  }
}
