/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security;

import java.util.*;

/** 
 * Represents a role by a name and description. A Role may zero or more checkpoints. 
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Role.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Role implements Comparable { 
  
  private String id = null;
  private String name;
  private String description;  
  private Properties attributes = null;
  
  
  private TreeSet checkpoints = null;
  private TreeSet subRoles = null;
  
  /** Creates a new role.
   */  
  public Role() {
    this.checkpoints = new TreeSet();
    this.subRoles = new TreeSet();
  }

  /** Creates a new role.
   * @param name Role name.
   * @param description Role description.
   */  
  public Role(String name, String description) {
    this();
    this.name = name;
    this.description = description;
  }
  
  /** Adds a check point to role.
   */  
  public void addCheckPoint(CheckPoint checkPoint) {
    checkpoints.add(checkPoint);
  }
  
  /** Removes a check point from role.
   */  
  public void removeCheckPoint(CheckPoint checkPoint) {
    checkpoints.remove(checkPoint);
  }
  
  /** Returns a list of check points.
   */  
  public CheckPoint[] getCheckPoints() {
    CheckPoint[] result = new CheckPoint[0];
    if (checkpoints.size() > 0) {
      result = new CheckPoint[checkpoints.size()];
      checkpoints.toArray(result);
    }
    return result;
  }
  
  /** 
   * Returns <CODE>true</CODE>if this role has the given check point, <CODE>false</CODE> otherwise.
   */ 
  public boolean hasCheckPoint(String checkPointName) {
    boolean result = false;
    Iterator it = checkpoints.iterator();
    while(!result && it.hasNext()) {
      result = ((CheckPoint) it.next()).getName().equals(checkPointName);
    }
    return result; 
  }
  
  /** Returns role name.
   */  
  public String getName() {
    return name;
  }
  
  /** Returns role description.
   */  
  public String getDescription() {
    return description;
  }

  /** Sets role name.
   */  
  public void setName(String name) {
    this.name = name;
  }
  
  /** Sets role description.
   */  
  public void setDescription(String description) {
    this.description = description;
  }
  
  /** Gets role identifier.
   */  
  public String getId() {
    return this.id;
  }

  /** Sets role identifier.
   * <P><STRONG>For internal use only.</STRONG></P>
   */  
  public void setId(String value) {
    this.id = value;
  }

  /** Checks wheter or not this role is equal to other.
   */  
  public boolean equals(Object obj) {
    if (obj instanceof Role) {
      Role r = (Role) obj;
      return r.getName().equals(getName());
    }
    return false;
  }
  
  /** Returns string respresentation of role.
   */  
  public String toString() {
    String result = "Role {" +
                    "\n  id: " + getId() + 
                    "\n  name: " + getName() + 
                    "\n  description: " + getDescription();
    
    CheckPoint[] checkpoints = getCheckPoints();
    for (int i=0; checkpoints != null && i < checkpoints.length; i++) {
      result += "\n  CheckPoint[" + i + "]: " + checkpoints[i].getName();
    }
    
    result += "\n}";               
    return result;
  }
  
  /** Compares this role to other.
   */  
  public int compareTo(Object obj) {
    int result = 0;
    if (obj instanceof Role) {
      result = getName().compareTo(((Role) obj).getName());
    }
    return result;
  }
  
  
  /**
   * Returns the attribute of this role.
   */
  public Properties getAttributes() {
    return this.attributes;
  }
  
  /**
   * Sets a user attribute to this role.
   */
  public void setAttribute(String name, String value) {
    if (attributes == null) {
      attributes = new Properties();
    }
    attributes.setProperty(name, value);
  }
  
  /**
   * Returns the value of the specified role attribute.
   */
  public String getAttribute(String name) {
    if (attributes == null) return null;
    return attributes.getProperty(name);
  }
  
  /**
   * Removes the specified attribute from this role.
   */
  public void removeAttribute(String name) {
    if (attributes != null) {
      attributes.remove(name);
    }
  }
  
  
}