/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security;

import java.util.ArrayList;
import java.util.Date;
import javax.servlet.http.HttpSession;
import org.moremotion.security.broker.BrokerCommunicationContext;
import org.moremotion.security.broker.SecurityBrokerConfig;

import org.moremotion.security.domain.SecurityDomain;
import org.moremotion.security.domain.SecurityDomainException;
import org.moremotion.security.domain.SecurityDomainManager;
import org.moremotion.security.domain.SignedUser;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.MiscUtils;
import org.moremotion.util.MutableInt;
import org.moremotion.util.XString;


/**
 * Represents a Security Context that is the gateway for user related security operations. 
 * The object of this class is created by the {@link SecurityManager#getSecurityContext} method.
 * 
 * The operations that update the security information are reflected to the persistency through the security broker interface that is
 * defined by the SecurityBrokerConfig.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SecurityContext.java 72 2008-03-18 14:12:35Z erkan $
 */

public class SecurityContext {
  
  private static final String SA_NAME = "moremotion.security.current.user.";
  private static final String JUSTLOGGEDOUT = "moremotion.security.justloggedout.";
  private static final String DEFAULT_SUBDOMAIN = "";
  
  private SecurityBrokerConfig sbconfig;
  private MoreMotionRequest request;
  private HttpSession session;
  private BrokerCommunicationContext bcc = null;
  private SecurityDomain secdom;
  private String secdomName = null;
  private String subdomain = null;
  private String sessionId;
  
  private RememberUser remember = null;
  
  protected SecurityContext(MoreMotionRequest request, SecurityBrokerConfig sbconfig, String domainName) {
    this.request = request;
    this.sbconfig = sbconfig;
    this.session = request.getSession();
    secdomName = domainName;
    secdom = SecurityDomainManager.getDomain(domainName);
    this.sessionId = request.getSession().getId();
    
    subdomain = (String)session.getAttribute(SA_NAME + "domain" + domainName + ".subdomain");
    if (subdomain == null) subdomain = DEFAULT_SUBDOMAIN;
    
    this.remember = new RememberUser(request,secdomName);
  }
  
  public void prepareBroker()  {
    if (bcc == null) {
      try {
        bcc = sbconfig.broker.prepare(request, sbconfig.brokerInitObject);
        bcc.pool.setProperty("subdomain", subdomain);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
  
  /**
   * Returns the request associated with this context.
   */
  public MoreMotionRequest getRequest() {
    return this.request;
  }
  
  /**
   * Sets the subdomain of this context
   */
  public void setSubdomain(String subdomain) {
    this.subdomain = subdomain;
    session.setAttribute(SA_NAME + "domain" + secdomName + ".subdomain", subdomain);
    
    if (bcc != null) {
      bcc.pool.setProperty("subdomain", subdomain);
    }
  }
  
  /**
   * Returns the subdomain associated with this context
   */
  public String getSubdomain() {
    return this.subdomain;
  }
  
  
  /**
   * Returns the name of the security domain associated with this context.
   */
  public String getSecurityDomain() {
    return this.secdomName;
  }
  
  
  public RememberUser getRememberUser() {
    return this.remember;
  }
  
  /**
   * Returns the specified portion of the active users
   * @param subDomainName
   */
  public ArrayList getActiveUsers(String subDomainName, int from, int amount) {
    return secdom.getActiveUserList(subDomainName, from, amount);
  }
  
  /**
   * Returns the number of the active users
   */
  public int getActiveUserCount(String subDomainName) {
    return secdom.getActiveUserCount(subDomainName);
  }
  
  
  /**
   * Returns the security broker definition associated with this context.
   */
  public SecurityBrokerConfig getSecurityBrokerDefinition() {
    return this.sbconfig;
  }
  
  
  /**
   * Creates a new user in this context.
   * @param name The user name
   * @param password The plain password
   * @param email The email of the user
   * @param fullname The full name of the user
   * @param accDsbld <CODE>true</CODE> to create the new user as disabled
   * @return The user object created.
   */
  public User newUser(String name, String password, String email, String fullname, boolean accDsbld) {
    User user = new User();
    user.setBroker(sbconfig.broker);
    user.setName(name);
    user.setPassword(password);
    user.setEmail(email);
    user.setFullname(fullname);
    user.setAccountDisabled(accDsbld);
    return user;
  }
  
  /**
   * Authenticates the given user ubject and the specified password (plain) and returns <CODE>true</CODE>
   * if user is found and its password mathes.
   */
  public boolean authenticateUser(User user, String password) {
    boolean result = false;
    try {
      sbconfig.broker.authenticateUser(user, password);
      result = true;
    } catch (Exception ex) {}
    return result;
  }
  
  /**
   * Returns <CODE>true</CODE> if the given user object has the authority for the specified checkpoint.
   */
  public boolean hasAuthority(User user, String checkPoint) {
    if (user.isAccountDisabled()) {
      return false;
    }
    if (user.getName().equals(sbconfig.rootUserName)) {
      return true;
    }
    
    if (checkPoint.equals("IS_LOGGED_IN")) {
      User currentUser = getCurrentUser();
      if (currentUser != null & currentUser.equals(user)) {
        return true;
      }
    }
    
    if (user.hasCheckPoint(checkPoint)) {
      return true;
    }
    return false;
  }
  
  /**
   * Returns the name of the root user.
   * Unless otherwise defined the name of the root user is 'sa'.
   */
  public String getRootUserName() {
    return sbconfig.rootUserName;
  }
  
  /**
   * Returns the name of the role name of the system administrator.
   * Unless otherwise defined the name of the system administrator role is 'SYS_ADM'.
   */
  public String getAdminRoleName() {
    return sbconfig.adminRoleName;
  }
  
  public User getCurrentUser(boolean refresh) {
    if (refresh) {
      request.removeAttribute(SA_NAME + secdomName);
    }
    return getCurrentUser();
  }
  
  /**
   * Returns the user object that currently logged-in. Returns <CODE>null</CODE> if no user is logged-in.
   */
  public User getCurrentUser() {
    
    String jlo = (String) request.getAttribute(JUSTLOGGEDOUT + secdomName);
    if (jlo != null && jlo.equals("true")) {
      return null;
    }
    
    User user = (User) request.getAttribute(SA_NAME + secdomName);
    if (user != null) return user;
    
    boolean restored = false;
    
    try {
      
      String oldsid = request.getParameter("_sessionid");
      if (oldsid != null /* && request.getMethod().equalsIgnoreCase("POST") */ ) {
        String userName = secdom.trustedLogin(oldsid, this.sessionId);
        subdomain = XString.before(userName,'~', DEFAULT_SUBDOMAIN);
        if (bcc != null) {
          bcc.pool.setProperty("subdomain", subdomain);
        }
        userName = XString.after(userName,'~', userName);
        user = getUser(userName);
        user.setLModified( (new Date()).getTime() );
        session.setAttribute(SA_NAME + secdomName, user);
      }
      SignedUser su = secdom.getSignedUser(this.sessionId);
      
      if (su == null) {
        
        if (secdom.isKilled(this.sessionId)) {
          forcedLogout();
          return null;
        }
        
        if (remember.isLoggedIn()) {
          try {
            if (remember.getRecentUserName() != null) {
              user = getUser(remember.getRecentUserName());
              setSubdomain(remember.getRecentSubDomain());
              login(user);
              restored = true;
              su = secdom.getSignedUser(this.sessionId);
            }
          } catch (Exception ex){}
        }
        
      } 
      
      if (su == null) return null;
      
      else {
        user = (User)session.getAttribute(SA_NAME + secdomName);
        if (su.lmodified > user.getLModified()) {
          user = getUser(user.getName());
          user.setLModified( (new Date()).getTime() );
          session.setAttribute(SA_NAME + secdomName, user);
        }
        user.setSignInTime(su.getSignInTime());
        user.setLastReferredTime(su.getLastReferredTime());
        if (restored) {
          remember.setUser(user.getName(),subdomain);
          remember.touchCookie();
        }
        request.setAttribute(SA_NAME + secdomName,user);
        return user;
        
      }
    } catch (Exception e) {
      // e.printStackTrace();
      return null;
    }
  }
  
  
  public SignedUser getSignerUser() {
    return secdom.getSignedUser(this.sessionId);
  }
  
  /**
   * Logs in the given user.
   * @param user The user object to login
   * @throws SecurityDomainException if login fails
   */
  public void login(User user) throws SecurityDomainException {
    login(user,false);
  }
  
  /**
   * Logs in the given user.
   * @param user The user object to login
   * @param multipleLoginAllowed If <CODE>false</CODE> and is the user is already logged-in it fires MultipleLoginDisallowedException.
   * @throws SecurityDomainException if login fails
   * @throws MultipleLoginDisallowedException if the user is already logged-in.
   */
  public void login(User user, boolean multipleLoginAllowed) throws SecurityDomainException {
    secdom.login(subdomain, user.getName(), this.sessionId, multipleLoginAllowed);
    this.session.setAttribute(SA_NAME + secdomName, user);
    this.session.setAttribute("moremotion.refresh.id", MiscUtils.timeStamp() );
    request.removeAttribute(JUSTLOGGEDOUT + secdomName);
    remember.setUser(user.getName(),subdomain);
  }
  
  
  public int registerFailedLoginAttempt(String userName, long suspendPeriod) {
    return secdom.failedLoginAttempt(subdomain, userName, suspendPeriod);
  }
  
  public int flaTries(String userName, long suspendPeriod) {
    return secdom.isUserSuspended(subdomain, userName, suspendPeriod);
  }
  
  
  /**
   * Logs out the current user
   */
  public void logout() {
    secdom.logout(this.sessionId);
    this.session.removeAttribute("moremotion.page.table");
    this.session.removeAttribute(SA_NAME + secdomName);
    this.request.setAttribute(JUSTLOGGEDOUT + secdomName,"true");
    this.session.setAttribute("moremotion.refresh.id", MiscUtils.timeStamp() );
  }
  
  /**
   * Logs out the specified user.
   */
  public void logout(String userName) {
    secdom.logoutUser(subdomain, userName);
  }
  
  public void forcedLogout() {
    this.session.removeAttribute("moremotion.page.table");
    this.session.removeAttribute(SA_NAME + secdomName);
    this.request.setAttribute("moremotion.security.justloggedout." + secdomName,"true");
    this.session.setAttribute("moremotion.refresh.id", MiscUtils.timeStamp() );
  }
  
  /**
   * Encryptes the specified plain password. Encryption is performed by the security broker attacted to the context.
   */
  public String encryptPassword(String password) {
    return sbconfig.broker.encryptPassword(password);
  }
  
  /**
   * Returns a User object by its username.
   * @param username User name.
   * @return User object.
   * @throws MissingSecurityEntityException If user cannot be found.
   */
  public User getUser(String username) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getUser(this.bcc, username);
  }
  
  /**
   * Returns a User object by its email.
   * @param email Email of user.
   * @return User object.
   * @throws MissingSecurityEntityException If user cannot be found.
   */
  public User getUserByEmail(String email) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getUserByEmail(this.bcc, email);
  }
  
  /**
   * Returns a list of all the users.
   * @return User objects array.
   */
  public User[] getUsers() throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getUsers(this.bcc);
  }
  
  /**
   * Returns a sublist of the users.
   * @param from the start index.
   * @param amount the number of the users to return.
   * @param nou A return value: Total number of users found
   * @return User objects array.
   */
  public User[] getUsers(int from, int amount, MutableInt nou) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getUsers(this.bcc, from, amount, nou);
  }
  
  /**
   * Returns a sublist of users that match with search criteria
   * @param searchCriteria the string that contains the search criteria
   * @param from the start index.
   * @param amount the number of the users to return.
   * @param nou A return value: Total number of users found
   * @return User objects array.
   */
  public User[] searchUsers(String searchCriteria, int from, int amount, MutableInt nou) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.searchUsers(this.bcc, searchCriteria, from, amount, nou);
  }
  
  /**
   * Saves given user. Makes it persistent through the associated security broker.
   *
   * @param user User object
   * @throws SaveSecurityEntityException If user cannot be saved.
   */
  public void saveUser(User user) throws SaveSecurityEntityException {
    
    if (user.getName().equals(sbconfig.rootUserName)) {
      if (user.isAccountDisabled()) {
        throw new SaveSecurityEntityException("Root user cannot be disabled!");
      }
    }
    if (bcc == null) prepareBroker();
    sbconfig.broker.saveUser(this.bcc, user);
    if (user.isAccountDisabled()) {
      secdom.logoutUser(subdomain, user.getName());
    } else {
      secdom.userModified(subdomain, user.getName());
    }
  }
  
  /**
   * Deletes the given user.
   *
   * @param userName name of the user
   * @throws DeleteSecurityEntityException If user cannot be deleted.
   */
  public void deleteUser(String userName) throws DeleteSecurityEntityException {
    if (userName.equals(sbconfig.rootUserName)) {
      throw new DeleteSecurityEntityException("Root user cannot be deleted!");
    }
    
    if (bcc == null) prepareBroker();
    sbconfig.broker.deleteUser(this.bcc, userName);
    secdom.logoutUser(subdomain, userName);
  }
  
  /**
   * Finds a role by its name.
   * @param roleName Role name.
   * @return Role object.
   * @throws MissingSecurityEntityException If role cannot be found.
   */
  public Role getRole(String roleName) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getRole(this.bcc, roleName);
  }
  
  /**
   * Lists all roles.
   * @return Role objects array.
   */
  public Role[] getRoles() {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getRoles(this.bcc);
  }
  
  /**
   * Saves a role.
   *
   * @param role Role object.
   * @throws SaveSecurityEntityException If role cannot be saved.
   */
  public void saveRole(Role role) throws SaveSecurityEntityException {
    
    if (bcc == null) prepareBroker();
    sbconfig.broker.saveRole(this.bcc, role);
    
    secdom.allUsersModified(subdomain);
    
  }
  
  /**
   * Deletes a role by its name.
   *
   * @param roleName Role name.
   * @throws DeleteSecurityEntityException If role cannot be deleted.
   */
  public void deleteRole(String roleName) throws DeleteSecurityEntityException {
    if (bcc == null) prepareBroker();
    sbconfig.broker.deleteRole(this.bcc, roleName);
  }
  
  /**
   * Lists all check points.
   * @return CheckPoint ojects array.
   */
  public CheckPoint[] getCheckPoints() {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getCheckPoints(this.bcc);
  }
  
  /**
   * Gets an CheckPoint by its name.
   * @return CheckPoint object.
   * @param checkPointName Check point name.
   * @throws MissingSecurityEntityException If check point cannot be found.
   */
  public CheckPoint getCheckPoint(String checkPointName) throws MissingSecurityEntityException {
    if (bcc == null) prepareBroker();
    return sbconfig.broker.getCheckPoint(this.bcc, checkPointName);
  }
  
  /**
   * Saves a check point.
   *
   * @param checkPoint CheckPoint object.
   * @throws SaveSecurityEntityException If check point cannot be saved.
   */
  public void saveCheckPoint(CheckPoint checkPoint) throws SaveSecurityEntityException {
    
    if (bcc == null) prepareBroker();
    sbconfig.broker.saveCheckPoint(this.bcc, checkPoint);
  }
  
  /**
   * Deletes an check point.
   *
   * @param checkPointName Check point name.
   * @throws DeleteSecurityEntityException If check point cannot be deleted.
   */
  public void deleteCheckPoint(String checkPointName) throws DeleteSecurityEntityException {
    if (bcc == null) prepareBroker();
    sbconfig.broker.deleteCheckPoint(this.bcc, checkPointName);
    
  }
  
  /**
   * Sets a security broker variable.
   *
   * @param name variable name.
   * @param value variavle value.
   * @throws SaveSecurityEntityException If property value cannot be set.
   */
  public void setVar(String name, String value) throws SaveSecurityEntityException {
    if (bcc == null) prepareBroker();
    bcc.pool.setProperty("_setvar_" + name, value);
  }
  
  
}