/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security;

import java.io.IOException;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.page.CurrentPage;
import org.moremotion.security.broker.SecurityBroker;
import org.moremotion.security.broker.SecurityBrokerConfig;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * This class is responsible for providing a security context to the caller.
 * It loads the specified security broker class (if not yet loaded) and associates
 * with the provided context together with the security domain.
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SecurityManager.java 115 2008-04-08 11:56:01Z erkan $
 */
public class SecurityManager {

  private static SecurityContext getSecurityContext(MoreMotionRequest request, String secdom, String brokerName, String conn) throws SecurityContextException {

    String securityContextKey = "moremotion.security.context.key." + secdom + brokerName + conn;

    SecurityContext securityContext = (SecurityContext) request.getAttribute(securityContextKey);
    if (securityContext != null) {
      return securityContext;
    }

    try {

      SecurityBrokerConfig sbconfig = getSecurityBrokerConfig(brokerName);

      String className = sbconfig.getClassName();

      if (className == null || className.length() == 0) {
        throw new SecurityContextException("SECURITY_BROKER_CLASS_NAMER_MISSING",new String[]{brokerName},null);
      }

      if (sbconfig.brokerInitObject == null) {
        synchronized (sbconfig) {
          if (sbconfig.brokerInitObject == null) {
            sbconfig.broker = (SecurityBroker) Class.forName(className).newInstance();
            sbconfig.brokerInitObject = sbconfig.broker.init(request, conn, sbconfig.getParameters());
          }
        }
      }

      securityContext = new SecurityContext(request, sbconfig, secdom);

    } catch (Exception e) {
      throw new SecurityContextException(e);
    }
    request.setAttribute(securityContextKey, securityContext);
    return securityContext;
  }

  private static SecurityBrokerConfig getSecurityBrokerConfig(String name) throws ConfigurationException {

    ConfigUnit unit = ConfigCache.getUnit("securityBroker", name);
    if (unit == null) {
      throw new ConfigurationException("CONFIG_UNIT_MISSING", new String[]{name}, null);
    }

    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }

    return (SecurityBrokerConfig) unit.getDataObject();
  }

  private static void parse(ConfigUnit unit) throws ConfigurationException {

    ConfigNodeList nodelist = unit.getConfigNodeList();

    SecurityBrokerConfig sbconfig = new SecurityBrokerConfig(unit.getName(), nodelist);
    unit.setDataObject(sbconfig);

  }

  /**
   * Returns a security context associated with the specified Access Control Definition object.
   * @param request An MoreMotionRequest object
   * @param securityDomainName The name of the <CODE>securityDomain</CODE> configuration element in the MoreMotion configuration
   * @return The Security Context built
   * @throws SecurityContextException if context cannot be built
   */
  public static SecurityContext getSecurityContext(MoreMotionRequest request, String securityDomainName) throws SecurityContextException {

    try {
      
      Config sdconfig = ConfigManager.getConfig("securityDomain", securityDomainName);
      if (sdconfig == null) {
        throw new SecurityContextException("SECURITY_DOMAIN_CONFIGURATION_MISSING",new String[]{securityDomainName},null);
      }
      String brokerName = sdconfig.getParameter("broker").resolve(request).stringValue();
      String conn = sdconfig.getParameter("conn").stringValue();

      SecurityContext sctx = getSecurityContext(request, securityDomainName, brokerName, conn);
      sctx.getRememberUser().setOption(sdconfig.getParameter("cookie.opt").stringValue("2"),sdconfig.getParameter("cookie.maxage").intValue(2592000));
      return sctx;

    } catch (ConfigurationException ex) {
      throw new SecurityContextException(ex);
    }


  }

  public static boolean checkAccessAuthority(MoreMotionRequest request, String acdStr, CurrentPage currpage) throws Exception {
    AccessControlDefinition acd = new AccessControlDefinition(acdStr);
    return checkAccessAuthority(request, acd, currpage);
  }

  public static boolean checkAccessAuthority(MoreMotionRequest request, AccessControlDefinition acd) throws Exception {
    return checkAccessAuthority(request, acd, null);
  }

  public static boolean checkAccessAuthority(MoreMotionRequest request, AccessControlDefinition acd, CurrentPage currpage) throws Exception {

    String cp = "";

    if (acd.hasCheckPoint()) {

      SecurityContext sctx = null;
      try {
        sctx = getSecurityContext(request, acd.getSecurityDomain());
      } catch (SecurityContextException sce) {
        if (sce.getMessage().equals("NO_SECURITY_COMPONENT_AVAILABLE")) {
          return true;
        }
        throw sce;
      }

      User currentUser = sctx.getCurrentUser();
      try {
        try {
          cp = request.resolve(acd.getCheckPoint());
        } catch (Exception ex) {
          ex.printStackTrace();
        }
        if (cp != null && cp.equals("IS_LOGGED_IN")) {
          if (currentUser == null) {
            throw new NoUserAttachedException("NO_USER_IN_SESSION");
          }
        }

        boolean isRoleCp = false;
        boolean hasRoleCp = false;
        if (cp.startsWith("IS_") && cp.length() > 3) {
          try {
            String roleName = cp.substring(3);
            sctx.getRole(roleName); // Check such a role exists.
            isRoleCp = true;
            Role[] userRoles = currentUser.getRoles();
            for (int i = 0; i < userRoles.length; i++) {
              if (roleName.equals(userRoles[i].getName())) {
                hasRoleCp = true;
                break;
              }
            }
          } catch (Exception e) {
          }
        }

        if (!isRoleCp) {
          sctx.getCheckPoint(cp); // If there is no such CheckPoint then MissingSecurityEntityException fired
        }

        if (currentUser == null) {
          throw new NoUserAttachedException("NO_USER_IN_SESSION");
        }

        if ((isRoleCp && !hasRoleCp) || !sctx.hasAuthority(currentUser, cp)) {
          throw new AuthorizationException("Insufficient authority!");
        }

      } catch (AuthorizationException ae) {

        if (currpage == null) {
          return false;
        }

        String pg = acd.getFailurePage();
        if (pg != null) {
          request.createMessage("moremotion.PageGenerator", "NOT_AUTHORIZED_TO_ACCESS", new String[]{acd.getCheckPoint()}, null);
          request.generatingErrorPage();
          request.generatePage(request.resolve(pg));
          return false;

        } else {
          request.createMessage("moremotion.PageGenerator", "NOT_AUTHORIZED_TO_ACCESS", new String[]{acd.getCheckPoint()}, null);
          request.generateDefaultErrorPage();
          return false;
        }

      } catch (NoUserAttachedException nuae) {

        String pg = acd.getLoginPage();

        if (pg != null && pg.length() > 0) {

          if (currpage != null) {
            currpage.pushSelfName();
            currpage.flushNextPages();
          }

          request.generatePage(request.resolve(pg));

          return false;

        } else {

          try {

            if (currpage != null) {
              currpage.pushSelfName();
              currpage.flushNextPages();
            }
            request.getResponse().sendRedirect(request.getContextPath() + "/");
            return false;

          } catch (IOException ioe) {
            throw new org.moremotion.page.PageGenerationException("Failed to redirect to main page!", null, ioe);
          }
        }

      } catch (MissingSecurityEntityException e) {
      // Check point cannot be found. Allow the user to see the page.
      }
    }

    return true;
  }
}

class BrokerDef {

  String id;
  String name;
  String className;
  String params;
  String conDSType;
  String rootUserName;
  String adminRoleName;

  public String toString() {
    return "id:" + id + ", name:" + name + ", className:" + className +
            ", params:" + params + ", conDSType:" + conDSType + ", rootUserName:" + rootUserName +
            ", adminRoleName:" + adminRoleName;
  }
}
