/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security;


import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.TreeSet;
import org.moremotion.security.broker.SecurityBroker;

/** 
 * Represent a user. A User has a user name, a password, a full name, an email, an account
 * enabled/disabled flag, attributes and roles. 
 * 
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: User.java 51 2008-03-10 11:46:54Z erkan $
 */
public class User implements Comparable {
  
  private SecurityBroker broker = null;
  private long lmodified;
  private long signInTime;
  private long lastReferredTime;
  private String id = null;
  private String username;
  private String password;
  private String email;
  private String fullname;
  private boolean accountDisabled = false;
  private TreeSet roles;
  private Properties attributes = null;
  
  
  /** Creates a new user.
   */
  public User() {
    roles = new TreeSet();
  }
  
  /** Creates a new user.
   * @param name User name.
   * @param password Encrypted Password.
   * @param email Email.
   * @param fullname Full name.
   * @param accDsbld Account enabled/disabled flag. True for enabled, false for disabled.
   */
  public User(String name, String password, String email, String fullname, boolean accDsbld) {
    this();
    this.username = name;
    this.password = password;
    this.email = email;
    this.fullname = fullname;
    ////    if (name.equals("sa")) accDsbld = false;
    this.accountDisabled = accDsbld;
    this.lmodified = (new Date()).getTime();
    // UserManager.Instance().setUserPassword(this, password);
  }
  
  /**
   * Sets the security broker to this object .
   */
  public void setBroker(SecurityBroker broker) {
    this.broker = broker;
  }
  
  /**
   * Returns the last modification time.
   */
  public long getLModified() {
    return this.lmodified;
  }
  
  /**
   * Sets the modification time.
   */
  public void setLModified(long value) {
    this.lmodified = value;
  }
  
  /**
   * Returns the sign in time.
   */
  public long getSignInTime() {
    return this.signInTime;
  }
  
  /**
   * Sets the sign in time.
   */
  public void setSignInTime(long value) {
    this.signInTime = value;
  }
  
  /**
   * Returns the last referred time.
   */
  public long getLastReferredTime() {
    return this.lastReferredTime;
  }
  
  /**
   * Sets the last referred time.
   */
  public void setLastReferredTime(long value) {
    this.lastReferredTime = value;
  }
  
  /**
   * Returns the attribute of this user.
   */
  public Properties getAttributes() {
    return this.attributes;
  }
  
  /**
   * Sets a user attribute to this user.
   */
  public void setAttribute(String name, String value) {
    if (attributes == null) {
      attributes = new Properties();
    }
    attributes.setProperty(name, value);
  }
  
  /**
   * Returns the value of the specified user attribute.
   */
  public String getAttribute(String name) {
    if (attributes == null) return null;
    return attributes.getProperty(name);
  }
  
  /**
   * Removes the specified attribute from this user.
   */
  public void removeAttribute(String name) {
    if (attributes != null) {
      attributes.remove(name);
    }
  }
  
  /** 
   * Authenticates the specified password agains the password of this user and returns <CODE>true</CODE> if they match.
   */
  public boolean authenticate(String password) {
    if (!isAccountDisabled()) {
      try {
        broker.authenticateUser(this, password);
        return true;
      } catch (Exception e) {
//        e.printStackTrace();
        return false;
      }
    }
    return false;
  }
  
  /** 
   * Returns roles of this user.
   */
  public Role[] getRoles() {
    Role[] result = new Role[roles.size()];
    roles.toArray(result);
    return result;
  }
  
  /** 
   * Returns <CODE>true</CODE> if this user is disabled.
   */
  public boolean isAccountDisabled() {
    return accountDisabled;
  }
  
  /** 
   * Sets the user account as enabled or disabled.
   */
  public void setAccountDisabled(boolean value) {
    this.accountDisabled = value;
  }
  
//  /** Returns whether or not the user is logged in.
//   */
//  public boolean isLoggedIn() {
//    return loggedIn;
//  }
  
//  /** Sets the user's login status
//   */
//  public void setLoggedIn(boolean value) {
//    this.loggedIn = value;
//  }
  
  /** 
   * Adds a new role to this user.
   */
  public void addRole(Role role) {
    roles.add(role);
  }
  
  /** 
   * Removes a role from this user.
   */
  public void removeRole(Role role) {
    roles.remove(role);
  }
  
  /** 
   * Returns the name of this user.
   */
  public String getName() {
    return username;
  }
  
  /** 
   * Returns the email address of this user.
   */
  public String getEmail() {
    return email;
  }
  
  /** 
   * Sets the user name of this user.
   */
  public void setName(String username) {
    this.username = username;
  }
  
  /** 
   * Sets the email address of this user.
   */
  public void setEmail(String email) {
    this.email = email;
  }
  
  /** 
   * Returns the encrypted password of this user. 
   */
  public String getPassword() {
    return password;
  }
  
  /** 
   * Encrypts the specified password and stores in this user.
   */
  public void setPassword(String password) {
    this.password = broker.encryptPassword(password);
  }
  
  /** 
   * Returns the full name of this user.
   */
  public String getFullname() {
    return fullname;
  }
  
  /** 
   * Sets the full name of this user.
   */
  public void setFullname(String fullname) {
    this.fullname = fullname;
  }
  
  /** 
   * Returns the database record identity of this user. Generally table row id.
   */
  public String getId() {
    return this.id;
  }
  
  /** Sets the database record identity of this user. Generally table row id.
   * <P><B>Note that: </B>This method is used only by Security Brokers after retrieving the user info from database.</P>
   */
  public void setId(String value) {
    this.id = value;
  }
  
  /** 
   * Checks the authority of user against a check point.
   * @param checkPoint CheckPoint.
   * @return true if user has authority, false if not.
   */
  public boolean hasCheckPoint(String checkPoint) {
    
    Role[] roles = getRoles();
      
    if (checkPoint.startsWith("IS_")) {
      String roleName = checkPoint.substring(3);
      for (int i = 0; i < roles.length ; i++) {
        Role role = roles[i];
        if (roleName.equals(role.getName())) {
          return true;
        }
      }
      return false;
    }
    
    for (int i = 0; i < roles.length ; i++) {
      Role role = roles[i];
      
      CheckPoint[] checkPoints = role.getCheckPoints();
      if (checkPoints != null) {
        for (int j = 0; j < checkPoints.length; j++) {
          CheckPoint cp = checkPoints[j];
          if (checkPoint.equals(cp.getName())) {
            return true;
          }
        }
      }
    }
    
    return false;
  }
  
  /**
   * Returns the names of the checkpoints that this user has.
   */
  public String[] getCheckPointNames() {
    TreeSet cpnames = new TreeSet();
    Iterator it = roles.iterator();
    while (it.hasNext()) {
      Role r = (Role)it.next();
      cpnames.add("IS_" + r.getName());
      CheckPoint[] cps = r.getCheckPoints();
      for (int j = 0; j < cps.length; j++) {
        String cpn = cps[j].getName();
        boolean found = false;
        Iterator ti = cpnames.iterator();
        while (ti.hasNext()) {
          String s = (String)ti.next();
          if (s.equals(cpn)) break;
        }
        if (!found) {
          cpnames.add(cpn);
        }
      }
    }
    String[] result = new String[cpnames.size()];
    cpnames.toArray(result);
    return result;
  }
  
  
  /** 
   * Returns string representation of this user.
   */
  public String toString() {
    String result = "User {" +
    "\n  id: " + getId() +
    "\n  username: " + getName() +
    "\n  password: " + getPassword() +
    "\n  email: " + getEmail() +
    "\n  fullname: " + getFullname() +
    "\n  account disabled: " + isAccountDisabled();
//    "\n  logged In: " + isLoggedIn();
    Role[] roles = getRoles();
    for (int i=0; roles != null && i < roles.length; i++) {
      result += "\n  Role[" + i + "]: " + roles[i].getName();
    }
    result +="\n}";
    return result;
  }
  
//  /**
//   * Returns XML representation of this user.
//   * <P>Sample XML:</P>
//   * <PRE>
//   * &lt;id&gt;0615&lt;/id&gt;
//   * &lt;name&gt;john&lt;/name&gt;
//   * &lt;email&gt;john.coombes@moryazilim.com&lt;/email&gt;
//   * &lt;fullname&gt;John Coombes&lt;/fullname&gt;
//   * &lt;attribute_x&gt;attribute_x Value&lt;/attribute_x&gt;
//   * &lt;loggedin&gt;true&lt;/loggedin&gt;
//   * &lt;checkpoints&gt;
//   *   &lt;CAN_DO_THIS/&gt;
//   *   &lt;CAN_DO_THAT/&gt;
//   * &lt;/checkpoints&gt;
//   * </PRE>
//   */
//  public String toXml(boolean loggedin) {
//    StringBuffer sb = new StringBuffer(150);
//    
//    sb.append("<id>");
//    sb.append(id);
//    sb.append("</id>");
//    
//    sb.append("<username>");
//    sb.append(username);
//    sb.append("</username>");
//    
//    sb.append("<email>");
//    sb.append(email);
//    sb.append("</email>");
//    
//    sb.append("<fullname>");
//    if (fullname == null) {
//      sb.append(username);
//    } else {
//      sb.append(fullname);
//    }
//    sb.append("</fullname>");
//    
//    if (attributes != null) {
//      Enumeration e = attributes.keys();
//      while (e.hasMoreElements()) {
//        String name = (String)e.nextElement();
//        String value = attributes.getProperty(name);
//        sb.append("<");
//        sb.append(name);
//        sb.append(">");
//        sb.append(value);
//        sb.append("</");
//        sb.append(name);
//        sb.append(">");
//      }
//    }
//    
//    if (loggedin) {
//      sb.append("<loggedin>true</loggedin>");
//      
//      String[] cps = getCheckPointNames();
//      if (cps != null) {
//        sb.append("<checkpoints>");
//        for (int i = 0; i < cps.length; i++) {
//          sb.append("<checkpoint>");
//          sb.append(cps[i]);
//          sb.append("</checkpoint>");
//        }
//        sb.append("</checkpoints>");
//      }
//    }
//    
//    return sb.toString();
//  }
  
  /** 
   * Compares this user to other.
   */
  public int compareTo(Object obj) {
    int result = 0;
    if (obj instanceof User) {
      result = getName().compareTo(((User) obj).getName());
    }
    return result;
  }
}