/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security.broker;

import java.util.Map;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.security.User;
import org.moremotion.security.Role;
import org.moremotion.security.CheckPoint;
import org.moremotion.security.AuthenticationException;
import org.moremotion.security.MissingSecurityEntityException;
import org.moremotion.security.DeleteSecurityEntityException;
import org.moremotion.security.SaveSecurityEntityException;
import org.moremotion.util.MutableInt;

/**
 * Defines an Interface for security brokers. With this interface MoreMotion's security system is isolated from the persistency
 * (where the security information stored).
 * 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SecurityBroker.java 51 2008-03-10 11:46:54Z erkan $
 */
public interface SecurityBroker {

  /**
   * Initialises the broker. This method is called by the security manager right after the class that implements this interface is loaded.
   * @param request MoreMotionRequest object
   * @param connectionDatasource The name of the MoreMotion datasource that is defined in datasources dialog of mmDesigner.
   * @param brokerParams The Map object that contains the broker parameters
   * @return An object created by the broker. This object will be kept by the security manager and passed to the broker
   * again by the <CODE>prepare()</CODE> call.
   * @throws SecurityBrokerException if initialisation fails.
   */
  public Object init(MoreMotionRequest request, String connectionDatasource, Map brokerParams) throws SecurityBrokerException;

  /**
   * Prepares the broker for the successive calls. This method is called once for each Servlet Request to prepare
   * the broker for the successive calls until the request is responded. For relational database brokers this is a good
   * time to get a database connection.
   * @param request MoreMotionRequest object
   * @param initializedObject The object created by the <CODE>init()</CODE> method.
   * @return An object created by this method, for releational database
   * brokers generally this object is a connection object or contains an connection object.
   * @throws SecurityBrokerException if preparation fails.
   */
  public BrokerCommunicationContext prepare(MoreMotionRequest request, Object initializedObject) throws SecurityBrokerException;

  /**
   *
   */
//  public void cleanUp(BrokerCommunicationContext bcc) throws SecurityBrokerException;

  /**
   * Gets a user by its name.
   * @param bcc BrokerCommunication Object
   * @param userName User name.
   * @throws MissingSecurityEntityException If user cannot be found.
   * @return User object.
   */
  public User getUser(BrokerCommunicationContext bcc, String userName) throws MissingSecurityEntityException;

  /**
   * Gets a user by its Email address.
   * @param bcc BrokerCommunication Object
   * @param email User E-mail.
   * @throws MissingSecurityEntityException If no user is found with given email.
   * @return User object.
   */
  public User getUserByEmail(BrokerCommunicationContext bcc, String email) throws MissingSecurityEntityException;

  /**
   * Deletes a user.
   * @param bcc BrokerCommunication Object
   * @param userName User name.
   * @throws DeleteSecurityEntityException If deletion fails.
   */
  public void deleteUser(BrokerCommunicationContext bcc, String userName) throws DeleteSecurityEntityException;

  /**
   * Saves a user.
   * @param bcc BrokerCommunication Object
   * @param user User object.
   * @throws SaveSecurityEntityException If user cannot be saved.
   */
  public void saveUser(BrokerCommunicationContext bcc, User user) throws SaveSecurityEntityException;

  /**
   * Returns list of all users.
   * @param bcc BrokerCommunication Object
   * @return User objects array.
   */
  public User[] getUsers(BrokerCommunicationContext bcc) throws MissingSecurityEntityException;

  /**
   * Returns a sublist of the users.
   * @param bcc BrokerCommunication Object
   * @param from the start index.
   * @param amount the number of the users to return.
   * @return User objects array.
   */
  public User[] getUsers(BrokerCommunicationContext bcc, int from, int amount, MutableInt noi) throws MissingSecurityEntityException;

  /**
   * Returns list of that users that match with search criteria
   * @param bcc BrokerCommunication Object
   * @param searchCriteria the string that contains the search criteria
   * @param from the start index.
   * @param amount the number of the users to return.
   * @return User objects array.
   */
  public User[] searchUsers(BrokerCommunicationContext bcc, String searchCriteria, int from, int amount, MutableInt noi) throws MissingSecurityEntityException;

  /**
   * Authenticates a user with password (plain)
   * @param user User object.
   * @param password String.
   * @throws Authenticate if user is not authenticated.
   */
  public void authenticateUser(User user, String password) throws AuthenticationException;

  /**
   * Encryptes the given plain password. The Broker should implement password encryption here
   * @param password Plain Password String.
   */
  public String encryptPassword(String password);


  /**
   * Returns a role by its name.
   * @param bcc BrokerCommunication Object
   * @param roleName Role name.
   * @throws MissingSecurityEntityException If role cannot be found.
   * @return Role object.
   */
  public Role getRole(BrokerCommunicationContext bcc, String roleName) throws MissingSecurityEntityException;

  /**
   * Deletes a role by its Name.
   * @param bcc BrokerCommunication Object
   * @param roleName Role Name.
   * @throws DeleteSecurityEntityException If role cannot be found.
   */
  public void deleteRole(BrokerCommunicationContext bcc, String roleName) throws DeleteSecurityEntityException;

  /**
   * Returns a list all of roles.
   * @param bcc BrokerCommunication Object
   * @return Role objects array.
   */
  public Role[] getRoles(BrokerCommunicationContext bcc);

  /**
   * Saves a role.
   * @param bcc BrokerCommunication Object
   * @param role Role object.
   * @throws SaveSecurityEntityException If role cannot be saved.
   */
  public void saveRole(BrokerCommunicationContext bcc, Role role) throws SaveSecurityEntityException;


  /**
   * Finds a check point by its name.
   * @param bcc BrokerCommunication Object
   * @param checkPointName Check point name.
   * @return CheckPoint object.
   * @throws MissingSecurityEntityException If check point cannot be found.
   */
  public CheckPoint getCheckPoint(BrokerCommunicationContext bcc, String checkPointName) throws MissingSecurityEntityException;


  /**
   * Returns a lists of all checkpoints.
   * @param bcc BrokerCommunication Object
   * @return CheckPoint objects array.
   */
  public CheckPoint[] getCheckPoints(BrokerCommunicationContext bcc);


  /**
   * Deletes a check point.
   * @param bcc BrokerCommunication Object
   * @param checkPointName Check point name.
   * @throws DeleteSecurityEntityException If deletion fails.
   */
  public void deleteCheckPoint(BrokerCommunicationContext bcc, String checkPointName) throws DeleteSecurityEntityException;


  /**
   * Saves a check point.
   * @param bcc BrokerCommunication Object
   * @param checkPoint CheckPoint object.
   * @throws SaveSecurityEntityException If check point cannot be saved.
   */
  public void saveCheckPoint(BrokerCommunicationContext bcc, CheckPoint checkPoint) throws SaveSecurityEntityException;

}
