/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MoreMotionRequest.java
 *
 * Created on July 9, 2002, 6:45 PM
 */

package org.moremotion.servlet;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.util.Hashtable;
import java.util.Map;
import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMManager;
import org.moremotion.adom.ADOMNode;
import org.moremotion.cache.VariablePool;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.MScriptException;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.evaluator.Vars;
import org.moremotion.exception.MoreMotionException;
import org.moremotion.page.CommandDefinition;
import org.moremotion.page.CurrentPage;
import org.moremotion.page.PageGenerationException;
import org.moremotion.page.PageGenerator;
import org.moremotion.page.ParmToEncrypt;
import org.moremotion.resource.Resource;
import org.moremotion.util.XString;

/**
 * MoreMotionRequest is the most important class of the MoreMotion Application Framework.
 * It provides methods suitable with the way MoreMotion responds to request.
 * One significant functionality provided by this class is decoding the request parameters correctly.
 *
 * <P><B>Request Parameter Decoding</B></P>
 *
 * <P>When a request is sent to the server, the parameters existing in the request are encoded by the
 * browsers using an encoding, i.e. ISO-8859-1. When an application server receives a request,
 * it decodes these parameters and prepare a request object to pass to the Servlet.</P>
 *
 * <P>If the application server does not use the same encoding that the browser used to encode the
 * parameters during the decoding, the request parameters can get corrupted.
 * Unfortunately this is the case for Tomcat. Tomcat always decode the request parameters using ISO-8859-1.</P>
 *
 * <P>MoreMotion solves this problem with the help of the request parameter "_enc". In order to benefit
 * from this functionality, a hidden input field with name "_enc" must be placed in the form elements and
 * their values must be set to the encoding of the current page.</P>
 *
 * <P>If you get the value of a request parameter by using the methods provided by this class, i.e. getParameter(),
 * the value will be decoded using the encoding value supplied by this "_enc" parameter. </P>
 * <PRE>
 * String name = request.getParameter("name",null);
 * // The name string will get the decoded "name" parameter.
 * </PRE>
 * However, some application servers can nicely guess the case and reads request parameters correctly.
 * In that case you should disable the parameter conversion by setting the following system parameter to false.
 *
 * <PRE>
 * &lt;enableParameterConversion&gt;false&lt;/enableParameterConversion&gt;
 * </PRE>
 *
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MoreMotionRequest.java 186 2008-07-31 18:23:46Z erkan $
 */
public class MoreMotionRequest implements HttpServletRequest {

  private HttpServletRequest request;
  private HttpServletResponse response;

  private String _enc = null;
  private boolean conversionRequired = true;
  private Hashtable forcedDatasources = null;
  private VariablePool vpool = null;
  private MMSymbolResolver mmsr = null;
  private boolean requestResponded = false;
  private int allowCaching = 0; // -1: No, 0: Page Defined, 1: Yes
  private String lang = null;
  private Vars vars = null;
  private CurrentPage currpage = null;
  private boolean generatingErrorPage = false;
  private String[] sourceAreaNames = null;

  
  public MoreMotionRequest(HttpServletRequest request, HttpServletResponse response) {
    this.request = request;
    this.response = response;
    setConvRequired();
  }

  /**
   * This method can be used to notify to the MoreMotion that a response is already given by you so he should not bother
   * generating a page.
   */
  public void setResponded(boolean value) {
    requestResponded = value;
  }

  /**
   * Returns <CODE>true</CODE> if the a response is already prepared for this request.
   */
  public boolean getResponded() {
    return requestResponded;
  }

  public void setAllowCaching(int value) {
    this.allowCaching = value;  
  }
  
  public int getAllowCaching() {
    return this.allowCaching;
  }
  
  public boolean generatingErrorPage() {
    return generatingErrorPage;
  }

  public void setGeneratingErrorPage() {
    this.generatingErrorPage = true;
  }

  public void setSourceAreaNames(String names) {
    sourceAreaNames = XString.split(names,',');
  }
  
  public String[] getSourceAreaNames() {
    return this.sourceAreaNames;
  }
  
  /**
   * Returns the Symbol MScriptResolver object that this request is associated.
   */
  public MMSymbolResolver getSymbolResolver() {
    if (this.mmsr == null) {
      mmsr = new MMSymbolResolver(this);
    }
    return this.mmsr;
  }

  public void adjustVarsToPage(String pageName) {
    getVars().adjustToPage(pageName);
  }

  public Vars getVars() {
    if (vars == null) vars = new Vars(this);
    return vars;
  }

  /**
   * Returns the debugging level of an unit.
   * @param unit The name of the unit.
   * @return A number between 0 - 3 where 0: debuging is off, 1: Level 1 debugging, 2: Level 2 debugging and 3: Level 3 debugging.
   */
  public int getUnitDebugLevel(String unit) {
    String swname = "__debugswitch_" + unit;
    String value = (String) getSession().getAttribute(swname);
    if (value == null) return 0;
    if (value.equals("1")) return 1;
    if (value.equals("2")) return 2;
    if (value.equals("3")) return 3;
    return 0;
  }

  /**
   * Returns the variable pool object that is associated with this request
   */
  public VariablePool getVPool() {
    if (vpool == null) {
      vpool = new VariablePool();
      vpool.put("LANG",getLanguage());
      vpool.put("ULANG",getLanguage().toUpperCase());
    }
    return vpool;
  }

  private static final String CURRENT_LANGUAGE = "moremotion.current.user.language";

  private void saveLanguage(String lang) {
    getSession().setAttribute(CURRENT_LANGUAGE, lang);
    if (vpool != null) {
      vpool.put("LANG",lang);
      vpool.put("ULANG",lang.toUpperCase());
    }
    setCookieValue(CURRENT_LANGUAGE, lang, 3600000);
  }

  /**
   * Returns the current language code
   */
  public String getLanguage() {

    if (lang == null) {
      lang = getPlainParameter("_lang",null);
      if (lang != null && lang.equals("ml")) lang = null;
      if (lang == null) {
        lang = (String)getSession().getAttribute(CURRENT_LANGUAGE);
        if (lang == null) {
          lang = getCookieValue(CURRENT_LANGUAGE);
        }
      }
      if (lang == null) lang = "en";
      saveLanguage(lang);
    }
    return lang;
  }


  /**
   * The name of the origin page that is taken from request parameter "_originpage". Note that the
   * origin page name is generally required to access to the configuration of the page where the originating
   * element is located; therefore "_originpage" request parameter should be supplied for such cases.
   *
   */
  public String getOriginPage() {
    
    try {
      
      CurrentPage cpage = getCurrentPage();
      if (cpage == null) {
        String originPage = request.getParameter("_originpage");
        if (originPage == null || originPage.trim().length() == 0) {
          return "index";
        }
        cpage = new CurrentPage(this, originPage);
      }
      
      return cpage.getPageName(true);
      
    } catch (PageGenerationException ex) {
      return "index";
    }
    
  }

  /**
   * Sets the current language code
   */
  private void setLanguage(String value) {
    if (value != null && value.equals("ml")) return;
    lang = value;
    getSession().setAttribute(CURRENT_LANGUAGE, lang);
    getVPool().put("LANG",lang);
    getVPool().put("ULANG",lang.toUpperCase());
  }

  private void setConvRequired() {

    if (!org.moremotion.servlet.SystemConfig.enableParameterConversion()) {
      conversionRequired = false;
      return;
    }
    _enc = request.getParameter("_enc");
    if (_enc == null || _enc.equals("")) {
      conversionRequired = false;
      return;
    } else {
      _enc = _enc.toLowerCase();
    }
    conversionRequired = org.moremotion.servlet.SystemConfig.serverEncoding().indexOf(_enc) == -1;
  }

  /**
   * Sets a cookie.
   * @param cookieName the Name of the cookie to set.
   * @param cookieValue the value for the cookie.
   * @param maxAge the duration of the cookie is seconds.
   */
  public void setCookieValue(String cookieName, String cookieValue, int maxAge) {
    Cookie cookie = new Cookie(cookieName, cookieValue);
    cookie.setMaxAge(maxAge);
    cookie.setPath("/");
    response.addCookie(cookie);
  }

  /**
   * Returns the value of the specified cookie.
   * @param cookieName the Name of the cookie to set.
   * @return <CODE>null</CODE> if the cookie is not found, otherwise the value of it.
   */
  public String getCookieValue(String cookieName) {
    Cookie[] cookies = request.getCookies();
    for (int i=0; cookies != null && i < cookies.length; i++) {
      if (cookies[i].getName().equals(cookieName)) {
        String value = cookies[i].getValue();
        if (value.length() == 0) value = null;
        return value;
      }
    }
    
    return null;
  }

  /**
   * Creates a new Request Datasource for the message if not already created
   * and adds the given message in it.
   *
   * @param unitName Name of the reporting unit. This name will be used to name the data source to be created and the resource associated with it.
   *
   * @param messageId Either a message text or a resource Id. The message will be taken from the resource file <CODE><i>unitName</i>.res</CODE> with this Id. If no resource
   * is found with the given id in the resource file then <CODE>messageId</CODE> is used as the message text.
   *
   * @param arguments the message arguments which will be used to replace argument symbols (%0, %1, etc.) existing in the message text.
   *
   * @param details Either an <CODE>Exception</CODE> object that keeps the original exception or a <CODE>String</CODE> object that keeps the message details.
   *
   * @return created <CODE>ADOMNode</CODE> object to let you add extra information to the message created.
   *
   * <P> The XML Structure of the created message datasource is as follows:
   * <PRE>
   * &lt;<i>unitName</i>_messages&gt;
   *    &lt;message id=&quot;messageId&quot;&gt;
   *      &lt;text&gt;Message text taken from the resource file with the given messageId&lt;/text&gt;
   *      &lt;unitname&gt;Given unit name&lt;/unitname&gt;
   *      &lt;details&gt;Message Details. Either the Stack Trace of the orginal exception or the given details as String&lt;/details&gt;
   *    &lt;/message&gt;
   * &lt;/<i>unitName</i>_messages&gt;
   * </PRE>
   * </P>
   * <P>
   * Example:
   * <PRE>
   *  ADOMNode msg = request.CreateMessage("myunit","MISSING_TABLE",new String[]{databaseName, tableName},e);
   *  msg.setNodeValue("extrainfo","extra info .....");
   *  request.generateDefaultErrorPage();  // To display messages using MoreMotion's default error page
   *  request.generatePage("MyErrorPage"); // To display messages using your error page.
   *                                       // You can prepare your Error Page using template element "\cpn\mor\ErrorPageTemplate.mmel"
   * </PRE>
   * </P>
   */
  public ADOMNode createMessage(String unitName, String messageId, String[] arguments, Object details) {

    Exception exception = null;
    String detailsStr = null;

    if (details instanceof Exception) {
      exception = (Exception)details;
    } else if (details instanceof String) {
      detailsStr = (String)details;
    }

    if (messageId == null || messageId.length() == 0) {
      messageId = "Error!";
    }

    String resbundName = "moremotion";

    if (unitName != null) {
      resbundName = XString.before(unitName,'.');
    }

    String adomname = resbundName + "_messages";
    ADOM adom = getRequestADOM(adomname);
    if (adom == null) {
      registerMessageAdom(adomname);
      adom = newRequestADOM(adomname);
      adom.setResBundleName(resbundName);
      includeInPageXML(adomname,resbundName,adom);
    }
    ADOMNode messageNode = new ADOMNode("message");
    adom.addNode(messageNode);

    messageNode.addAttribute("id",messageId);

    String messageText = "";
    if (messageId != null && messageId.length() > 0) {
      messageText = (new Resource(messageId,arguments,resbundName)).toString();
    }
    messageNode.setNodeValue("unitname",unitName);
    messageNode.setNodeValue("text",messageText);
    if (exception != null) {
      messageNode.setNodeValue("details", org.moremotion.exception.ExceptionUtils.getStackTrace(exception));
    } else if (details != null) {
      messageNode.setNodeValue("details", detailsStr);
    }

    return messageNode;
  }

  /**
   * Creates a new Request Datasource for the message if not already created
   * and adds the given message in it. If the Exception object is an instance of a
   * {@link org.moremotion.exception.MoreMotionException MoreMotionException} this
   * method can extract the message id, arguments end details from it.
   * @see #createMessage(String unitName, String messageId, String[] arguments, Object details)
   * <P>
   * If the given exception is or contains a <CODE>java.sql.SQLException</CODE> the returned ADOMNode object
   * contains two additional entries that are "jdbc-error-code" and "jdbc-sql-state".
   * <P>
   */
  public ADOMNode createMessage(Exception exception) {
    java.sql.SQLException se = null;
    ADOMNode msgNode;
    String unitName;
    if (exception instanceof MoreMotionException) {
      MoreMotionException e = (MoreMotionException)exception;
      se = e.getOriginalSQLException();
      unitName = e.getResBundleName();
      msgNode = createMessage(unitName, e.getMessage(), e.getArguments(), e);
    } else {
      msgNode = createMessage(null, exception.getMessage(), null, exception);
      if (exception instanceof java.sql.SQLException) {
        se = (java.sql.SQLException)exception;
      }
    }

    if (se != null) {
      msgNode.setNodeValue("jdbc-error-code",se.getErrorCode() + "");
      msgNode.setNodeValue("jdbc-sql-state",se.getSQLState());
    }
    return msgNode;

  }
  
  private void registerMessageAdom(String adomName) {
    String adomNames = (String)getAttribute("moremotion.message.adom.names");
    if (adomNames == null) {
      adomNames = new String(adomName);
    } else {
      adomNames = adomNames + ',' + adomName;
    }
    setAttribute("moremotion.message.adom.names",adomNames);
  }


  /**
   * This method forces MoreMotion to include the content of the specified datasource to the page XML
   * data even though there is no explicit reference to it in the page to be displayed next.
   * @param dsname The name of the datasource to include to the page XML.
   * @param resBundleName The name of the resource bundle that will be used to resolve
   * $res() functions existing in the datasource content.
   * @param adom the ADOM object that will provide the content.
   */
  public void includeInPageXML(String dsname, String resBundleName, ADOM adom) {
    if (forcedDatasources == null) forcedDatasources = new Hashtable();
    forcedDatasources.put(dsname + '=' + resBundleName, adom);
  }

  /**
   * Returns the list of the datasource thats were forced to be included in the page XML data.
   */
  public Hashtable getForcedDatasources() {
    return this.forcedDatasources;
  }

  /**
   * Returns the original HttpServletRequest object.
   */
  public HttpServletRequest getHttpServletRequest() {
    return this.request;
  }

  /**
   * Generates (Displays) the specified page.
   *
   * @param pageName Name of the page to generate. It is usually the name of the XSL file without extension.
   * @throws PageGenerationException if generation fails.
   */
  public void generatePage(String pageName) throws PageGenerationException {

    PageGenerator pg = new PageGenerator();
    pg.generate(this,pageName);

  }

  /**
   * Generates the default error page.
   */
  public void generateDefaultErrorPage() throws PageGenerationException {
    setPageLocation("");
    setGeneratingErrorPage();
    String pageName = SystemConfig.getParameter("page-of-error","/moremotion/error/ErrorPage");
    generatePage(pageName);
  }


  /**
   * Resolves the MScript functions existing in the given string and returns it.
   * @param str the input string
   * @return resolved String
   */
  public String resolve(String str) throws MScriptException {
    if (str == null || str.length() < 4 || str.indexOf('@') == -1) {
      return str;
    }
    MScriptResolver r = new MScriptResolver(str, getSymbolResolver());
    return r.resolve();
  }


  private ParmToEncrypt[] nodesToEncrypt = null;
  private String getEncrKey(String parmName) {

    if (currpage == null || generatingErrorPage) {
      // Current Page is not yet set. Just return false.
      return null;
    }

    try {

      if (nodesToEncrypt == null ) {
        CommandDefinition cdef = currpage.getCurrentCommandDefinition();
        if (cdef != null) {
          nodesToEncrypt = cdef.getNodesToEncrypt(this);
        } else {
          nodesToEncrypt = new ParmToEncrypt[0];
        }
      }

      for (int i = 0; i < nodesToEncrypt.length; i++) {
        if (parmName.equals(nodesToEncrypt[i].name)) {
          return nodesToEncrypt[i].enckey;
        }
      }

    } catch (Exception e) {
      e.printStackTrace();
    }

    return null;
  }


  /**
   * Returns the value of the specified parameter. Note that this method cannot be used to retrieve the
   * values of the encrypted parameters.
   */
  public String getPlainParameter(java.lang.String parmName) {
    return convert(this.request.getParameter(parmName));
  }

  private String pageLocation = null;
  
  public String getPageLocation() {
    if (this.pageLocation == null) {
      this.pageLocation = this.request.getParameter("loc");
      if (this.pageLocation == null) this.pageLocation = "";
    }
    return this.pageLocation;
  }
  
  public void setPageLocation(String loc) {
    this.pageLocation = loc;
  }
  
  /**
   * Returns the value of the specified parameter. Note that this method cannot be used to retrieve the
   * values of the encrypted parameters.
   */
  public String getPlainParameter(String parmName, String defValue) {
    String result = getPlainParameter(parmName);
    if (result == null) result = defValue;
    else if (result.length() == 0) {
      if (defValue != null) result = defValue;
    } else {
      result = result.trim();
    }
    return result;
  }


  /**
   * Returns the value of the specified request parameter. It returns <CODE>null</CODE> if the parameters is not found.
   */
  public String getParameter(java.lang.String parmName) {

    String key = getEncrKey(parmName);
    if (key != null) {
      try {
        key = key + SystemConfig.requestParametersEncryptionKey();
        return SystemContext.encryptor().decrypt(this.request.getParameter(parmName),key);
      } catch (Exception e) {
        return "DECRYPT_ERROR!";
      }
    }

    return convert(this.request.getParameter(parmName));
  }

  /**
   * Returns the value of the specified request parameter. If the parameter is not found returns the default value.
   */
  public String getParameter(String parmName, String defValue) {
    String result = getParameter(parmName);
    if (result == null) result = defValue;
    else if (result.length() == 0) {
      if (defValue != null) result = defValue;
    } else {
      result = result.trim();
    }
    return result;
  }

  /**
   * Returns the value of the request parameters as <CODE>int</CODE>.
   * If the parameter is not found returns the default value.
   */
  public int getParameterAsInt(String parmName, int defValue) {
    int result = defValue;
    String resultStr = getParameter(parmName);
    try {
      result = Integer.parseInt(resultStr);
    } catch(NumberFormatException nfe) {
      // ignored
    }
    return result;
  }

  /**
   * Returns the value of the request parameters as <CODE>double</CODE>.
   * If the parameter is not found returns the default value.
   */
  public double getParameterAsDouble(String parmName, double defValue) {
    double result = defValue;
    String resultStr = getParameter(parmName);
    try {
      result = Double.parseDouble(resultStr);
    } catch(NumberFormatException nfe) {
      // ignored
    }
    return result;
  }

  /**
   * Returns the value of the request parameters as <CODE>boolean</CODE>.
   * If the parameter is not found returns the default value.
   */
  public boolean getParameterAsBoolean(String prmname, boolean defVal) {
    boolean result = defVal;
    String resultStr = getParameter(prmname);
    if (resultStr != null) {
      result = Boolean.valueOf(resultStr).booleanValue();
    }
    return result;
  }

  /**
   * Returns a String array that contains the values of the specified parameter. This method is used if the request potentially
   * contains the same parameter more than one. i.e. <CODE>MyServlet?code=1&amp;code=4&amp;code=11</CODE>
   */
  public String[] getParameterValues(java.lang.String parmName) {

    String[] values = this.request.getParameterValues(parmName);

    if (values == null) {
      return null;
    }

    String[] ra = new String[values.length];

//    if (needsEncr(parmName)) {
//      for (int i = 0;  i < values.length; i++){
//        try {
//          String key = parmName + SystemConfig.requestParametersEncryptionKey();
//          ra[i] = SystemContext.encryptor().decrypt(values[i],key);
//        } catch (Exception e) {
//          ra[i] = "DECRYPT_ERROR!";
//        }
//      }
//      return ra;
//    }


    if (!conversionRequired) {
      return values;
    }

    for (int i = 0;  i < values.length; i++){
      ra[i] = convert(values[i]);
    }
    return ra;
  }

  private String convert(String value) {

    if (!conversionRequired || value == null || value.length() == 0) return value;

    try {
      String result = new String(value.getBytes(SystemConfig.serverEncoding()), _enc);
      return result;

    } catch(UnsupportedEncodingException usee) {
      return value;
    }
  }

  /**
   * Returns the HttpServletResponse object.
   */
  public HttpServletResponse getResponse() {
    return response;
  }


  /**
   * Returns the ServletContext object
   */
  public ServletContext getContext() {
    return SystemContext.servletContext();
  }

  /**
   * Returns the Application Path of the current web application.
   */
  public String getApplicationPath() {
    String requestURI = this.getRequestURI();
    String requestURL = this.getRequestURL().toString();
    return requestURL.substring(0,requestURL.indexOf(requestURI)) + request.getContextPath();
  }

  /**
   * Returns the file object of the real file for the specified file name relative to web application path.
   * @param fileName Relative file name
   * @return <CODE>File</CODE> object of the real file.
   */
  public File getRealFile(String fileName) {

    if (fileName.startsWith("/") || fileName.startsWith("\\") ||
        (fileName.length() > 2 && fileName.charAt(1) == ':') ) {
      // Absolute File Name
      return new File(fileName) ;
    }

    return new File(getContext().getRealPath("/"), fileName);

  }


  /**
   * Creates a new protected Request ADOM and saves it in the request.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public ADOM newRequestADOM(String name) {
    return ADOMManager.newRequestADOM(this,name);
  }


  /**
   * Creates a new protected Session ADOM and saves it in the session.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public ADOM newSessionADOM(String name) {
    return ADOMManager.newSessionADOM(this.getSession(),name);
  }


  /**
   * Creates a new Application ADOM and saves it in the servlet context.
   * @param name Name of the ADOM.
   * @return Newly created ADOM.
   */
  public ADOM newApplicationADOM(String name) {
    return ADOMManager.newApplicationADOM(SystemContext.servletContext(),name);
  }


  /**
   * Returns the ADOM saved to request by the given name.
   * @param name Name of the ADOM.
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public ADOM getRequestADOM(String name) {
    return ADOMManager.getRequestADOM(this,name);
  }

  /**
   * Returns the ADOM saved to session by the given name.
   * @param name Name of the ADOM.
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public ADOM getSessionADOM(String name) {
    return ADOMManager.getSessionADOM(this.getSession(),name);
  }

  /**
   * Returns the ADOM saved to servlet context by the given name.
   * @param name Name of the ADOM.
   * @return <CODE>null</CODE> if ADOM cannot be found.
   */
  public ADOM getApplicationADOM(String name) {
    return ADOMManager.getApplicationADOM(SystemContext.servletContext(),name);
  }

  /**
   * Removes ADOM from request by given name.
   */
  public void removeRequestADOM(String name) {
    ADOMManager.removeRequestADOM(this,name);
  }

  /**
   * Removes ADOM from session by given name.
   */
  public void removeSessionADOM(String name) {
    ADOMManager.removeSessionADOM(this.getSession(),name);
  }

  /**
   * Removes ADOM from servlet context by given name.
   */
  public void removeApplicationADOM(String name) {
    ADOMManager.removeApplicationADOM(SystemContext.servletContext(),name);
  }

  /**
   * Sets the CurrentPage object to let the methods of this class to access all the necessary information
   * about the current page.
   */
  public void setCurrentPage(CurrentPage value) throws PageGenerationException {
    this.currpage = value;
    if (value != null) {
      setLanguage( currpage.getPageInfo().getPageConfig().getLanguage() );
    }
  }

  public CurrentPage getCurrentPage() {
    return this.currpage;
  }
  
  
  public void setNoCache() {

    if (getHeader("user-agent").toUpperCase().indexOf("FIREFOX") > -1) {
      response.setHeader("Cache-Control","no-store"); // HTTP 1.1
    } else {
      response.setHeader("Cache-Control","no-cache"); // HTTP 1.1
    }
    response.setHeader("Pragma","no-cache");        // HTTP 1.0
    response.setDateHeader("Expires", -1);          // Prevents caching at the proxy server
    
  }
  
  /*
   * The same as before
   */

  public java.lang.String getContentType() {
    return request.getContentType();
  }

  public boolean isSecure() {
    return request.isSecure();
  }

  public java.lang.String getServerName() {
    return request.getServerName();
  }

  public int getServerPort() {
    return request.getServerPort();
  }

  public java.util.Locale getLocale() {
    return request.getLocale();
  }

  public java.lang.Object getAttribute(java.lang.String str) {
    return request.getAttribute(str);
  }

  public java.io.BufferedReader getReader() throws java.io.IOException {
    return request.getReader();
  }

  public java.lang.String getScheme() {
    return request.getScheme();
  }

  public boolean isRequestedSessionIdFromCookie() {
    return request.isRequestedSessionIdFromCookie();
  }

  public javax.servlet.http.HttpSession getSession() {
    return request.getSession();
  }

  public javax.servlet.http.HttpSession getSession(boolean param) {
    return request.getSession(param);
  }

  public java.util.Enumeration getParameterNames() {
    return request.getParameterNames();
  }

  public java.util.Enumeration getLocales() {
    return request.getLocales();
  }

  public java.lang.String getRemoteHost() {
    return request.getRemoteHost();
  }

  public int getContentLength() {
    return request.getContentLength();
  }

  public java.lang.String getRealPath(java.lang.String str) {
    return SystemContext.servletContext().getRealPath(str);
  }

  public javax.servlet.ServletInputStream getInputStream() throws java.io.IOException {
    return request.getInputStream();
  }

  public java.lang.String getPathTranslated() {
    return request.getPathTranslated();
  }

  public java.lang.String getServletPath() {
    return request.getServletPath();
  }

  public javax.servlet.RequestDispatcher getRequestDispatcher(java.lang.String str) {
    return request.getRequestDispatcher(str);
  }

  public void removeAttribute(java.lang.String str) {
    request.removeAttribute(str);
  }

  public int getIntHeader(java.lang.String str) {
    return request.getIntHeader(str);
  }

  public java.util.Enumeration getHeaderNames() {
    return request.getHeaderNames();
  }

  public java.lang.String getCharacterEncoding() {
    return request.getCharacterEncoding();
  }

  public java.lang.String getPathInfo() {
    return request.getPathInfo();
  }

  public javax.servlet.http.Cookie[] getCookies() {
    return request.getCookies();
  }

  public java.lang.String getRemoteAddr() {
    return request.getRemoteAddr();
  }

  public java.lang.String getQueryString() {
    return request.getQueryString();
  }

  public java.lang.String getAuthType() {
    return request.getAuthType();
  }

  public boolean isUserInRole(java.lang.String str) {
    return request.isUserInRole(str);
  }

  public java.lang.String getMethod() {
    return request.getMethod();
  }

  public java.lang.String getHeader(java.lang.String str) {
    return request.getHeader(str);
  }

  public java.lang.String getRequestURI() {
    return request.getRequestURI();
  }

  public void setAttribute(java.lang.String str, java.lang.Object obj) {
    request.setAttribute(str, obj);
  }

  public boolean isRequestedSessionIdFromUrl() {
    return request.isRequestedSessionIdFromURL();
  }

  public boolean isRequestedSessionIdFromURL() {
    return request.isRequestedSessionIdFromURL();
  }

  public java.security.Principal getUserPrincipal() {
    return request.getUserPrincipal();
  }

  public java.util.Enumeration getAttributeNames() {
    return request.getAttributeNames();
  }

  public java.lang.String getRequestedSessionId() {
    return request.getRequestedSessionId();
  }

  public long getDateHeader(java.lang.String str) {
    return request.getDateHeader(str);
  }

  public java.lang.String getProtocol() {
    return request.getProtocol();
  }

  public java.lang.String getRemoteUser() {
    return request.getRemoteUser();
  }

  public boolean isRequestedSessionIdValid() {
    return request.isRequestedSessionIdValid();
  }

  public java.lang.String getContextPath() {
    return request.getContextPath();
  }

  public java.util.Enumeration getHeaders(java.lang.String str) {
    return request.getHeaders(str);
  }

  public void setCharacterEncoding(java.lang.String str) throws java.io.UnsupportedEncodingException {
    request.setCharacterEncoding(str);
  }

  public Map getParameterMap() {
    return request.getParameterMap();
  }

  public StringBuffer getRequestURL() {
    return request.getRequestURL();
  }

  public String getLocalAddr() {
    return "";
  }

  public String getLocalName() {
    return "";
  }

  public int getLocalPort() {
    return 0;
  }

  public int getRemotePort() {
    return 0;
  }

}

