/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ItemParser.java
 *
 * Created on November 14, 2003, 5:38 PM
 */

package org.moremotion.util;

/**
 * String Parser parses the items in a string with the specified delimiter. 
 * The difference of this class from the
 * java.util.StringTokenizer is that this class ignores the delimiters enclosed with (), [], {} , '', "" characters.
 * <P><B>Example:</B><PRE>
 * StringParser sp = new StringParser("(xxx,yyy)',',[(qqq,www),(ppp,rrr)],zzz",',');
 * sp.nextItem(); // returns "(xxx,yyy)','"
 * sp.nextItem(); // returns "[(qqq,www),(ppp,rrr)]
 * sp.nextItem(); // returns "zzz"
 * </PRE>
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: StringParser.java 51 2008-03-10 11:46:54Z erkan $
 */
public class StringParser {
  
  
  private int csr = 0;
  private String str = "";
  private char dlm = ' ';
  private char[] bracebuffer;
  private int icount = -1;
  private boolean trim = false;
  
  
  /**
   * Creates a new StringParser
   */
  public StringParser(String str, char dlm) {
    this.str = str;
    this.dlm = dlm;
    bracebuffer = new char[20];
    csr = 0;
  }
  
  /**
   * Returns <CODE>true</CODE> if end of string is reached.
   */
  public boolean eos() {
    return csr >= str.length();
  }
  
  public boolean hasMore() {
    return csr < str.length();
  }
  
  public void trimmed() {
    if (this.dlm != ' ') {
      this.trim = true;
    }
  }
  
  /**
   * Returns the first item in the string.
   */
  public String firstItem() {
    csr = 0;
    return nextItem();
  }
  
  /**
   * Returns the next item in the string.
   */
  public String nextItem() {
    boolean nsfound = dlm == ' ' ? false : true; 
    if (eos()) return "";
    int braceindex = -1;
    char quote = ' ';
    int i = csr;
    for (; i < str.length(); i++) {
      char cc = str.charAt(i);
      char ch = ' ';
      if (cc == dlm && nsfound && quote == ' ' && braceindex == -1) break;
      if (!nsfound && cc != ' ') nsfound = true;
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
      }
      if (ch != ' ') {
        bracebuffer[++braceindex] = ch;
        ch = ' ';
      }
      
    }
    String result;
    if (this.trim /* && (str.charAt(csr) == ' ' || (i > 0 && str.charAt(i-1) == ' ')) */ ) {
      while (csr < str.length() && str.charAt(csr) <= ' ') csr++;
      int j;
      for (j = i; (j > 0 && str.charAt(j-1) == ' '); j--);
      result = str.substring(csr,j);
    } else {
      result = str.substring(csr,i);
    }
    csr = i + 1;
    
    return result;
  }
  
  /**
   * Returns the next item in the string.
   */
  public int itemCount() {
    if (icount != -1) {
      return icount;
    }
    int bi = -1;
    char[] bb = new char[20];
    char quote = ' ';
    for (int i = 0; i < str.length(); i++) {
      char cc = str.charAt(i);
      char ch = ' ';
      if (cc == dlm && quote == ' ' && bi == -1) {
        icount++;
        continue;
      }
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        else if (bi > -1 && cc == bb[bi]) bi--;
      }
      if (ch != ' ') {
        bb[++bi] = ch;
        ch = ' ';
      }
      
    }
    icount++;
    return icount;
  }
  
  /**
   * Moves the cursor to the first position.
   */
  public void reset() {
    csr = 0;
  }
  
  /**
   * Returns the next item as <CODE>int</CODE> and if next item is not a number it returns the default value.
   */
  public int nextItemAsInt(int defValue) {
    try {
      return Integer.parseInt(this.nextItem());
    } catch(Exception e){
      return defValue;
    }
  }
  
  public String nextEnclosed() {
    String str = this.nextItem();
    if (str.length() > 2) {
      str = str.substring(1,str.length()-1);
    }
    return str;
  }
  
  
  /**
   * Returns the next item as <CODE>boolean</CODE>. If the next item is not "true" returns <CODE>false</CODE>.
   */
  public boolean nextItemAsBoolean() {
    return this.nextItem().equalsIgnoreCase("true") ? true : false;
  }
  
  /**
   * This static method returns the position of the specified character in the given string.
   * The chars enclosed with (), [], {} , '', "" characters are ignored.
   * @param str The string where <CODE>c</CODE> will be searched.
   * @param startPos Search Start Position
   * @param c The character to be searched.
   * @return The position of the <CODE>c</CODE> in the <CODE>str</CODE>. If <CODE>c</CODE> is not found -1 is returned.
   */
  public static int locateChar(String str, int startPos, char c) {
    int i = startPos;
    if (i > str.length()) return -1;
    char[] bracebuffer = new char[20];
    int braceindex = -1;
    char quote = ' ';
    for (; i < str.length(); i++) {
      char cc = str.charAt(i);
      char ch = ' ';
      if (cc == c && quote == ' ' && braceindex == -1) return i;
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        //        ch = ' ';
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        // else if (cc == '<') ch = '>';
        else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
      }
      if (ch != ' ') {
        bracebuffer[++braceindex] = ch;
        ch = ' ';
      }
      
    }
    return -1;
  }
  
  /**
   * This static method returns the position of the specified character in the given string buffer.
   * The chars enclosed with (), [], {} , '', "" characters are ignored.
   * @param buffer The string buffer where <CODE>c</CODE> will be searched.
   * @param startPos Search Start Position
   * @param c The character to be searched.
   * @return The position of the <CODE>c</CODE> in the <CODE>str</CODE>. If <CODE>c</CODE> is not found -1 is returned.
   */
  public static int locateChar(StringBuffer buffer, int startPos, char c) {
    int i = startPos;
    if (i > buffer.length()) return -1;
    char[] bracebuffer = new char[20];
    int braceindex = -1;
    char quote = ' ';
    for (; i < buffer.length(); i++) {
      char cc = buffer.charAt(i);
      char ch = ' ';
      if (cc == c && quote == ' ' && braceindex == -1) return i;
      if (quote == ' ') {
        if (cc == '\'' || cc == '"') quote = cc;
      }
      else if (quote == cc) quote = ' ';
      if (quote == ' ') {
        //        ch = ' ';
        if (cc == '(') ch = ')';
        else if (cc == '[') ch = ']';
        else if (cc == '{') ch = '}';
        // else if (cc == '<') ch = '>';
        else if (braceindex > -1 && cc == bracebuffer[braceindex]) braceindex--;
      }
      if (ch != ' ') {
        bracebuffer[++braceindex] = ch;
        ch = ' ';
      }
      
    }
    return -1;
  }
  
  /**
   * This static method extracts the values of the parameters
   * existing in a string in <CODE> parm1=parm1value; parm2=parm2 value; </CODE>
   * format.
   * @param str The string where <CODE>param</CODE> will be searched.
   * @param param The name of the parameter whose value to be extracted.
   * @return The value of the parameter
   */
  public static String extractParameter(String str, String param) {
    if (str == null || str.length() == 0) return "";
    if (param == null || param.length() == 0) return "";
    String prm = new String(param + "=");
    int pos = str.indexOf(param);
    if (pos == -1) return "";
    pos = pos + prm.length();
    int endpos = locateChar(str, pos , ';');
    if (endpos == -1) endpos = str.length();
    return str.substring(pos,endpos).trim();
  }
  
  public static void main(String[] args) throws Exception {
    
    long t1 = System.currentTimeMillis();
    
    for (int j = 0; j < 999999; j++) {
      StringParser p = new StringParser("aa ;bb ;cc ;dd ;ee ;ff ",';');
      p.trimmed();
      while (!p.eos()) {
        p.nextItem();
        //p.nextItem().trim();
        
        // System.out.println("'" + p.nextItem() + "'");
      }
    }
    long t2 = System.currentTimeMillis();
    
    System.out.println( (t2-t1) + "");
    
    //    System.out.println(p.itemCount() + "");
  }
  
}
