/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SecurityDomain.java
 *
 * Created on 15 September 2004 Wednesday, 14:24
 */

package org.moremotion.security.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SecurityDomain.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SecurityDomain {
  
  private String name = null;
  private Hashtable sessions = null;
  private Hashtable subDomains = null;
  private FailedLoginAttempts flas = null;
  private Hashtable killedSessions = null;
  
  /**
   * Creates a new instance of SecurityDomain
   */
  public SecurityDomain() {
  }
  
  public SecurityDomain(String name) {
    this.name = name;
    sessions = new Hashtable();
    subDomains = new Hashtable();
    killedSessions = new Hashtable();
    flas = new FailedLoginAttempts();
  }
  
  /**
   * Returns the name of this Security Domain
   */
  public String getName() {
    return this.name;
  }
  
  /**
   * Returns a Hashtable that keeps the users which are signed in to the given subdomain
   */
  private Hashtable getSubDomainUsers(String subDomainName) {
    Hashtable users = (Hashtable)subDomains.get(subDomainName);
    if (users == null) {
      synchronized (subDomains) {
        users = new Hashtable();
        subDomains.put(subDomainName, users);
      }
    }
    return users;
  }
  
  /**
   * Logges in the specified user in this domain. 
   * @param subDomainName The name of the subdomain
   * @param userName the user name
   * @param sid The current Http Session Id
   */
  public void login(String subDomainName, String userName, String sid) throws SecurityDomainException {
    login(subDomainName, userName, sid, true);
  }
  
  /**
   * Logges in the specified user in this domain.
   * @param subDomainName The name of the subdomain
   * @param userName the user name
   * @param sid The current Http Session Id
   * @param multipleLoginAllowed flag
   */
  public void login(String subDomainName, String userName, String sid, boolean multipleLoginAllowed) throws SecurityDomainException {
    
    SignedUser su = (SignedUser)sessions.get(sid);
    if (su != null) {
      if (!multipleLoginAllowed) {
        su.killOthers(sid,this.sessions,this.killedSessions);
      } else {
        su.touch(sid);
      }
      return;
    }
    
    Hashtable users = getSubDomainUsers(subDomainName);
    su = (SignedUser)users.get(userName);
    if (su == null) {
      su = new SignedUser(userName, sid, users);
      users.put(userName,su);
      sessions.put(sid, su);
      su.signIn(sid);
    } else {
      su.signIn(sid);
      sessions.put(sid,su);
      if (!multipleLoginAllowed) {
        su.killOthers(sid,this.sessions,this.killedSessions);
      }
    }
    flas.remove(subDomainName, userName);
  }
  
  /**
   * Checks if the given session is just killed. This function removes the given session id from the
   * killedSessions Hashtable before returning. 
   */
  public boolean isKilled(String sid) {
    boolean killed = killedSessions.get(sid) != null;
    if (killed) killedSessions.remove(sid);
    return killed;
  }
  
  public int getNumberOfFailedLoginAttemps(String subDomainName, String userName, String sid) {
    return 0;
  }
  
  public void reportFailedLoginAttempt(String subDomainName, String userName, String sid) {
  }
  
  
  private String getSubDomainName(Hashtable subDomainUsers) {
    Enumeration en = subDomains.keys();
    while (en.hasMoreElements()) {
      String subDomainName = (String)en.nextElement();
      if (subDomainUsers == subDomains.get(subDomainName)) {
        return subDomainName;
      }
    }
    return "";
  }
  
  /**
   * Logges in the user without authentication incase it has a valid session in another domain
   * @param oldsid The session id from another context
   * @param newsid The current session in the current context
   */
  public String trustedLogin(String oldsid, String newsid) throws SecurityDomainException {
    
    SignedUser su = (SignedUser)sessions.get(newsid);
    if (su != null) {
      return getSubDomainName(su.ownerUserList) + "~" + su.userName;
    }
    su = (SignedUser)sessions.get(oldsid);
    
    if (su == null) {
      return null;
    }
    su.signIn(newsid);
    
    sessions.put(newsid,su);
    
    return getSubDomainName(su.ownerUserList) + "~" + su.userName;
  }
  
  /**
   * Logges out the user attached to the given Http Session
   */
  public void logout(String sid) {
    SignedUser su = (SignedUser)sessions.get(sid);
    if (su != null){
      logoutUser(su.ownerUserList, su.userName, sid);
    }
    
  }
  
  /**
   * Logges out all the sessions of the specified user from this domain
   * @param subDomainName The name of the subdomain
   * @param userName the user name
   */
  public void logoutUser(String subDomainName, String userName) {
    
    Hashtable users = getSubDomainUsers(subDomainName);
    logoutUser(users, userName, null);
    
  }
  
  private void logoutUser(Hashtable users, String userName, String sid) {
    
    SignedUser su = (SignedUser)users.get(userName);
    if (su != null) {
      if (sid == null) {
        su.killOthers(null,this.sessions,this.killedSessions);
        users.remove(userName);
      } else {
        su.signOut(sid);
        if (su.getSessionCount() == 0) {
          users.remove(userName);
        }
        sessions.remove(sid);
      }
    }
  }
  
  /**
   * Returns the Signed User associated with the given Http Session
   */
  public SignedUser getSignedUser(String sid) {
    SignedUser su = (SignedUser)sessions.get(sid);
    if (su != null) {
      su.touch(sid);
    }
    return su;
  }
  
  /**
   * Used to denote that the specified user account is modified
   */
  public void userModified(String subDomainName, String userName) {
    Hashtable users = getSubDomainUsers(subDomainName);
    
    SignedUser su = (SignedUser)users.get(userName);
    if (su != null) {
      su.lmodified = (new Date()).getTime();
    }
  }
  
  public void allUsersModified(String subDomainName) {
    Hashtable users = getSubDomainUsers(subDomainName);
    
    long md = (new Date()).getTime();
    Enumeration en = users.elements();
    while (en.hasMoreElements()) {
      SignedUser su = (SignedUser)en.nextElement();
      su.lmodified = md;
    }
  }
  
  private int getUserCount(Hashtable users) {
    int count = 0;
    Iterator itr = users.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      SignedUser su = (SignedUser)entry.getValue();
      count += su.getSessionCount() > 0 ? 1 : 0;
    }
    return count;
  }
  
  public int getActiveUserCount(String subDomainName) {
    if (subDomainName == null || subDomainName.trim().length() == 0) {
      return getActiveUserCount_();
    }
    return getUserCount(getSubDomainUsers(subDomainName));
  }
  
  private int getActiveUserCount_() {
    
    int count = 0;
    Iterator itr = subDomains.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      Hashtable users = (Hashtable)entry.getValue();
      
      count += getUserCount(users);
    }
    
    return count;
  }
  
  public ArrayList getActiveUserList(String subDomainName, int from, int amount) {
    if (subDomainName == null || subDomainName.trim().length() == 0) {
      return getActiveUserList_(from,amount);
    }
    
    Hashtable users = getSubDomainUsers(subDomainName);
    int i = 1;
    int c = 0;
    ArrayList a = new ArrayList();
    Iterator itr = users.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      SignedUser su = (SignedUser)entry.getValue();
      if (su.getSessionCount() > 0) {
        
        if (i == from) {
          c++;
          if (c > amount) break;
          a.add(su);
        } else {
          i++;
        }
      }
    }
    
    return a;
  }
  
  private ArrayList getActiveUserList_(int from, int amount) {
    
    int i = 1;
    int c = 0;
    ArrayList a = new ArrayList();
    
    Enumeration enu = subDomains.keys();
    while (enu.hasMoreElements()) {
      String subDomainName = (String)enu.nextElement();
      Hashtable users = getSubDomainUsers(subDomainName);
      Iterator itr = users.entrySet().iterator();
      while (itr.hasNext()) {
        Map.Entry entry = (Map.Entry)itr.next();
        SignedUser su = (SignedUser)entry.getValue();
        if (su.getSessionCount() > 0) {
          if (i == from) {
            c++;
            if (c > amount) break;
            a.add(su);
            // a.add(subDomainName + "/" + su.userName);
          } else {
            i++;
          }
        }
      }
    }
    
    return a;
  }
  
  public int failedLoginAttempt(String subDomainName, String userName, long suspendPeriod) {
    return flas.failedLoginAttempt(subDomainName, userName, suspendPeriod);
  }
  
  public int isUserSuspended(String subDomainName, String userName, long suspendPeriod) {
    return flas.flaTries(subDomainName, userName, suspendPeriod);
  }
  
  
}
